@@ -1,6 +1,6 @@
 /*
  * NVIDIA CUDA Debugger CUDA-GDB
- * Copyright (C) 2007-2024 NVIDIA Corporation
+ * Copyright (C) 2007-2025 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -160,37 +160,42 @@ void
 cuda_debugapi::cuda_api_error(CUDBGResult res, const char *fmt, ...)
 {
   va_list args;
-  char errStr[cuda_debugapi::ErrorStringMaxLength] = {0};
-  char errStrEx[cuda_debugapi::ErrorStringExMaxLength] = {0};
+  std::array<char, cuda_debugapi::ErrorStringExMaxLength> errStr;
 
   va_start (args, fmt);
-  vsnprintf (errStr, sizeof(errStr), fmt, args);
+  vsnprintf (errStr.data (), errStr.size (), fmt, args);
   va_end (args);
 
-  get_error_string_ex (errStrEx, sizeof(errStrEx), nullptr);
+  std::array<char, cuda_debugapi::ErrorStringExMaxLength> errStrEx;
+  get_error_string_ex (errStrEx.data (), errStrEx.size (), nullptr);
   throw_error (GENERIC_ERROR, "Error: %s, error=%s, error message=%s\n",
-               errStr, cudbgGetErrorString(res), errStrEx);
+	       errStr.data (), cudbgGetErrorString (res), errStrEx.data ());
 }
 
 void
 cuda_debugapi::cuda_api_print_api_call_result (const char *function, CUDBGResult res)
 {
-  if (res != CUDBG_SUCCESS)
+  // CUDBG_ERROR_NO_EVENT_AVAILABLE "errors" are expected, don't log them
+  // and don't try to fetch extended error information in these cases
+  // either.
+  if ((res != CUDBG_SUCCESS) && (res != CUDBG_ERROR_NO_EVENT_AVAILABLE))
     {
-      char errStrEx[cuda_debugapi::ErrorStringExMaxLength] = {0};
+      std::array<char, cuda_debugapi::ErrorStringExMaxLength> errStrEx;
+      get_error_string_ex (errStrEx.data (), errStrEx.size (), nullptr);
 
-      get_error_string_ex (errStrEx, sizeof(errStrEx), nullptr);
-      cuda_api_trace ("%s API call received result: %s, error message=%s",
-		      function, cudbgGetErrorString((CUDBGResult) res), errStrEx);
+      cuda_api_trace ("%s Debugger API call error result: %s, error message=%s",
+		      function, cudbgGetErrorString ((CUDBGResult)res),
+		      errStrEx.data ());
     }
 }
 
-const char*
-cuda_api_mask_string(const cuda_api_warpmask* m)
+const char *
+cuda_api_mask_string (const cuda_api_warpmask *m)
 {
-    static char buf[sizeof(m->mask)*2 + 3] = {0};
-    sprintf(buf, "0x%.*llx", (int)sizeof(buf) - 3, (unsigned long long)(m->mask));
-    return buf;
+  constexpr size_t num_chars = sizeof (m->mask) * 2;
+  static char buf[num_chars + 3] = { 0 };
+  sprintf (buf, "0x%.*llx", (int)num_chars, (unsigned long long)(m->mask));
+  return buf;
 }
 
 int
@@ -578,35 +583,6 @@ cuda_debugapi::read_generic_memory (uint32_t dev, uint32_t sm, uint32_t wp, uint
   return true;
 }
 
-bool
-cuda_debugapi::read_pinned_memory (uint64_t addr, void *buf, uint32_t sz)
-{
-  if (!api_state_initialized ())
-    return false;
-
-  CUDA_API_PROFILE (readPinnedMemory);
-
-  CUDBGResult res = s_instance.m_cudbgAPI->readPinnedMemory (addr, buf, sz);
-  cuda_api_print_api_call_result (__FUNCTION__, res);
-
-  if (res == CUDBG_ERROR_MEMORY_MAPPING_FAILED)
-    {
-      CUDA_API_TRACE ("memory mapping failed: address 0x%lx len %u", addr, sz);
-      return false;
-    }
-
-  if (res != CUDBG_SUCCESS)
-    CUDA_API_ERROR (res, "failed to read pinned memory at address 0x%lx size %u", addr, sz);
-
-  if ((sz == 4) || (sz == 8))
-    CUDA_API_TRACE ("address 0x%lx len %u = 0x%lx", addr, sz,
-		    (sz == 4) ? (uint64_t)*(uint32_t *)buf : (uint64_t)*(uint64_t *)buf);
-  else
-    CUDA_API_TRACE ("address 0x%lx len %u", addr, sz);
-
-  return true;
-}
-
 void
 cuda_debugapi::read_param_memory (uint32_t dev, uint32_t sm, uint32_t wp, uint64_t addr, void *buf, uint32_t sz)
 {
@@ -666,17 +642,18 @@ cuda_debugapi::read_local_memory (uint32_t dev, uint32_t sm, uint32_t wp, uint32
 
   if (!target_has_execution () && (res == CUDBG_ERROR_MISSING_DATA))
     CUDA_API_ERROR_DEV_SM_WARP_LANE (res, dev, sm, wp, ln,
-				     "local memory is not available in this corefile");
+				     "local memory address 0x%lx len %u is not available in this corefile",
+				     addr, sz);
 
   if (res != CUDBG_SUCCESS)
     {
-      CUDA_API_ERROR_DEV_SM_WARP_LANE (res, dev, sm, wp, ln,
-				       "failed to read local memory at address 0x%lx size %u",
-				       addr, sz);
+      CUDA_API_TRACE_DEV_SM_WARP_LANE (dev, sm, wp, ln,
+				       "failed to read local memory at address 0x%lx size %u result %s",
+				       addr, sz, cudbgGetErrorString(res));
       return false;
     }
 
-  CUDA_API_TRACE_DEV_SM_WARP_LANE (dev, sm, wp, ln, "0x%lx (%u)", addr, sz);
+  CUDA_API_TRACE_DEV_SM_WARP_LANE (dev, sm, wp, ln, "0x%lx size %u", addr, sz);
 
   return true;
 }
@@ -928,25 +905,6 @@ cuda_debugapi::write_generic_memory (uint32_t dev, uint32_t sm, uint32_t wp, uin
   return true;
 }
 
-bool
-cuda_debugapi::write_pinned_memory (uint64_t addr, const void *buf, uint32_t sz)
-{
-  if (!api_state_initialized ())
-    return false;
-
-  CUDA_API_PROFILE (writePinnedMemory);
-
-  CUDBGResult res = s_instance.m_cudbgAPI->writePinnedMemory (addr, buf, sz);
-  cuda_api_print_api_call_result (__FUNCTION__, res);
-
-  if (res != CUDBG_SUCCESS && res != CUDBG_ERROR_MEMORY_MAPPING_FAILED)
-    CUDA_API_ERROR (res, "failed to write pinned memory at address 0x%lx size %u", addr, sz);
-
-  CUDA_API_TRACE ("0x%lx (%u)", addr, sz);
-
-  return res == CUDBG_SUCCESS;
-}
-
 void
 cuda_debugapi::write_param_memory (uint32_t dev, uint32_t sm, uint32_t wp, uint64_t addr, const void *buf, uint32_t sz)
 {
@@ -997,9 +955,9 @@ cuda_debugapi::write_local_memory (uint32_t dev, uint32_t sm, uint32_t wp, uint3
 
   if (res != CUDBG_SUCCESS)
     {
-      CUDA_API_ERROR_DEV_SM_WARP_LANE (res, dev, sm, wp, ln,
-				       "failed to write local memory at address 0x%lx size %u",
-				       addr, sz);
+      CUDA_API_TRACE_DEV_SM_WARP_LANE (dev, sm, wp, ln,
+				       "failed to write local memory at address 0x%lx size %u result %s",
+				       addr, sz, cudbgGetErrorString(res));
       return false;
     }
 
@@ -1418,27 +1376,6 @@ cuda_debugapi::get_num_upredicates (uint32_t dev, uint32_t *numPredicates)
  CUDA_API_TRACE_DEV (dev, "%u", *numPredicates);
 }
 
-void
-cuda_debugapi::is_device_code_address (uint64_t addr, bool *is_device_address)
-{
-  if (!api_state_initialized ())
-    {
-      *is_device_address = false;
-      return;
-    }
-
-  CUDA_API_PROFILE (isDeviceCodeAddress);
-
-  CUDBGResult res = s_instance.m_cudbgAPI->isDeviceCodeAddress (addr, is_device_address);
-  cuda_api_print_api_call_result (__FUNCTION__, res);
- 
- if (res != CUDBG_SUCCESS)
-   CUDA_API_ERROR (res, "failed to determine if address 0x%lx corresponds "
-		   "to the host or device", addr);
-
- CUDA_API_TRACE ("0x%lx is-code %d", addr, *is_device_address); 
-}
-
 void
 cuda_debugapi::handle_set_callback_api_error (CUDBGResult res)
 {
@@ -1659,8 +1596,13 @@ cuda_debugapi::read_warp_state (uint32_t dev, uint32_t sm, uint32_t wp, CUDBGWar
   CUDA_API_PROFILE (readWarpState);
 
   CUDBGResult res;
-  if (api_version ().m_revision >= 148)
+  if (api_version ().m_revision >= 154)
     res = s_instance.m_cudbgAPI->readWarpState (dev, sm, wp, state);
+  else if (api_version ().m_revision >= 148)
+    {
+      res = s_instance.m_cudbgAPI->readWarpState127 (dev, sm, wp, reinterpret_cast<CUDBGWarpState127*>(state));
+      memset (reinterpret_cast<char *>(state) + sizeof(CUDBGWarpState127), 0, sizeof(*state) - sizeof(CUDBGWarpState127));
+    }
   else
     {
       res = s_instance.m_cudbgAPI->readWarpState120 (dev, sm, wp, reinterpret_cast<CUDBGWarpState120*>(state));
@@ -2353,66 +2295,57 @@ cuda_debugapi::read_warp_resources (uint32_t dev, uint32_t sm, uint32_t wp, CUDB
 }
 
 void
-cuda_api_clear_mask(cuda_api_warpmask* mask)
+cuda_debugapi::get_cbu_warp_state (uint32_t dev, uint32_t sm,
+				   const cuda_api_warpmask &wp_mask,
+				   CUDBGCbuWarpState *states,
+				   uint32_t numStates)
 {
-  memset ((void *)mask, 0, sizeof(*mask));
+  gdb_assert (states);
+  gdb_assert (numStates);
+
+  if (!api_state_initialized ())
+    return;
+
+  CUDA_API_PROFILE (getCbuWarpState);
+
+  memset (states, 0, sizeof (*states) * numStates);
+
+  CUDBGResult res = CUDBG_ERROR_NOT_SUPPORTED;
+  if (api_version ().m_revision >= 153)
+    res = s_instance.m_cudbgAPI->getCbuWarpState (dev, sm, wp_mask.mask,
+						  states, numStates);
+  cuda_api_print_api_call_result (__FUNCTION__, res);
+
+  if ((res != CUDBG_SUCCESS) && (res != CUDBG_ERROR_NOT_SUPPORTED))
+    CUDA_API_ERROR_DEV_SM (res, dev, sm, "failed to get CBU warp state");
+
+  CUDA_API_TRACE_DEV_SM (dev, sm,
+			 "wp_mask %" WARP_MASK_FORMAT " requested states %u",
+			 cuda_api_mask_string (&wp_mask), numStates);
 }
 
 void
-cuda_api_set_bit(cuda_api_warpmask* m, int i, int v)
+cuda_debugapi::consume_cuda_logs (CUDBGCudaLogMessage *logMessages, uint32_t numMessages,
+		   uint32_t *numConsumed)
 {
-  if (v)
-    m->mask |= (1ULL << (i%64));
-  else
-    m->mask &= ~(1ULL << (i%64));
-}
+  gdb_assert (logMessages);
+  gdb_assert (numConsumed);
 
-int
-cuda_api_get_bit(const cuda_api_warpmask* m, int i)
-{
-  return (m->mask & (1ULL << (i%64))) != 0;
-}
+  if (!api_state_initialized ())
+    return;
 
-int
-cuda_api_has_bit(const cuda_api_warpmask* m)
-{
-  return !!(m->mask);
-}
+  CUDA_API_PROFILE (consumeCudaLogs);
 
-int
-cuda_api_has_multiple_bits(const cuda_api_warpmask* m)
-{
-  return !!(m->mask & (m->mask - 1));
-}
+  memset (logMessages, 0, sizeof (*logMessages) * numMessages);
+  *numConsumed = 0;
 
-int
-cuda_api_eq_mask(const cuda_api_warpmask* m1, const cuda_api_warpmask* m2)
-{
-  return memcmp(m1, m2, sizeof(*m1)) == 0;
-}
+  CUDBGResult res = CUDBG_ERROR_NOT_SUPPORTED;
+  if (api_version ().m_revision >= 155)
+    res = s_instance.m_cudbgAPI->consumeCudaLogs (logMessages, numMessages,
+						  numConsumed);
 
-void
-cuda_api_cp_mask(cuda_api_warpmask* dst, const cuda_api_warpmask* src)
-{
-  memcpy(dst, src, sizeof(*dst));
-}
-
-cuda_api_warpmask*
-cuda_api_or_mask(cuda_api_warpmask* dst, const cuda_api_warpmask* m1, const cuda_api_warpmask* m2)
-{
-  dst->mask = m1->mask | m2->mask;
-  return dst;
-}
-cuda_api_warpmask*
-cuda_api_and_mask(cuda_api_warpmask* dst, const cuda_api_warpmask* m1, const cuda_api_warpmask* m2)
-{
-  dst->mask = m1->mask & m2->mask;
-  return dst;
-}
-
-cuda_api_warpmask*
-cuda_api_not_mask(cuda_api_warpmask* dst, const cuda_api_warpmask* m1)
-{
-  dst->mask = ~m1->mask;
-  return dst;
+  if ((res != CUDBG_SUCCESS) && (res != CUDBG_ERROR_NOT_SUPPORTED)
+      && (res != CUDBG_ERROR_NO_EVENT_AVAILABLE))
+    throw_error (GENERIC_ERROR, "Error: consumeCudaLogs, error=%s.\n",
+		 cudbgGetErrorString (res));
 }
