@@ -1,6 +1,6 @@
 /*
  * NVIDIA CUDA Debugger CUDA-GDB
- * Copyright (C) 2007-2024 NVIDIA Corporation
+ * Copyright (C) 2007-2025 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -38,6 +38,7 @@
 #include "cuda-tdep.h"
 #include "cuda-util-stream.h"
 
+#include <fcntl.h>
 #include <signal.h>
 #include <spawn.h>
 #if !defined(__QNX__)
@@ -200,12 +201,11 @@ cuda_instruction::eval_is_control_flow (const bool skip_subroutines)
 }
 
 template <typename M, typename K, typename V>
-static void inline map_insert_or_assign (M &map, const K &key, const V &value)
+static void inline map_insert_or_assign (M &map, const K &key, V &&value)
 {
   auto it = map.find (key);
   if (it != map.end ())
     {
-      // Assignment operator is not available because of const members.
       it->second = value;
     }
   else
@@ -218,10 +218,17 @@ static void inline map_insert_or_assign (M &map, const K &key, const V &value)
 gdb::optional<cuda_instruction>
 cuda_module_disassembly_cache::disassemble_instruction (uint64_t pc)
 {
-  auto from_elf = disassemble_instruction (pc, disassembly_source::ELF);
-  if (from_elf)
-    return from_elf;
-  return disassemble_instruction (pc, disassembly_source::DEVICE);
+  /* Check the setting of disassemble_from */
+  auto source = cuda_options_disassemble_from_elf_image () ? disassembly_source::ELF
+							   : disassembly_source::DEVICE;
+  /* Disassemble the instruction */
+  auto insn = disassemble_instruction (pc, source);
+
+  /* Try to fallback to debug API if we failed to disassemble from ELF */
+  if (!insn && source == disassembly_source::ELF)
+    insn = disassemble_instruction (pc, disassembly_source::DEVICE);
+
+  return insn;
 }
 
 gdb::optional<cuda_instruction>
@@ -280,14 +287,14 @@ cuda_module_disassembly_cache::add_function_to_cache (
     case disassembly_source::ELF:
       for (const cuda_instruction &insn : function.instructions ())
 	{
-	  map_insert_or_assign (m_elf_map, pc, insn);
+	  map_insert_or_assign (m_elf_map, pc, cuda_instruction (insn));
 	  pc += m_insn_size;
 	}
       break;
     case disassembly_source::DEVICE:
       for (const cuda_instruction &insn : function.instructions ())
 	{
-	  map_insert_or_assign (m_device_map, pc, insn);
+	  map_insert_or_assign (m_device_map, pc, cuda_instruction (insn));
 	  pc += m_insn_size;
 	}
       break;
@@ -300,9 +307,44 @@ class cuobjdump_process
 {
   pid_t m_pid;
   int m_stdout_fd;
+  int m_stderr_fd;
+
+  /* Set up a spawn process to replace the target fd with a supplied pipe.
+     Returns the read-end of that pipe. */
+  int set_up_fd_redirect (posix_spawn_file_actions_t *file_actions, int pipe[2], int target_fd)
+  {
+    int rfd = pipe[0];
+    int wfd = pipe[1];
+
+    /* Close read-end in child process */
+    int ret = posix_spawn_file_actions_addclose (file_actions, rfd);
+    CUDA_ERR_IF (ret != 0, CUDA_TRACE_DISASSEMBLER,
+                 "cuobjdump_process: add close failed: %s",
+                 safe_strerror (ret));
+
+    /* Redirect target fd to pipe write-end in child process.
+       Original target fd will be closed by dup2. */
+    ret = posix_spawn_file_actions_adddup2 (file_actions, wfd, target_fd);
+    CUDA_ERR_IF (ret != 0, CUDA_TRACE_DISASSEMBLER,
+                 "cuobjdump_process: add dup2 failed: %s",
+                 safe_strerror (ret));
+
+    /* Close write-end in child process, since it has been dup2'd
+       into the target fd we can close the duplicate one. */
+    ret = posix_spawn_file_actions_addclose (file_actions, wfd);
+    CUDA_ERR_IF (ret != 0, CUDA_TRACE_DISASSEMBLER,
+                 "cuobjdump_process: add close failed: %s",
+                 safe_strerror (ret));
+
+    return rfd;
+  }
 
 public:
-  cuobjdump_process () : m_pid (-1), m_stdout_fd (-1) {}
+  cuobjdump_process ()
+    : m_pid (-1),
+      m_stdout_fd (-1),
+      m_stderr_fd (-1)
+  {}
 
   DISABLE_COPY_AND_ASSIGN (cuobjdump_process);
 
@@ -312,6 +354,38 @@ public:
     return m_stdout_fd;
   }
 
+  int
+  stderr_fd () const
+  {
+    return m_stderr_fd;
+  }
+
+  std::string get_stderr ()
+  {
+    const int STDERR_BUFSZ = 512;
+    std::string str;
+    ssize_t nr;
+
+    gdb::unique_xmalloc_ptr<char> buff((char *)xmalloc (STDERR_BUFSZ));
+    str.reserve (STDERR_BUFSZ);
+
+    do
+      {
+        nr = read (m_stderr_fd, buff.get (), STDERR_BUFSZ);
+	if (nr < 0)
+	  {
+	    cuda_trace_domain (CUDA_TRACE_DISASSEMBLER, 
+			       "cuobjdump_process: failed to read from stderr: %s",
+			       safe_strerror (errno));
+	    break;
+	  }
+        if (nr > 0)
+            str.append (buff.get (), nr);
+      } while (nr > 0);
+
+    return str;
+  }
+
   void
   exec (uint64_t pc, const std::string &filename, const char *function_name,
 	const bool generate_json)
@@ -319,67 +393,48 @@ public:
     std::string cuobjdump_str ("cuobjdump");
     std::vector<std::string> cuobjdump_args
 	= { cuobjdump_str, "--dump-sass", filename };
+
     if (generate_json)
-      {
-	cuobjdump_args.push_back ("-json");
-      }
-    if (function_name)
+      cuobjdump_args.push_back ("-json");
+
+    if (function_name && function_name[0])
       {
 	cuobjdump_args.push_back ("--function");
 	cuobjdump_args.push_back (function_name);
-	cuda_trace_domain (CUDA_TRACE_DISASSEMBLER,
-			   "disassembler command (ELF): pc 0x%lx: %s "
-			   "--dump-sass %s --function '%s'",
-			   pc, cuobjdump_str.c_str (), filename.c_str (),
-			   function_name);
       }
-    else
-      cuda_trace_domain (
-	  CUDA_TRACE_DISASSEMBLER,
-	  "disassembler command (ELF): pc 0x%lx: %s --dump-sass %s", pc,
-	  cuobjdump_str.c_str (), filename.c_str ());
 
-    int stdout_pipe_fds[2];
-    CUDA_ERR_IF (pipe (stdout_pipe_fds) == -1, CUDA_TRACE_DISASSEMBLER,
-		 "Failed to create a pipe for cuobjdump: %s",
-		 safe_strerror (errno));
+    if (cuda_options_trace_domain_enabled (CUDA_TRACE_DISASSEMBLER))
+      {
+	// Build up and log the cuobjdump invocation command line
+	std::string args;
+	for (auto arg : cuobjdump_args)
+	  {
+	    args.append(" ");
+	    args.append(arg);
+	  }
+	cuda_trace_domain (CUDA_TRACE_DISASSEMBLER,
+			   "disassembler command (ELF): pc 0x%lx: %s", pc, args.c_str ());
+      }
 
-    int ret;
+    int stdout_fds[2];
+    int stderr_fds[2];
     posix_spawn_file_actions_t file_actions;
 
-    ret = posix_spawn_file_actions_init (&file_actions);
+    CUDA_ERR_IF (pipe (stdout_fds) == -1 || pipe (stderr_fds) == -1, CUDA_TRACE_DISASSEMBLER,
+                 "cuobjdump_process: pipe failed: %s",
+                 safe_strerror (errno));
+
+    int ret = posix_spawn_file_actions_init (&file_actions);
     CUDA_ERR_IF (
 	ret != 0, CUDA_TRACE_DISASSEMBLER,
 	"Failed to spawn cuobjdump. posix_spawn_file_actions_init failed: %s",
 	safe_strerror (ret));
 
-    /* Close read-ends */
-    ret = posix_spawn_file_actions_addclose (&file_actions,
-					     stdout_pipe_fds[0]);
-    CUDA_ERR_IF (ret != 0, CUDA_TRACE_DISASSEMBLER,
-		 "Failed to spawn cuobjdump. "
-		 "posix_spawn_file_actions_addclose failed: %s",
-		 safe_strerror (ret));
+    m_stdout_fd = set_up_fd_redirect (&file_actions, stdout_fds, STDOUT_FILENO);
+    m_stderr_fd = set_up_fd_redirect (&file_actions, stderr_fds, STDERR_FILENO);
 
-    /* Redirect stdout/stderr */
-    ret = posix_spawn_file_actions_adddup2 (&file_actions, stdout_pipe_fds[1],
-					    STDOUT_FILENO);
-    CUDA_ERR_IF (ret != 0, CUDA_TRACE_DISASSEMBLER,
-		 "Failed to spawn cuobjdump. "
-		 "posix_spawn_file_actions_adddup2 stdout failed: %s",
-		 safe_strerror (ret));
-
-    /* Close write-ends */
-    ret = posix_spawn_file_actions_addclose (&file_actions,
-					     stdout_pipe_fds[1]);
-    CUDA_ERR_IF (ret != 0, CUDA_TRACE_DISASSEMBLER,
-		 "Failed to spawn cuobjdump. "
-		 "posix_spawn_file_actions_addclose failed: %s",
-		 safe_strerror (ret));
-
-    pid_t child_pid;
     auto argv = vector_to_argv (cuobjdump_args);
-    ret = posix_spawnp (&child_pid, cuobjdump_str.c_str (), &file_actions,
+    ret = posix_spawnp (&m_pid, cuobjdump_str.c_str (), &file_actions,
 			NULL, const_cast<char *const *> (argv.get ()),
 			environ);
     if (ret != 0)
@@ -400,27 +455,28 @@ public:
 		     safe_strerror (ret));
 
 	argv.get ()[0] = cuobjdump_path.c_str ();
-	ret = posix_spawnp (&child_pid, cuobjdump_path.c_str (), &file_actions,
+	ret = posix_spawnp (&m_pid, cuobjdump_path.c_str (), &file_actions,
 			    NULL, const_cast<char *const *> (argv.get ()),
 			    environ);
       }
 
     posix_spawn_file_actions_destroy (&file_actions);
 
+    /* Close write-end in current process, we don't need it.
+       This will also allow the read-end from this process to return EOF if no more data are available. */
+    close (stdout_fds[1]);
+    close (stderr_fds[1]);
+
     CUDA_ERR_IF (ret != 0, CUDA_TRACE_DISASSEMBLER,
 		 "Failed to spawn cuobjdump. posix_spawnp failed: %s",
 		 safe_strerror (ret));
-
-    close (stdout_pipe_fds[1]);
-
-    m_pid = child_pid;
-    m_stdout_fd = stdout_pipe_fds[0];
   }
 
   int
   cleanup ()
   {
     close (m_stdout_fd);
+    close (m_stderr_fd);
 
     int child_status;
     pid_t wait_ret = waitpid (m_pid, &child_status, 0);
@@ -437,13 +493,18 @@ public:
 gdb::optional<cuda_instruction>
 cuda_module_disassembly_cache::populate_from_elf_image (const uint64_t pc)
 {
-  /* collect all the necessary data */
-  kernel_t kernel = cuda_current_focus::get ().logical ().kernel ();
-  auto module = kernel_get_module (kernel);
-  auto filename = module->filename ();
   static constexpr bool generate_json_output = true;
 
-  cuda_trace_domain (CUDA_TRACE_DISASSEMBLER, "populate (ELF): pc 0x%lx", pc);
+  // If we couldn't find the cubin, we can't disassemble from the elf
+  // image.
+  const auto module = cuda_state::find_module_by_address (pc);
+  if (!module)
+    {
+      warning ("Could not find cubin to disassemble for pc 0x%lx", pc);
+      return gdb::optional<cuda_instruction> ();
+    }
+  const auto filename = module->filename ();
+  cuda_trace_domain (CUDA_TRACE_DISASSEMBLER, "populate (ELF): found pc 0x%lx in %s", pc, filename.c_str ());
 
   /* Generate the dissassembled code by using cuobjdump Can be
      per-function (faster, but may be invoked multiple times for a
@@ -460,15 +521,24 @@ cuda_module_disassembly_cache::populate_from_elf_image (const uint64_t pc)
      parser fails to parse the cuobjdump output */
   if (m_cuobjdump_json)
     {
+      cuda_trace_domain (CUDA_TRACE_DISASSEMBLER,
+          "Trying to parse cuobjdump json output");
+
       proc.exec (pc, filename, function_name.get (), generate_json_output);
 
       /* parse the json output */
 
       const bool parsed = parse_disasm_output_json (proc.stdout_fd ());
 
-      const bool successed = (proc.cleanup () == 0);
+      /* Dump stderr into trace logs */
+      std::string proc_err = proc.get_stderr ();
+      if (!proc_err.empty ())
+	cuda_trace_domain (CUDA_TRACE_DISASSEMBLER,
+			   "cuobjdump_process error: %s",
+			   proc_err.c_str ());
 
-      if (successed)
+      const bool success = (proc.cleanup () == 0);
+      if (success)
 	{
 	  if (parsed)
 	    return cache_lookup (pc, disassembly_source::ELF);
@@ -482,12 +552,22 @@ cuda_module_disassembly_cache::populate_from_elf_image (const uint64_t pc)
       m_cuobjdump_json = false;
     }
 
+  cuda_trace_domain (CUDA_TRACE_DISASSEMBLER,
+        "Failed to parse cuobjdump json output, falling back to plaintext disassembly");
+
   proc.exec (pc, filename, function_name.get (), !generate_json_output);
 
   parse_disasm_output (proc.stdout_fd (), module);
-  const bool successed = (proc.cleanup () == 0);
+  const bool success = (proc.cleanup () == 0);
 
-  CUDA_ERR_IF (!successed, CUDA_TRACE_DISASSEMBLER,
+  /* Dump stderr into trace logs */
+  std::string proc_err = proc.get_stderr ();
+  if (!proc_err.empty ())
+    cuda_trace_domain (CUDA_TRACE_DISASSEMBLER,
+                       "cuobjdump_process error: %s",
+                       proc_err.c_str ());
+
+  CUDA_ERR_IF (!success, CUDA_TRACE_DISASSEMBLER,
 	       "Failed to cleanup cuobjdump");
 
   return cache_lookup (pc, disassembly_source::ELF);
@@ -522,7 +602,7 @@ cuda_module_disassembly_cache::populate_from_device_memory (const uint64_t pc)
     return gdb::optional<cuda_instruction> ();
 
   cuda_instruction instruction (buf);
-  map_insert_or_assign (m_device_map, pc, instruction);
+  map_insert_or_assign (m_device_map, pc, cuda_instruction (instruction));
   cuda_trace_domain (CUDA_TRACE_DISASSEMBLER,
 		     "disasm (debugAPI): pc 0x%lx: %s", pc, buf);
   return instruction;
@@ -786,8 +866,8 @@ cuda_module_disassembly_cache::parse_disasm_output (const int fd, cuda_module* m
 	      pc = entry_pc + current_offset;
 
 	      /* insert the disassembled instruction into the map */
-	      map_insert_or_assign (m_elf_map, pc,
-				    cuda_instruction (current_insn));
+	      map_insert_or_assign (
+		  m_elf_map, pc, cuda_instruction (current_insn));
 	      last_pc = pc;
 	      cuda_trace_domain (CUDA_TRACE_DISASSEMBLER,
 				 "offset-insn: cache pc 0x%lx insn: %s", pc,
