@@ -1,6 +1,6 @@
 /*
  * NVIDIA CUDA Debugger CUDA-GDB
- * Copyright (C) 2007-2024 NVIDIA Corporation
+ * Copyright (C) 2007-2025 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -147,7 +147,7 @@ autostep_report_exception_device (int nsteps, int before_ln,
       cuda_coord_set<cuda_coord_set_type::threads, select_valid | select_sngl>
           lane{ filter };
       gdb_assert (lane.size ());
-      cuda_current_focus::set (const_cast<cuda_coords &> (*lane.begin ()));
+      switch_to_cuda_thread (*lane.begin ());
     }
 
   cuda_current_focus::printFocus (false);
@@ -181,17 +181,14 @@ autostep_report_exception_device (int nsteps, int before_ln,
 static uint64_t
 find_end_pc (uint64_t pc)
 {
-  const struct block *bl;
-  struct bound_minimal_symbol msymbol;
-
-  bl = block_for_pc ((CORE_ADDR)pc);
+  const struct block *bl = block_for_pc ((CORE_ADDR)pc);
   if (bl)
     return bl->end ();
 
-  msymbol = lookup_minimal_symbol_by_pc ((CORE_ADDR)pc);
+  struct bound_minimal_symbol msymbol = lookup_minimal_symbol_by_pc ((CORE_ADDR)pc);
   if (msymbol.minsym)
-    return  msymbol.minsym->value_raw_address ()
-           + msymbol.minsym->size ();
+    return  (uint64_t)msymbol.minsym->unrelocated_address ()
+           + (uint64_t)msymbol.minsym->size ();
 
   return (uint64_t)-1LL;
 }
@@ -199,10 +196,9 @@ find_end_pc (uint64_t pc)
 static int
 count_instructions (uint64_t pc, uint64_t end_pc)
 {
-  kernel_t kernel = cuda_current_focus::get ().logical ().kernel ();
   int count = 0;
-
-  auto module = kernel_get_module (kernel);
+  const auto kernel = cuda_current_focus::get ().logical ().kernel ();
+  const auto module = kernel->module ();
   gdb_assert (module);
 
   auto disassembler = module->disassembler ();
@@ -358,7 +354,7 @@ select_next_valid_warp (void)
               gdb_assert (lane.size ());
               next_coord = *lane.begin ();
             }
-          cuda_current_focus::set (next_coord);
+	  switch_to_cuda_thread (next_coord);
 
           const auto &p = next_coord.physical ();
           const auto &l = next_coord.logical ();
@@ -422,7 +418,7 @@ set_next_device_iteration (void)
     }
   else
     {
-      struct linetable_entry *best_item = NULL; /* ignored */
+      const linetable_entry *best_item = NULL; /* ignored */
 
       /* Search for all PCs which correspond to (current + remaining) line.
        * Try to pick up lowest possible address after current pc
@@ -662,8 +658,8 @@ update_device_autostep_state (CORE_ADDR pc)
   struct symtab_and_line before_sal = astep_state.cur_sal;
   uint32_t inst_size = astep_state.device.inst_size;
 
-  /* Update current coords */
-  cuda_current_focus::set (c);
+  /* Update current CUDA focus */
+  switch_to_cuda_thread (c);
 
   /* If the lane is not active, keep stepping it until it is.  Keep all the
      data unchanged until the lane becomes active.  */
