@@ -1,6 +1,6 @@
 /*
  * NVIDIA CUDA Debugger CUDA-GDB
- * Copyright (C) 2015-2024 NVIDIA Corporation
+ * Copyright (C) 2015-2025 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -83,7 +83,7 @@ cv_get_last_driver_api_error_func_name (CORE_ADDR *name)
    returns a number of active kernels */
 static uint32_t
 cuda_convenience_get_present_blocks_kernels (
-    struct value **kernel_id_array_value, struct value **block_idx_array_value)
+    value **kernel_id_array_value, value **block_idx_array_value)
 {
   gdb_assert (kernel_id_array_value);
   gdb_assert (block_idx_array_value);
@@ -133,23 +133,23 @@ cuda_convenience_get_present_blocks_kernels (
   /* if no block or kernel, then return empty arrays */
   if (!kernels.size ())
     {
-      *kernel_id_array_value = allocate_value (type_array);
-      *block_idx_array_value = allocate_value (type_array);
+      *kernel_id_array_value = value::allocate (type_array);
+      *block_idx_array_value = value::allocate (type_array);
       return 0;
     }
 
   gdb_assert (kernels.size () == blocks_itr.size ());
 
   /* Create the block value array */
-  std::vector<struct value *> kernel_block_idx_val_arr;
+  std::vector<value *> kernel_block_idx_val_arr;
   /* Iterate over every block iterator for every kernel*/
   for (auto &block_itr : blocks_itr)
     {
-      std::vector<struct value *> block_idx_val_arr;
+      std::vector<value *> block_idx_val_arr;
       // Iterate over every block in the block iterator
       for (const auto &block : block_itr)
         {
-          std::vector<struct value *> block_dim_val_arr;
+          std::vector<value *> block_dim_val_arr;
           block_dim_val_arr.push_back (value_from_longest (
               type_uint32, (LONGEST)block.logical ().blockIdx ().x));
           block_dim_val_arr.push_back (value_from_longest (
@@ -157,13 +157,13 @@ cuda_convenience_get_present_blocks_kernels (
           block_dim_val_arr.push_back (value_from_longest (
               type_uint32, (LONGEST)block.logical ().blockIdx ().z));
           block_idx_val_arr.push_back (
-              value_array (1, 3, block_dim_val_arr.data ()));
+              value_array (1, block_dim_val_arr));
         }
       // Value arrays require each element to be the same size - push back any
       // uninitialized values up to max_blocks
       for (auto i = max_blocks - block_itr.size (); i > 0; i--)
         {
-          std::vector<struct value *> block_dim_val_arr;
+          std::vector<value *> block_dim_val_arr;
           block_dim_val_arr.push_back (
               value_from_longest (type_uint32, (LONGEST)CUDA_INVALID));
           block_dim_val_arr.push_back (
@@ -171,25 +171,25 @@ cuda_convenience_get_present_blocks_kernels (
           block_dim_val_arr.push_back (
               value_from_longest (type_uint32, (LONGEST)CUDA_INVALID));
           block_idx_val_arr.push_back (
-              value_array (1, 3, block_dim_val_arr.data ()));
+              value_array (1, block_dim_val_arr));
         }
       // Push back the completed block arrays for this kernel
       kernel_block_idx_val_arr.push_back (
-          value_array (1, max_blocks, block_idx_val_arr.data ()));
+          value_array (1, block_idx_val_arr));
     }
   /* We have completed the block value array. */
   *block_idx_array_value
-      = value_array (1, kernels.size (), kernel_block_idx_val_arr.data ());
+      = value_array (1, kernel_block_idx_val_arr);
 
   /* Create the kernelId value array */
-  std::vector<struct value *> kernel_val_arr;
+  std::vector<value *> kernel_val_arr;
   /* Iterate over every kernelId */
   for (auto kernelId : kernels)
     kernel_val_arr.push_back (
         value_from_longest (type_uint32, (LONGEST)kernelId));
   /* We have completed the kernelId value array. */
   *kernel_id_array_value
-      = value_array (1, kernels.size (), kernel_val_arr.data ());
+      = value_array (1, kernel_val_arr);
 
   return kernels.size ();
 }
@@ -214,42 +214,6 @@ cv_set_uint64_var (const char *name, uint64_t val)
   set_internalvar (var, value_from_longest (type, (LONGEST)val));
 }
 
-static void
-cv_update_hw_vars (void)
-{
-  cv_set_uint32_var ("cuda_latest_launched_kernel_id",
-                     cuda_latest_launched_kernel_id ());
-
-  if (!cuda_current_focus::isDevice ())
-    {
-      cv_set_uint32_var ("cuda_num_devices", CUDA_INVALID);
-      cv_set_uint32_var ("cuda_num_sm", CUDA_INVALID);
-      cv_set_uint32_var ("cuda_num_warps", CUDA_INVALID);
-      cv_set_uint32_var ("cuda_num_lanes", CUDA_INVALID);
-      cv_set_uint32_var ("cuda_num_registers", CUDA_INVALID);
-      cv_set_uint32_var ("cuda_num_uregisters", CUDA_INVALID);
-      cv_set_uint32_var ("cuda_num_registers_allocated", CUDA_INVALID);
-      cv_set_uint32_var ("cuda_shared_memory_size", CUDA_INVALID);
-      return;
-    }
-
-  const auto &c = cuda_current_focus::get ().physical ();
-  cv_set_uint32_var ("cuda_num_devices", cuda_state::get_num_devices ());
-  cv_set_uint32_var ("cuda_num_sm", cuda_state::device_get_num_sms (c.dev ()));
-  cv_set_uint32_var ("cuda_num_warps",
-                     cuda_state::device_get_num_warps (c.dev ()));
-  cv_set_uint32_var ("cuda_num_lanes",
-                     cuda_state::device_get_num_lanes (c.dev ()));
-  cv_set_uint32_var ("cuda_num_registers",
-                     cuda_state::device_get_num_registers (c.dev ()));
-  cv_set_uint32_var ("cuda_num_uregisters",
-                     cuda_state::device_get_num_uregisters (c.dev ()));
-  cv_set_uint32_var ("cuda_num_registers_allocated",
-        	     cuda_state::warp_registers_allocated (c.dev (), c.sm (), c.wp ()));
-  cv_set_uint32_var ("cuda_shared_memory_size",
-        	     cuda_state::warp_shared_mem_size (c.dev (), c.sm (), c.wp ()));
-}
-
 static void
 cv_update_focus_vars (void)
 {
@@ -281,8 +245,8 @@ cv_update_focus_vars (void)
   const auto &p = current.physical ();
   const auto &l = current.logical ();
   auto kernel = cuda_state::warp_get_kernel (p.dev (), p.sm (), p.wp ());
-  auto grid_dim = kernel_get_grid_dim (kernel);
-  auto block_dim = kernel_get_block_dim (kernel);
+  const auto grid_dim = kernel->grid_dim ();
+  const auto block_dim = kernel->block_dim ();
   bool active
       = cuda_state::lane_valid (p.dev (), p.sm (), p.wp (), p.ln ())
         && cuda_state::lane_active (p.dev (), p.sm (), p.wp (), p.ln ());
@@ -326,36 +290,6 @@ cv_update_lineno_var (void)
   cv_set_uint32_var ("cuda_thread_lineno", lineno);
 }
 
-static void
-cv_update_call_depth_var (void)
-{
-  uint32_t depth = CUDA_INVALID;
-
-  if (cuda_current_focus::isDevice ())
-    {
-      const auto &c = cuda_current_focus::get ().physical ();
-      depth = cuda_state::lane_get_call_depth (c.dev (), c.sm (), c.wp (),
-                                               c.ln ());
-    }
-
-  cv_set_uint32_var ("cuda_call_depth", depth);
-}
-
-static void
-cv_update_syscall_depth_var (void)
-{
-  uint32_t depth = CUDA_INVALID;
-
-  if (cuda_current_focus::isDevice ())
-    {
-      const auto &c = cuda_current_focus::get ().physical ();
-      depth = cuda_state::lane_get_syscall_call_depth (c.dev (), c.sm (),
-                                                       c.wp (), c.ln ());
-    }
-
-  cv_set_uint32_var ("cuda_syscall_call_depth", depth);
-}
-
 static void
 cv_update_api_failures_vars (void)
 {
@@ -376,7 +310,7 @@ static void
 cv_update_present_kernels_vars (void)
 {
   uint32_t num_kernels;
-  struct value *kernel_array, *blocks_array;
+  value *kernel_array, *blocks_array;
 
   num_kernels = cuda_convenience_get_present_blocks_kernels (&kernel_array,
                                                              &blocks_array);
@@ -404,13 +338,9 @@ struct cv_variable_group
 };
 
 static struct cv_variable_group cv_var_groups[] = {
-  { "hw_vars", false, cv_update_hw_vars,
-    "HW specific variables, like number of sms, lanes, warps, etc" },
   { "focus", false, cv_update_focus_vars, "CUDA focus specific variables" },
   { "lineno", false, cv_update_lineno_var,
     "Line number that matches $pc in focus" },
-  { "call_depth", false, cv_update_call_depth_var, "Call depth" },
-  { "syscall_depth", false, cv_update_syscall_depth_var, "Systemcall depth" },
   { "api_failures", false, cv_update_api_failures_vars,
     "API failure error number and function name" },
   { "present_kernels", false, cv_update_present_kernels_vars,
@@ -426,23 +356,26 @@ cuda_enable_convenience_variables_group (char *name, bool enable)
   struct cv_variable_group *grp;
   int rc = false;
 
+  auto mark = value_mark ();
   for (grp = cv_var_groups; grp->group_name; grp++)
     if (!name || strcasecmp (name, grp->group_name) == 0)
       {
-        grp->enabled = enable;
+	grp->enabled = enable;
+	if (grp->enabled)
+	  grp->update_func ();
         rc = true;
       }
+  value_free_to_mark (mark);
   return rc;
 }
 
 void
 cuda_update_convenience_variables (void)
 {
-  struct value *mark = NULL;
+  value *mark = NULL;
   struct cv_variable_group *grp;
 
   mark = value_mark ();
-
   for (grp = cv_var_groups; grp->group_name; grp++)
     if (grp->enabled)
       grp->update_func ();
@@ -478,3 +411,261 @@ cuda_build_covenience_variables_help_message (char *ptr, int size)
       size -= rc;
     }
 }
+
+struct cv_variable
+{
+  const char *name;
+  bool check_focus;
+  void *getter;
+  struct internalvar_funcs funcs;
+};
+
+template <typename T> using cv_lazy_getter = T (*) ();
+template <typename T> using cv_lazy_device_getter = T (*) (uint32_t dev_id);
+template <typename T> using cv_lazy_sm_getter = T (*) (uint32_t dev_id, uint32_t sm_id);
+template <typename T> using cv_lazy_warp_getter = T (*) (uint32_t dev_id, uint32_t sm_id, uint32_t wp_id);
+template <typename T> using cv_lazy_lane_getter = T (*) (uint32_t dev_id, uint32_t sm_id, uint32_t wp_id, uint32_t ln_id);
+
+template<typename T>
+static
+struct value *
+cv_lazy_var (const char *name,
+	     struct gdbarch *gdbarch,
+	     struct type *type,
+	     struct cv_variable *variable,
+	     cv_lazy_getter<T> getter)
+{
+  struct value *val = nullptr;
+
+  try
+    {
+      if (!variable->check_focus || cuda_current_focus::isDevice ())
+	{
+	  const T value_read = getter ();
+	  val = value_from_longest (type, (LONGEST)value_read);
+	}
+    }
+  catch (...)
+    {
+      cuda_trace ("error reading $%s\n", name);
+    }
+  if (!val)
+    val = value::allocate (builtin_type (gdbarch)->builtin_void);
+
+  return val;
+}
+
+template<typename T>
+static
+struct value *
+cv_lazy_var (const char *name,
+	     struct gdbarch *gdbarch,
+	     struct type *type,
+	     struct cv_variable *variable,
+	     cv_lazy_device_getter<T> getter)
+{
+  struct value *val = nullptr;
+
+  try
+    {
+      if (cuda_current_focus::isDevice ())
+	{
+	  const auto &c = cuda_current_focus::get ().physical ();
+	  const T value_read = getter (c.dev ());
+	  val = value_from_longest (type, (LONGEST)value_read);
+	}
+    }
+  catch (...)
+    {
+      cuda_trace ("error reading $%s\n", name);
+    }
+  if (!val)
+    val = value::allocate (builtin_type (gdbarch)->builtin_void);
+
+  return val;
+}
+
+template<typename T>
+static
+struct value *
+cv_lazy_var (const char *name,
+	     struct gdbarch *gdbarch,
+	     struct type *type,
+	     struct cv_variable *variable,
+	     cv_lazy_sm_getter<T> getter)
+{
+  struct value *val = nullptr;
+
+  try
+    {
+      if (cuda_current_focus::isDevice ())
+	{
+	  const auto &c = cuda_current_focus::get ().physical ();
+	  const T value_read = getter (c.dev (), c.sm ());
+	  val = value_from_longest (type, (LONGEST)value_read);
+	}
+    }
+  catch (...)
+    {
+      cuda_trace ("error reading $%s\n", name);
+    }
+  if (!val)
+    val = value::allocate (builtin_type (gdbarch)->builtin_void);
+
+  return val;
+}
+
+template<typename T>
+static
+struct value *
+cv_lazy_var (const char *name,
+	     struct gdbarch *gdbarch,
+	     struct type *type,
+	     struct cv_variable *variable,
+	     cv_lazy_warp_getter<T> getter)
+{
+  struct value *val = nullptr;
+
+  try
+    {
+      if (cuda_current_focus::isDevice ())
+	{
+	  const auto &c = cuda_current_focus::get ().physical ();
+	  const T value_read = getter (c.dev (), c.sm (), c.wp ());
+	  val = value_from_longest (type, (LONGEST)value_read);
+	}
+    }
+  catch (...)
+    {
+      cuda_trace ("error reading $%s\n", name);
+    }
+  if (!val)
+    val = value::allocate (builtin_type (gdbarch)->builtin_void);
+
+  return val;
+}
+
+template<typename T>
+static
+struct value *
+cv_lazy_var (const char *name,
+	     struct gdbarch *gdbarch,
+	     struct type *type,
+	     struct cv_variable *variable,
+	     cv_lazy_lane_getter<T> getter)
+{
+  struct value *val = nullptr;
+
+  try
+    {
+      if (cuda_current_focus::isDevice ())
+	{
+	  const auto &c = cuda_current_focus::get ().physical ();
+	  const T value_read = getter (c.dev (), c.sm (), c.wp (), c.ln ());
+	  val = value_from_longest (type, (LONGEST)value_read);
+	}
+    }
+  catch (...)
+    {
+      cuda_trace ("error reading $%s\n", name);
+    }
+  if (!val)
+    val = value::allocate (builtin_type (gdbarch)->builtin_void);
+
+  return val;
+}
+
+static value *
+cv_lazy_getter_uint32 (gdbarch *gdbarch, struct internalvar *var, void *data)
+{
+  const auto var_info = (cv_variable *) data;
+  return cv_lazy_var<uint32_t> (internalvar_name (var),
+  				gdbarch,
+				builtin_type (gdbarch)->builtin_uint32,
+				var_info,
+				(cv_lazy_getter<uint32_t>) var_info->getter);
+}
+
+static value *
+cv_lazy_getter_uint64 (gdbarch *gdbarch, struct internalvar *var, void *data)
+{
+  const auto var_info = (cv_variable *) data;
+  return cv_lazy_var<uint64_t> (internalvar_name (var),
+  				gdbarch,
+				builtin_type (gdbarch)->builtin_uint64,
+				var_info,
+				(cv_lazy_getter<uint64_t>) var_info->getter);
+}
+
+static value *
+cv_lazy_device_getter_uint32 (gdbarch *gdbarch, struct internalvar *var, void *data)
+{
+  const auto var_info = (cv_variable *) data;
+  return cv_lazy_var<uint32_t> (internalvar_name (var),
+  				gdbarch,
+				builtin_type (gdbarch)->builtin_uint32,
+				var_info,
+				(cv_lazy_device_getter<uint32_t>) var_info->getter);
+}
+
+#if 0
+/* Implemented for completeness sake, but commented out as it's
+   not used in the current codebase, and since it's static the compiler
+   will warn/error about it being unused. */
+
+static value *
+cv_lazy_sm_getter_uint32 (gdbarch *gdbarch, struct internalvar *var, void *data)
+{
+  const auto var_info = (cv_variable *) data;
+  return cv_lazy_var<uint32_t> (internalvar_name (var),
+  				gdbarch,
+				builtin_type (gdbarch)->builtin_uint32,
+				var_info,
+				(cv_lazy_sm_getter<uint32_t>) var_info->getter);
+}
+#endif
+
+static value *
+cv_lazy_warp_getter_uint32 (gdbarch *gdbarch, struct internalvar *var, void *data)
+{
+  const auto var_info = (cv_variable *) data;
+  return cv_lazy_var<uint32_t> (internalvar_name (var),
+  				gdbarch,
+				builtin_type (gdbarch)->builtin_uint32,
+				var_info,
+				(cv_lazy_warp_getter<uint32_t>) var_info->getter);
+}
+
+static value *
+cv_lazy_lane_getter_uint32 (gdbarch *gdbarch, struct internalvar *var, void *data)
+{
+  const auto var_info = (cv_variable *) data;
+  return cv_lazy_var<uint32_t> (internalvar_name (var),
+  				gdbarch,
+				builtin_type (gdbarch)->builtin_uint32,
+				var_info,
+				(cv_lazy_lane_getter<uint32_t>) var_info->getter);
+}
+
+static struct cv_variable cv_lazy_table[] = {
+  { "cuda_call_depth", true, (void *)cuda_state::lane_get_call_depth, { cv_lazy_lane_getter_uint32 } },
+  { "cuda_latest_launched_kernel_id", false, (void *)cuda_state::last_launched_kernel_id, { cv_lazy_getter_uint64 } },
+  { "cuda_num_devices", false, (void *)cuda_state::get_num_devices, { cv_lazy_getter_uint32 } },
+  { "cuda_num_lanes", true, (void *)cuda_state::device_get_num_lanes, { cv_lazy_device_getter_uint32 } },
+  { "cuda_num_registers_allocated", true, (void *)cuda_state::warp_registers_allocated, { cv_lazy_warp_getter_uint32 } },
+  { "cuda_num_registers", true, (void *)cuda_state::device_get_num_registers, { cv_lazy_device_getter_uint32 } },
+  { "cuda_num_sms", true, (void *)cuda_state::device_get_num_sms, { cv_lazy_device_getter_uint32 } },
+  { "cuda_num_uregisters", true, (void *)cuda_state::device_get_num_uregisters, { cv_lazy_device_getter_uint32 } },
+  { "cuda_num_warps", true, (void *)cuda_state::device_get_num_warps, { cv_lazy_device_getter_uint32 } },
+  { "cuda_shared_memory_size", true, (void *)cuda_state::warp_shared_mem_size, { cv_lazy_warp_getter_uint32 } },
+  { "cuda_syscall_call_depth", true, (void *)cuda_state::lane_get_call_depth, { cv_lazy_lane_getter_uint32 } },
+  { nullptr }
+};
+
+void _initialize_cuda_lazy_convvars ();
+void
+_initialize_cuda_lazy_convvars ()
+{
+  for (auto i = 0; cv_lazy_table[i].name; i++)
+    create_internalvar_type_lazy (cv_lazy_table[i].name, &cv_lazy_table[i].funcs, &cv_lazy_table[i]);
+}
