@@ -1,6 +1,6 @@
 /*
  * NVIDIA CUDA Debugger CUDA-GDB
- * Copyright (C) 2007-2024 NVIDIA Corporation
+ * Copyright (C) 2007-2025 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -431,11 +431,10 @@ public:
 		CuDim3 clusterIdx = CUDA_INVALID_DIM;
 		if (validWarp)
 		  {
-		    kernel_t kernel
-			= cuda_state::warp_get_kernel (dev, sm, wp);
+		    const auto kernel = cuda_state::warp_get_kernel (dev, sm, wp);
 		    gdb_assert (kernel);
-		    kernelId = kernel_get_id (kernel);
-		    CuDim3 clusterDim = kernel_get_cluster_dim (kernel);
+		    kernelId = kernel->id ();
+		    const auto& clusterDim = kernel->cluster_dim ();
 		    if ((clusterDim.x != 0) && (clusterDim.y != 0)
 			&& (clusterDim.z != 0))
 		      {
