@@ -1,6 +1,6 @@
 /*
  * NVIDIA CUDA Debugger CUDA-GDB
- * Copyright (C) 2007-2024 NVIDIA Corporation
+ * Copyright (C) 2007-2025 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -531,7 +531,7 @@ cuda_frame_prev_register (frame_info_ptr next_frame, void **this_cache,
       else
         {
           int i;
-          gdb::array_view<const gdb_byte> buf = value_contents (value);
+          gdb::array_view<const gdb_byte> buf = value->contents ();
           gdb_printf (gdb_stdlog, "[");
           for (i = 0; i < register_size (gdbarch, regnum); i++)
             gdb_printf (gdb_stdlog, "%02x", buf[i]);
