@@ -1,6 +1,6 @@
 /*
  * NVIDIA CUDA Debugger CUDA-GDB
- * Copyright (C) 2007-2024 NVIDIA Corporation
+ * Copyright (C) 2007-2025 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -173,33 +173,9 @@ cuda_nat_linux<BaseTarget>::xfer_partial (enum target_object object,
 #endif
     }
 
+  /* Special handling for object types */
   switch (object)
     {
-    /* If focus is on GPU, still try to read the address using host routines,
-       if it fails, see if this address is in pinned system memoryi, i.e. to
-       system memory that was allocated by the inferior through the CUDA API */
-    case TARGET_OBJECT_MEMORY:
-      {
-	status = BaseTarget::xfer_partial (object, annex, readbuf, writebuf,
-					   offset, len, xfered_len);
-	if (*xfered_len)
-	  return TARGET_XFER_OK;
-
-	if (readbuf
-	    && cuda_debugapi::read_pinned_memory (offset, readbuf, len))
-	  {
-	    *xfered_len = len;
-	    return TARGET_XFER_OK;
-	  }
-	else if (writebuf
-		 && cuda_debugapi::write_pinned_memory (offset, writebuf, len))
-	  {
-	    *xfered_len = len;
-	    return TARGET_XFER_OK;
-	  }
-
-	return status;
-      }
     /* The stack lives in local memory for ABI compilations. */
     case TARGET_OBJECT_STACK_MEMORY:
       {
@@ -226,11 +202,15 @@ cuda_nat_linux<BaseTarget>::xfer_partial (enum target_object object,
 
 	return status;
       }
+
     /* When stopping on the device, build a simple siginfo object */
     case TARGET_OBJECT_SIGNAL_INFO:
-
       return cuda_xfer_siginfo (object, annex, readbuf, writebuf, offset, len,
 				xfered_len);
+
+    default:
+      /* Fall through */
+      break;
     }
 
   /* Fallback to host routines for other types of memory objects */
@@ -411,7 +391,7 @@ cuda_nat_linux<BaseTarget>::resume (ptid_t ptid, int sstep, int host_sstep,
 	}
       cuda_sstep_reset (false);
       cuda_insert_step_resume_breakpoint_at_caller (get_current_frame ());
-      cuda_insert_breakpoints ();
+      insert_breakpoints ();
     }
 
   // resume the device
@@ -545,9 +525,6 @@ cuda_nat_linux<BaseTarget>::wait (ptid_t ptid, struct target_waitstatus *ws,
 				  target_wait_flags target_options)
 {
   ptid_t r = null_ptid;
-  uint32_t dev, dev_id;
-  uint64_t grid_id;
-  kernel_t kernel;
   bool cuda_event_found = false;
   CUDBGEvent event, asyncEvent;
 
@@ -612,10 +589,7 @@ cuda_nat_linux<BaseTarget>::wait (ptid_t ptid, struct target_waitstatus *ws,
 		  cuda_trace ("cuda_wait: single-stepped to kernel exit");
 		  /* Only destroy the kernel that has been stepped to its exit
 		   */
-		  dev_id = cuda_sstep_dev_id ();
-		  grid_id = cuda_sstep_grid_id ();
-		  kernel = kernels_find_kernel_by_grid_id (dev_id, grid_id);
-		  kernels_terminate_kernel (kernel);
+		  cuda_state::destroy_kernel (cuda_sstep_dev_id (), cuda_sstep_grid_id ());
 
 		  /* Consume any asynchronous events, if necessary.  We need to
 		   * do this explicitly here, since we're taking the quick path
@@ -626,10 +600,10 @@ cuda_nat_linux<BaseTarget>::wait (ptid_t ptid, struct target_waitstatus *ws,
 		    cuda_process_events (&asyncEvent, CUDA_EVENT_ASYNC);
 
 		  /* Update device state/kernels */
-		  kernels_update_terminated ();
+		  cuda_state::update_kernels_terminated ();
 		  cuda_update_convenience_variables ();
 
-		  struct thread_info *tp = find_thread_ptid (this, r);
+		  struct thread_info *tp = this->find_thread (r);
 		  gdb_assert (tp);
 		  tp->control.step_range_end = 1;
 		  tp->need_cuda_updated_focus = true;
@@ -682,7 +656,7 @@ cuda_nat_linux<BaseTarget>::wait (ptid_t ptid, struct target_waitstatus *ws,
     }
 
   /* Obtain the thread_info struct for 'r' */
-  struct thread_info *tp = find_thread_ptid (this, r);
+  struct thread_info *tp = this->find_thread (r);
   if (!tp)
     {
       cuda_current_focus::invalidate ();
@@ -725,7 +699,7 @@ cuda_nat_linux<BaseTarget>::wait (ptid_t ptid, struct target_waitstatus *ws,
 
   /* Suspend all the CUDA devices. */
   cuda_trace ("cuda_wait: suspend devices");
-  for (dev = 0; dev < cuda_state::get_num_devices (); ++dev)
+  for (auto dev = 0; dev < cuda_state::get_num_devices (); ++dev)
     cuda_state::device_suspend (dev);
 
   if (is_remote_target (this))
@@ -761,7 +735,7 @@ cuda_nat_linux<BaseTarget>::wait (ptid_t ptid, struct target_waitstatus *ws,
     }
 
   /* Update the info about the kernels */
-  kernels_update_terminated ();
+  cuda_state::update_kernels_terminated ();
 
   /* Decide which thread/kernel to switch focus to.
    * c is passed by reference and filled out by several
@@ -1001,12 +975,9 @@ cuda_nat_linux<BaseTarget>::insert_breakpoint (struct gdbarch *gdbarch,
   uint32_t dev;
   bool inserted;
 
-  gdb_assert (bp_tgt->owner != NULL
-	      || gdbarch_bfd_arch_info (gdbarch)->arch == bfd_arch_arm
-	      || gdbarch_bfd_arch_info (gdbarch)->arch == bfd_arch_aarch64);
-
-  if (!bp_tgt->owner || !bp_tgt->owner->cuda_breakpoint)
-    return BaseTarget::insert_breakpoint (gdbarch, bp_tgt);
+  /* Make sure we switch to the target gdbarch here */
+  if (!cuda_is_device_code_address (bp_tgt->reqstd_address))
+    return BaseTarget::insert_breakpoint (target_gdbarch (), bp_tgt);
 
   /* Insert the breakpoint on whatever device accepts it (valid address). */
   inserted = false;
@@ -1031,8 +1002,9 @@ cuda_nat_linux<BaseTarget>::remove_breakpoint (struct gdbarch *gdbarch,
   uint32_t dev;
   bool removed;
 
-  if (!bp_tgt->owner || !bp_tgt->owner->cuda_breakpoint)
-    return BaseTarget::remove_breakpoint (gdbarch, bp_tgt, reason);
+  /* Make sure we switch to the target gdbarch here */
+  if (!cuda_is_device_code_address (bp_tgt->placed_address))
+    return BaseTarget::remove_breakpoint (target_gdbarch (), bp_tgt, reason);
 
   /* Removed the breakpoint on whatever device accepts it (valid address). */
   removed = false;
@@ -1070,4 +1042,4 @@ cuda_nat_linux<BaseTarget>::detach (inferior *inf, int from_tty)
 
   /* Call the host detach routine. */
   BaseTarget::detach (inf, from_tty);
-}
\ No newline at end of file
+}
