@@ -1,6 +1,6 @@
 /*
  * NVIDIA CUDA Debugger CUDA-GDB
- * Copyright (C) 2007-2024 NVIDIA Corporation
+ * Copyright (C) 2007-2025 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -65,9 +65,11 @@
 
 bool cuda_debugging_enabled = false;
 
-struct cuda_cudart_symbols_st cuda_cudart_symbols;
+static struct objfile * cuda_cudart_symbols;
 static struct cuda_signal_info_st cuda_sigtrap_info;
 
+static struct objfile *cuda_create_builtins_objfile (void);
+
 #if defined(__linux__) && defined(GDB_NM_FILE)
 static void
 cuda_clear_pending_sigint (pid_t pid)
@@ -197,6 +199,8 @@ cuda_adjust_host_pc (ptid_t r)
     cuda_notification_consume_pending ();
 }
 
+#ifndef __QNXTARGET__
+/* Attach isn't yet supported on QNX */
 static void
 cuda_nat_attach (inferior *inf)
 {
@@ -204,10 +208,10 @@ cuda_nat_attach (inferior *inf)
   struct cmd_list_element *prefix_cmd = NULL;
   struct cmd_list_element *cmd = NULL;
   const char *cudbgApiAttach = "(void) cudbgApiAttach()";
-  CORE_ADDR debugFlagAddr;
-  CORE_ADDR resumeAppOnAttachFlagAddr;
-  CORE_ADDR attachDataAvailableFlagAddr;
-  unsigned char resumeAppOnAttach;
+  CORE_ADDR debugFlagAddr = 0;
+  CORE_ADDR resumeAppOnAttachFlagAddr = 0;
+  CORE_ADDR attachDataAvailableFlagAddr = 0;
+  unsigned char resumeAppOnAttach = 0;
   unsigned int timeOut = 5000; // ms
   unsigned int timeElapsed = 0;
   unsigned dev = 0;
@@ -336,7 +340,7 @@ cuda_nat_attach (inferior *inf)
 	   "restart the process with CUDA_VISIBLE_DEVICES environment "
 	   "variable set."));
   if (internal_error_code)
-    error (_ ("Attach failed due to the internal driver error 0x%llx\n"),
+    error (_ ("Attach failed due to the internal driver error: CUresult=%llu\n"),
 	   (unsigned long long)internal_error_code);
 
   debugFlagAddr = cuda_get_symbol_address (_STRING_ (CUDBG_IPC_FLAG_NAME));
@@ -402,7 +406,7 @@ cuda_nat_attach (inferior *inf)
 		  "Please verify that software preemption is disabled "
 		  "and that nvidia-cuda-mps-server is not running."));
       if (internal_error_code)
-	error (_ ("Attach failed due to the internal driver error 0x%llx\n"),
+	error (_ ("Attach failed due to the internal driver error: CUresult=%llu\n"),
 	       (unsigned long long)internal_error_code);
 
       if (timeElapsed < timeOut)
@@ -434,7 +438,7 @@ cuda_nat_attach (inferior *inf)
 	  prepare_execution_command (inf->top_target (), true);
 	  continue_1 (false);
 	  /* force resumed state to false */
-	  bool resumed_state;
+	  bool resumed_state = false;
 	  if (is_remote_target (inf->process_target ()))
 	    {
 	      resumed_state = current_inferior ()->process_target ()->commit_resumed_state;
@@ -484,6 +488,7 @@ cuda_nat_attach (inferior *inf)
   /* The inferior just got signaled, we're not expecting any other stop */
   inf->control.stop_soon = NO_STOP_QUIETLY;
 }
+#endif /* !__QNXTARGET__ */
 
 static void
 cuda_inferior_created (inferior *inf)
@@ -527,6 +532,16 @@ cuda_do_detach (inferior *inf)
       return;
     }
 
+  /* This is a bit of a hack. We are about to tear down the debug API so future
+   * calls would fail. But if there are any breakpoints set, those usually would
+   * be removed after detach completes. A bug was found with the debug API where
+   * if breakpoints are not removed, they would not get cleaned up on detach
+   * correctly and left set. We were masking this bug in previous implementations
+   * of CUDA-GDB. To work around this, always try to delete breakpoints belonging
+   * to the inferiors program space before we tear down the debug API. */
+  cuda_options_disable_break_on_launch ();
+  breakpoint_program_space_exit (inf->pspace);
+
   cuda_debugapi::set_attach_state (CUDA_ATTACH_STATE_DETACHING);
 
   /* Make sure the focus is set on the host */
@@ -586,11 +601,6 @@ cuda_do_detach (inferior *inf)
     {
       int cnt;
 
-      /* Clear all breakpoints */
-      cuda_delete_command (NULL, 0);
-      cuda_state::cleanup_breakpoints ();
-      cuda_options_disable_break_on_launch ();
-
       /* Now resume the app and wait for CUDA_ATTACH_STATE_DETACH_COMPLETE
        * event. */
       for (cnt = 0; cnt < 100
@@ -635,19 +645,18 @@ switch_to_cuda_thread (const cuda_coords &coords)
 {
   uint64_t pc;
 
-  cuda_current_focus::set (const_cast<cuda_coords &> (coords));
+  cuda_current_focus::set (coords);
 
-  thread_info *thr = find_thread_ptid (current_inferior ()->process_target (),
-				       inferior_ptid);
+  thread_info *thr
+      = current_inferior ()->process_target ()->find_thread (inferior_ptid);
   /* Only update if a host thread still exists. */
   if (thr)
     {
       switch_to_thread_keep_cuda_focus (thr);
 
       cuda_update_convenience_variables ();
-      cuda_update_cudart_symbols ();
 
-      if (const_cast<cuda_coords &> (coords).isValidOnDevice ())
+      if (coords.isValidOnDevice ())
 	pc = cuda_state::lane_get_pc (
 	    coords.physical ().dev (), coords.physical ().sm (),
 	    coords.physical ().wp (), coords.physical ().ln ());
@@ -658,15 +667,13 @@ switch_to_cuda_thread (const cuda_coords &coords)
     }
 }
 
-struct objfile *cuda_create_builtins_objfile (void);
-
 void
-cuda_update_cudart_symbols (void)
+cuda_init_cudart_symbols (void)
 {
   /* If not done yet, create a CUDA runtime symbols file */
-  if (!cuda_cudart_symbols.objfile)
+  if (!cuda_cudart_symbols)
     {
-      cuda_cudart_symbols.objfile = cuda_create_builtins_objfile ();
+      cuda_cudart_symbols = cuda_create_builtins_objfile ();
     }
 }
 
@@ -674,11 +681,15 @@ void
 cuda_cleanup_cudart_symbols (void)
 {
   /* Free the objfile if allocated */
-  if (cuda_cudart_symbols.objfile)
+  if (cuda_cudart_symbols)
     {
-      cuda_cudart_symbols.objfile->unlink ();
-      cuda_cudart_symbols.objfile = NULL;
+      cuda_cudart_symbols->unlink ();
+      cuda_cudart_symbols = NULL;
     }
+#ifdef __QNXTARGET__
+  /* Reset the RT symbols in qnx */
+  cuda_reset_qnx_symbols ();
+#endif
 }
 
 /*
@@ -694,7 +705,7 @@ cuda_init_field (struct field &fp, const char *name, const int offs,
   fp.set_name (xstrdup (name));
   fp.set_type (type);
   fp.set_loc_bitpos (offs * 8);
-  FIELD_BITSIZE (fp) = type->length () * 8;
+  fp.set_bitsize (type->length () * 8);
 }
 
 /* Allocates dim3 type as structure of 3 packed unsigned int: x, y and z */
@@ -703,9 +714,7 @@ cuda_alloc_dim3_type (struct objfile *objfile)
 {
   struct gdbarch *gdbarch = objfile->arch ();
   struct type *uint32_type = builtin_type (gdbarch)->builtin_unsigned_int;
-  struct type *dim3 = NULL;
-
-  dim3 = alloc_type (objfile);
+  struct type *dim3 = type_allocator (uint32_type).new_type ();
 
   dim3->set_name ("dim3");
   dim3->set_length (12);
@@ -766,22 +775,22 @@ cuda_alloc_blockvector (struct symtab *symtab, int nblocks)
   bv->set_num_blocks (nblocks);
 
   /* Allocate the GLOBAL block */
-  struct block *global_block = allocate_global_block (obstack);
-  global_block->set_multidict (
+  struct block *g_block = new (obstack) global_block;
+  g_block->set_multidict (
       mdict_create_hashed_expandable (symtab->language ()));
-  bv->set_block (GLOBAL_BLOCK, global_block);
+  bv->set_block (GLOBAL_BLOCK, g_block);
 
   /* Only allowed for the GLOBAL block */
-  set_block_compunit_symtab (global_block, symtab->compunit ());
+  g_block->set_compunit_symtab (symtab->compunit ());
 
   /* Allocate the STATIC block*/
-  struct block *static_block = allocate_block (obstack);
+  struct block *static_block = new (obstack) struct block;
   static_block->set_multidict (
       mdict_create_hashed_expandable (symtab->language ()));
   bv->set_block (STATIC_BLOCK, static_block);
 
   /* superblock of the static block is the global block - see block.h */
-  static_block->set_superblock (global_block);
+  static_block->set_superblock (g_block);
 
   return bv;
 }
@@ -795,7 +804,7 @@ cuda_alloc_blockvector (struct symtab *symtab, int nblocks)
  * clusterDim of type dim3 located at CUDBG_CLUSTERDIM_OFFSET
  * warpSize of type int located at CUDBG_WARPSIZE_OFFSET
  */
-struct objfile *
+static struct objfile *
 cuda_create_builtins_objfile (void)
 {
   struct objfile *objfile = NULL;
@@ -855,4 +864,4 @@ _initialize_cuda_nat ()
   gdb::observers::inferior_created.attach (cuda_inferior_created, "CUDA");
 
   cuda_debugging_enabled = true;
-}
\ No newline at end of file
+}
