@@ -1,6 +1,6 @@
 /*
  * NVIDIA CUDA Debugger CUDA-GDB
- * Copyright (C) 2007-2024 NVIDIA Corporation
+ * Copyright (C) 2007-2025 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -45,7 +45,6 @@
 #include "linux-nat.h"
 
 /* Various state used by cuda-linux-nat-template.h */
-extern struct cuda_cudart_symbols_st cuda_cudart_symbols;
 extern bool cuda_debugging_enabled;
 extern int cuda_host_want_singlestep;
 
@@ -55,7 +54,6 @@ extern void cuda_signal_restore_settings (int sig,
 extern void cuda_sigtrap_restore_settings (void);
 extern void cuda_sigtrap_set_silent (void);
 extern bool cuda_check_pending_sigint (pid_t pid);
-extern struct objfile *cuda_create_builtins_objfile (void);
 
 class inf_child_target;
 
