@@ -1,6 +1,6 @@
 /*
  * NVIDIA CUDA Debugger CUDA-GDB
- * Copyright (C) 2007-2024 NVIDIA Corporation
+ * Copyright (C) 2007-2025 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -40,11 +40,11 @@
 #include "elf/common.h"
 #include "elf/external.h"
 
-#define R_CUDA_NONE		0	// no relocation
-#define R_CUDA_32		1	// 32bit specific address
-#define R_CUDA_64		2	// 64bit specific address
-#define R_CUDA_G32		3	// 32bit generic address
-#define R_CUDA_G64		4	// 64bit generic address
+#define R_CUDA_NONE 0  // no relocation
+#define R_CUDA_32   1  // 32bit specific address
+#define R_CUDA_64   2  // 64bit specific address
+#define R_CUDA_G32  3  // 32bit generic address
+#define R_CUDA_G64  4  // 64bit generic address
 
 static void
 cuda_trace_module (const char *fmt, ...)
@@ -78,6 +78,8 @@ cuda_module* cuda_module::s_current_module = nullptr;
 
 std::multimap<uint64_t, CORE_ADDR> cuda_module::s_kernel_entry_points;
 
+cuda_rangemap<cuda_module *> cuda_module::s_cuda_objfile_section_ranges;
+
 cuda_module::cuda_module (uint64_t module_id,
 			  CUDBGElfImageProperties properties,
 			  cuda_context* context,
@@ -107,12 +109,8 @@ cuda_module::cuda_module (uint64_t module_id,
 
 cuda_module::~cuda_module ()
 {
-  cuda_trace ("Module destroy context_id 0x%llx module_id 0x%llx total %llu modules",
-	      m_context->id (), m_id, m_context->get_modules ().size () + 1);
-
-  // If loaded, unload and unlink the objfile
-  if (m_loaded)
-    unload_objfile (true);
+  cuda_trace ("Module destroy context_id 0x%lx module_id 0x%lx total %lu modules",
+	      m_context->id (), m_id, m_context->get_modules ().size ());
 
   // Remove any still pending locations from the kernel entry map for this module
   s_kernel_entry_points.erase (m_id);
@@ -143,20 +141,6 @@ cuda_module::flush_disasm_caches ()
     }
 }
 
-bool
-cuda_module::contains_address (CORE_ADDR addr) const
-{
-  struct obj_section* osect = nullptr;
-  ALL_OBJFILE_OSECTIONS (m_objfile, osect)
-    {
-      auto section = osect->the_bfd_section;
-      if (section && (section->vma <= addr) && (addr < section->vma + section->size))
-        return true;
-    }
-
-  return false;
-}
-
 void
 cuda_module::add_kernel_entry (CORE_ADDR addr)
 {
@@ -194,6 +178,15 @@ cuda_module::auto_breakpoints_update_locations ()
     }
 }
 
+cuda_module*
+cuda_module::find_cuda_module_by_address (CORE_ADDR addr)
+{
+  auto ret = s_cuda_objfile_section_ranges.get (addr);
+  if (ret)
+    return *ret;
+  return nullptr;
+}
+
 void
 cuda_module::read_cubin (void *image)
 {
@@ -268,9 +261,31 @@ cuda_module::write_cubin (const void *image)
     error (_("Error: The device ELF file size is incorrect!"));
 }
 
+/* Helper function to add sections to the range map using a functor */
+template <typename Func>
+inline void
+cuda_module::foreach_cuda_objfile_section (Func func)
+{
+  for (auto osect : m_objfile->sections ())
+    {
+      asection *section = osect->the_bfd_section;
+      if (!section)
+	continue;
+      /* Skip sections that do not have code */
+      if (!(section->flags & SEC_CODE))
+	continue;
+      /* skip sections that are unrelocated */
+      if (section->vma == 0)
+	continue;
+      func (section);
+    }
+}
+
 // cuda_module::load_objfile() reads the ELF image file into symbol table.
 // Native debugging calls cuda_module::save_buffer() first. Remote debugging
 // only calls load() since the ELF image file has already fetched from server.
+// TODO: This function should be reconciled with the cuda_new_objfile observer
+//      in cuda-tdep.cu.
 void
 cuda_module::load_objfile ()
 {
@@ -295,12 +310,15 @@ cuda_module::load_objfile ()
 
   // Identify this gdb objfile as being cuda-specific
   m_objfile->cuda_objfile     = true;
-  m_objfile->m_cuda_module    = this;
 
   m_objfile->per_bfd->gdbarch = cuda_get_gdbarch ();
 
-  // CUDA - skip prologue - temporary
-  m_objfile->cuda_producer_is_open64 = cuda_producer_is_open64;
+
+  // Use the helper function to add sections
+  foreach_cuda_objfile_section ([this] (asection *section) {
+    // Add this section to the range map
+    s_cuda_objfile_section_ranges.add (section->vma, section->size, this);
+  });
 
   // cubin is now loaded
   m_loaded   = true;
@@ -322,11 +340,13 @@ cuda_module::load_objfile ()
   // This must happen after the reinit_frame_cache above.
   if (!m_objfile->compunit_symtabs)
     cuda_decode_line_table (m_objfile);
-  
+
   // Don't try to update/set breakpoints for corefiles
   if (target_has_execution ())
     {
-      cuda_resolve_breakpoints (0, this);
+      breakpoint_re_set ();
+      // Need to manually update any forced auto breakpoint location
+      // for this new objfile.
       auto_breakpoints_update_locations ();
     }
 
@@ -348,11 +368,16 @@ cuda_module::unload_objfile (bool unlink_file)
   // Mark this objfile as being discarded.
   m_objfile->discarding = 1;
 
+  // Use the helper function to add sections
+  foreach_cuda_objfile_section ([] (asection *section) {
+    /* Remove this section from the range map */
+    s_cuda_objfile_section_ranges.remove_range (section->vma);
+  });
+
   // Make sure that all its users will be cleaned up.
   clear_current_source_symtab_and_line ();
   clear_displays ();
-  cuda_reset_invalid_breakpoint_location_section (m_objfile);
-
+  
   // Unlink the ELF image in /tmp/cuda-dbg/ if requested
   if (unlink_file)
     {
@@ -366,16 +391,6 @@ cuda_module::unload_objfile (bool unlink_file)
 
   m_loaded = false;
   m_uses_abi = false;
-
-  // Don't try to update/set breakpoints for corefiles
-  if (target_has_execution ())
-    {
-      // Update the forced breakpoints to remove locations for this image.
-      cuda_auto_breakpoints_remove_locations (this);
-
-      // Remove any user set breakpoints for this image.
-      cuda_unresolve_breakpoints (this);
-    }
 }
 
 void
