@@ -1,6 +1,6 @@
 /*
  * NVIDIA CUDA Debugger CUDA-GDB
- * Copyright (C) 2007-2024 NVIDIA Corporation
+ * Copyright (C) 2007-2025 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -104,21 +104,31 @@ cuda_options_initialize_debug_cuda_prefix (void)
  */
 static std::string cuda_debug_trace_string;
 static uint32_t cuda_debug_trace_flags = 0;
-static struct {
+static struct
+{
   const char *name;
   cuda_trace_domain_t domain;
   const char *description;
 } cuda_debug_trace_names[] = {
-  {"general", CUDA_TRACE_GENERAL, "show/hide general debug trace of the internal CUDA-specific functions"},
-  {"disasm", CUDA_TRACE_DISASSEMBLER, "When enabled trace GPU disassembly operations"},
-  {"elf", CUDA_TRACE_ELF, "show/hide CUDA ELF file processing trace messages"},
-  {"event", CUDA_TRACE_EVENT, "show/hide CUDA event trace messages"},
-  {"breakpoint", CUDA_TRACE_BREAKPOINT, "show/hide CUDA-specific breakpoint handling trace messages"},
-  {"api", CUDA_TRACE_API, "show/hide CUDA Debugger API trace messages"},
-  {"siginfo", CUDA_TRACE_SIGINFO, "When enabled, update $_siginfo if the application is signalled by a CUDA exception"},
-  {"state", CUDA_TRACE_STATE, "show/hide CUDA state trace messages"},
-  {"decode", CUDA_TRACE_STATE_DECODE, "show/hide CUDA state device update decoding messages"},
-  {NULL, (cuda_trace_domain_t) 0, NULL},
+  { "api", CUDA_TRACE_API, "show/hide CUDA Debugger API trace messages" },
+  { "breakpoint", CUDA_TRACE_BREAKPOINT,
+    "show/hide CUDA-specific breakpoint handling trace messages" },
+  { "decode", CUDA_TRACE_STATE_DECODE,
+    "show/hide CUDA state device update decoding messages" },
+  { "disasm", CUDA_TRACE_DISASSEMBLER,
+    "show/hide debug trace of GPU disassembly operations" },
+  { "elf", CUDA_TRACE_ELF,
+    "show/hide CUDA ELF file processing trace messages" },
+  { "event", CUDA_TRACE_EVENT, "show/hide CUDA event trace messages" },
+  { "general", CUDA_TRACE_GENERAL,
+    "show/hide general debug trace of the internal CUDA-specific functions" },
+  { "python", CUDA_TRACE_PYTHON,
+    "show/hide debug trace of the internal CUDA-Python specific functions" },
+  { "siginfo", CUDA_TRACE_SIGINFO,
+    "When enabled, update $_siginfo if the application is signalled by a CUDA "
+    "exception" },
+  { "state", CUDA_TRACE_STATE, "show/hide CUDA state trace messages" },
+  { NULL, (cuda_trace_domain_t)0, NULL },
 };
 
 bool
@@ -354,6 +364,7 @@ cuda_set_debug_convenience_vars (const char *args, int from_tty,
   if (groups && (strcasecmp(groups,"all")==0 || atoi(groups)>0))
     {
       cuda_enable_convenience_variables_group (NULL, true);
+      cuda_update_convenience_variables ();
       xfree (groups);
       return;
     }
@@ -373,6 +384,7 @@ cuda_set_debug_convenience_vars (const char *args, int from_tty,
       if (!rc)
         printf_unfiltered (_("Unknown variable group name \"%s\"\n"), grp);
     }
+  cuda_update_convenience_variables ();
   xfree (groups);
 }
 
@@ -648,7 +660,14 @@ cuda_show_disassemble_from (struct ui_file *file, int from_tty,
 static void
 cuda_set_disassemble_from (const char *args, int from_tty, struct cmd_list_element *c)
 {
-  cuda_state::flush_disasm_caches ();
+  if (!target_has_execution ()
+      && cuda_disassemble_from == cuda_disassemble_from_device_memory)
+    {
+      cuda_disassemble_from = cuda_disassemble_from_elf_image;
+      warning (_ ("Cannot disassemble from %s with current target, switching "
+		  "back to %s"),
+	       cuda_disassemble_from_device_memory, cuda_disassemble_from);
+    }
 }
 
 static void
@@ -678,6 +697,15 @@ cuda_options_disassemble_from_elf_image (void)
   return cuda_disassemble_from == cuda_disassemble_from_elf_image;
 }
 
+void
+cuda_options_set_disassemble_from_elf_image (void)
+{
+  if (cuda_disassemble_from != cuda_disassemble_from_elf_image)
+    warning (_ ("cuda option `disassemble_from' was reset from %s to %s"),
+	     cuda_disassemble_from, cuda_disassemble_from_elf_image);
+  cuda_disassemble_from = cuda_disassemble_from_elf_image;
+}
+
 /*
  * set cuda disassemble_per
  */
