@@ -1,6 +1,6 @@
 /*
  * NVIDIA CUDA Debugger CUDA-GDB
- * Copyright (C) 2007-2024 NVIDIA Corporation
+ * Copyright (C) 2007-2025 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -214,7 +214,7 @@ cuda_remote_notification_analyze (ptid_t ptid, struct target_waitstatus *ws)
      the thread that triggered this code path.  */
   if (inferior_ptid == null_ptid)
     {
-      struct thread_info *tp = find_thread_ptid (current_inferior ()->process_target (), ptid);
+      struct thread_info *tp = current_inferior ()->process_target ()->find_thread (ptid);
       if (tp != nullptr)
 	trap_expected = tp->control.trap_expected;
     }
@@ -280,7 +280,7 @@ cuda_remote_update_per_warp_info_in_sm (uint32_t dev, uint32_t sm,
   num_warps = cuda_state::device_get_num_warps (dev);
 
   cuda_debugapi::read_valid_warps (dev, sm, &valid_warps_mask_s);
-  gdb_assert (cuda_api_eq_mask(&valid_warps_mask_s, valid_warps_mask_c));
+  gdb_assert (valid_warps_mask_s == valid_warps_mask_c);
 
   for (wp = 0; wp < num_warps; wp++)
     {
@@ -309,7 +309,7 @@ cuda_remote_update_per_warp_info_in_sm (uint32_t dev, uint32_t sm,
 
   cuda_api_warpmask valid_warps_mask_s;
   remote_callbacks.extract_bin ((gdb_byte *) &valid_warps_mask_s, sizeof (valid_warps_mask_s));
-  gdb_assert (cuda_api_eq_mask(&valid_warps_mask_s, valid_warps_mask_c));
+  gdb_assert (valid_warps_mask_s == valid_warps_mask_c);
 
   for (uint32_t wp = 0; wp < num_warps; wp++)
     {
@@ -555,10 +555,10 @@ cuda_remote_query_device_spec (uint32_t dev_id, uint32_t *num_sms,
   remote_callbacks.extract_bin ((gdb_byte *) &res, sizeof (res));
   if (res != CUDBG_SUCCESS)
     error (_("Error: Failed to read device specification (error=%u).\n"), res);
-  remote_callbacks.extract_bin ((gdb_byte *) num_sms, sizeof (num_sms));
-  remote_callbacks.extract_bin ((gdb_byte *) num_warps, sizeof (num_warps));
-  remote_callbacks.extract_bin ((gdb_byte *) num_lanes, sizeof (num_lanes));
-  remote_callbacks.extract_bin ((gdb_byte *) num_registers, sizeof (num_registers));
+  remote_callbacks.extract_bin ((gdb_byte *) num_sms, sizeof (*num_sms));
+  remote_callbacks.extract_bin ((gdb_byte *) num_warps, sizeof (*num_warps));
+  remote_callbacks.extract_bin ((gdb_byte *) num_lanes, sizeof (*num_lanes));
+  remote_callbacks.extract_bin ((gdb_byte *) num_registers, sizeof (*num_registers));
   *dev_type = remote_callbacks.extract_string ();
   *sm_type  = remote_callbacks.extract_string ();
 }
