@@ -1,6 +1,6 @@
 /*
  * NVIDIA CUDA Debugger CUDA-GDB
- * Copyright (C) 2007-2024 NVIDIA Corporation
+ * Copyright (C) 2007-2025 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -56,11 +56,12 @@
  */
 
 /* Raw value decoding */
-#define REGMAP_CLASS(x) (x >> 24)
-#define REGMAP_REG(x) (x & 0xffffff)
-#define REGMAP_OFST(x) (x & 0xffffff)
-#define REGMAP_SP_REG(x) ((x >> 16) & 0xff)
-#define REGMAP_SP_OFST(x) (x & 0xffff)
+#define REGMAP_CLASS(x)   ((x) >> 24)
+#define REGMAP_REG(x)     ((x) & 0x00ffffff)
+#define REGMAP_PRED(x)    ((x) & 0x07)
+#define REGMAP_OFST(x)    ((x) & 0x00ffffff)
+#define REGMAP_SP_REG(x)  (((x) >> 16) & 0xff)
+#define REGMAP_SP_OFST(x) ((x) & 0x0000ffff)
 
 /* Structure/global variables for storing search results*/
 #define REGMAP_ENTRIES_ALLOC 64
@@ -207,10 +208,9 @@ regmap_get_half_register (regmap_t regmap, uint32_t idx,
   gdb_assert (REGMAP_CLASS (regmap->output.raw_value[idx])
 	      == REG_CLASS_REG_HALF);
 
-  cuda_gdbarch_tdep *tdep = gdbarch_tdep<cuda_gdbarch_tdep> (cuda_get_gdbarch ());
   raw_register = REGMAP_REG (regmap->output.raw_value[idx]);
   *in_higher_16_bits = raw_register & 1;
-  return tdep->first_regnum + (raw_register / 2);
+  return raw_register / 2;
 }
 
 uint32_t
@@ -221,8 +221,18 @@ regmap_get_register (regmap_t regmap, uint32_t idx)
   gdb_assert (REGMAP_CLASS (regmap->output.raw_value[idx])
 	      == REG_CLASS_REG_FULL);
 
-  cuda_gdbarch_tdep *tdep = gdbarch_tdep<cuda_gdbarch_tdep> (cuda_get_gdbarch ());
-  return REGMAP_REG (regmap->output.raw_value[idx]) + tdep->first_regnum;
+  return REGMAP_REG (regmap->output.raw_value[idx]);
+}
+
+uint32_t
+regmap_get_predicate (regmap_t regmap, uint32_t idx)
+{
+  gdb_assert (regmap);
+  gdb_assert (idx < regmap->output.num_entries);
+  gdb_assert (REGMAP_CLASS (regmap->output.raw_value[idx])
+	      == REG_CLASS_REG_PRED);
+
+  return REGMAP_PRED (regmap->output.raw_value[idx]);
 }
 
 uint32_t
@@ -237,10 +247,9 @@ regmap_get_half_uregister (regmap_t regmap, uint32_t idx,
   gdb_assert (REGMAP_CLASS (regmap->output.raw_value[idx])
 	      == REG_CLASS_UREG_HALF);
 
-  cuda_gdbarch_tdep *tdep = gdbarch_tdep<cuda_gdbarch_tdep> (cuda_get_gdbarch ());
   raw_register = REGMAP_REG (regmap->output.raw_value[idx]);
   *in_higher_16_bits = raw_register & 1;
-  return tdep->first_uregnum + (raw_register / 2);
+  return raw_register / 2;
 }
 
 uint32_t
@@ -251,8 +260,18 @@ regmap_get_uregister (regmap_t regmap, uint32_t idx)
   gdb_assert (REGMAP_CLASS (regmap->output.raw_value[idx])
 	      == REG_CLASS_UREG_FULL);
 
-  cuda_gdbarch_tdep *tdep = gdbarch_tdep<cuda_gdbarch_tdep> (cuda_get_gdbarch ());
-  return REGMAP_REG (regmap->output.raw_value[idx]) + tdep->first_uregnum;
+  return REGMAP_REG (regmap->output.raw_value[idx]);
+}
+
+uint32_t
+regmap_get_upredicate (regmap_t regmap, uint32_t idx)
+{
+  gdb_assert (regmap);
+  gdb_assert (idx < regmap->output.num_entries);
+  gdb_assert (REGMAP_CLASS (regmap->output.raw_value[idx])
+	      == REG_CLASS_UREG_PRED);
+
+  return REGMAP_PRED (regmap->output.raw_value[idx]);
 }
 
 uint32_t
@@ -263,8 +282,7 @@ regmap_get_sp_register (regmap_t regmap, uint32_t idx)
   gdb_assert (REGMAP_CLASS (regmap->output.raw_value[idx])
 	      == REG_CLASS_LMEM_REG_OFFSET);
 
-  cuda_gdbarch_tdep *tdep = gdbarch_tdep<cuda_gdbarch_tdep> (cuda_get_gdbarch ());
-  return REGMAP_SP_REG (regmap->output.raw_value[idx]) + tdep->first_regnum;
+  return REGMAP_SP_REG (regmap->output.raw_value[idx]);
 }
 
 uint32_t
@@ -289,6 +307,15 @@ regmap_get_offset (regmap_t regmap, uint32_t idx)
   return REGMAP_OFST (regmap->output.raw_value[idx]);
 }
 
+uint32_t
+regmap_get_raw_value (regmap_t regmap, uint32_t idx)
+{
+  gdb_assert (regmap);
+  gdb_assert (idx < regmap->output.num_entries);
+
+  return regmap->output.raw_value[idx];
+}
+
 /*
  * Determine if the value indicated by this register map is readable and
  * writable.
@@ -500,7 +527,7 @@ regmap_read_elf_section (struct objfile *objfile, char **buffer,
     return;
 
   /* read the section */
-  if (bfd_bread (ptr, size, abfd) != size)
+  if (bfd_read (ptr, size, abfd) != size)
     {
       xfree (ptr);
       return;
@@ -514,8 +541,6 @@ regmap_read_elf_section (struct objfile *objfile, char **buffer,
 static void
 regmap_extend_liverange (regmap_func_t *func)
 {
-  regmap_map_t *map;
-  regmap_map_t *map_end = &func->map[func->maps_no];
   uint32_t end_max = 0;
   uint32_t min_entries = 0;
   uint32_t max_entries = 0;
@@ -528,7 +553,8 @@ regmap_extend_liverange (regmap_func_t *func)
   // Find the max end for this function
   // Build the map of register names to a vector of ranges
   // PERF: replace this with a lookup of the function's PC range instead
-  for (map = &func->map[0]; map < map_end; map++)
+  auto *map_end = &func->map[func->maps_no];
+  for (auto *map = &func->map[0]; map < map_end; map++)
     {
       reg_to_map[*(uint64_t *)map->rname].emplace_back (map);
 
