@@ -1,6 +1,6 @@
 /*
  * NVIDIA CUDA Debugger CUDA-GDB
- * Copyright (C) 2007-2024 NVIDIA Corporation
+ * Copyright (C) 2007-2025 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -37,10 +37,14 @@ uint32_t      regmap_get_sp_register    (regmap_t regmap, uint32_t idx);
 uint32_t      regmap_get_sp_offset      (regmap_t regmap, uint32_t idx);
 uint32_t      regmap_get_offset         (regmap_t regmap, uint32_t idx);
 uint32_t      regmap_get_half_register  (regmap_t regmap, uint32_t idx,
-                                         bool *in_higher_16_bits);
+					 bool *in_higher_16_bits);
+uint32_t      regmap_get_predicate      (regmap_t regmap, uint32_t idx);
 uint32_t      regmap_get_uregister      (regmap_t regmap, uint32_t idx);
 uint32_t      regmap_get_half_uregister (regmap_t regmap, uint32_t idx,
 					 bool *in_higher_16_bits);
+uint32_t      regmap_get_upredicate     (regmap_t regmap, uint32_t idx);
+uint32_t      regmap_get_raw_value      (regmap_t regmap, uint32_t idx);
+
 
 bool regmap_is_readable     (regmap_t regmap);
 bool regmap_is_writable     (regmap_t regmap);
