@@ -1,6 +1,6 @@
 /*
  * NVIDIA CUDA Debugger CUDA-GDB
- * Copyright (C) 2007-2024 NVIDIA Corporation
+ * Copyright (C) 2007-2025 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -97,20 +97,17 @@ cuda_special_register_has_matching_values (regmap_t regmap)
 bool
 cuda_special_register_p (regmap_t regmap)
 {
-  uint32_t num_regs;
-  CUDBGRegClass reg_class;
-
   gdb_assert (regmap);
 
   /* Special register is made of at least one register */
-  num_regs = regmap_get_num_entries (regmap);
+  const auto num_regs = regmap_get_num_entries (regmap);
   if (num_regs == 0)
     return false;
 
   /* Only 1 full register does not require using the special register framework */
   if (num_regs == 1)
     {
-      reg_class = regmap_get_class (regmap, 0);
+      const auto reg_class = regmap_get_class (regmap, 0);
       if (reg_class == REG_CLASS_REG_FULL || reg_class == REG_CLASS_UREG_FULL)
 	return false;
     }
@@ -140,7 +137,6 @@ cuda_special_register_read_entry (regmap_t regmap,
   gdb_assert (regmap_is_readable (regmap));
 
   const auto& c = cuda_current_focus::get ().physical ();
-  cuda_gdbarch_tdep *tdep = gdbarch_tdep<cuda_gdbarch_tdep> (cuda_get_gdbarch ());
 
   sz = sizeof *buf;
 
@@ -148,45 +144,59 @@ cuda_special_register_read_entry (regmap_t regmap,
   {
     case REG_CLASS_REG_FULL:
       regnum = regmap_get_register (regmap, entry_idx);
-      *buf = cuda_state::lane_get_register (c.dev (), c.sm (), c.wp (), c.ln (), regnum - tdep->first_regnum);
+      *buf = cuda_state::lane_get_register (c.dev (), c.sm (), c.wp (), c.ln (), regnum);
       break;
 
     case REG_CLASS_MEM_LOCAL:
       gdb_assert (!cuda_current_active_elf_image_uses_abi ());
       offset = regmap_get_offset (regmap, entry_idx);
-      cuda_debugapi::read_local_memory (c.dev (), c.sm (), c.wp (), c.ln (), offset, buf, sz);
+      if (!cuda_debugapi::read_local_memory (c.dev (), c.sm (), c.wp (), c.ln (),
+					     offset, buf, sz))
+	error ("%s: Could not read local memory address 0x%x size %u",
+	       __FUNCTION__, offset, sz);
       break;
 
     case REG_CLASS_LMEM_REG_OFFSET:
       gdb_assert (cuda_current_active_elf_image_uses_abi ());
       sp_regnum = regmap_get_sp_register (regmap, entry_idx);
       offset = regmap_get_sp_offset (regmap, entry_idx);
-      stack_addr = cuda_state::lane_get_register (c.dev (), c.sm (), c.wp (), c.ln (), sp_regnum - tdep->first_regnum);
-      cuda_debugapi::read_local_memory (c.dev (), c.sm (), c.wp (), c.ln (), stack_addr + offset, buf, sz);
+      stack_addr = cuda_state::lane_get_register (c.dev (), c.sm (), c.wp (), c.ln (), sp_regnum);
+      if (!cuda_debugapi::read_local_memory (c.dev (), c.sm (), c.wp (), c.ln (),
+					     stack_addr + offset, buf, sz))
+	error ("%s: Could not read local memory address 0x%x size %u",
+	       __FUNCTION__, stack_addr + offset, sz);
       break;
 
     case REG_CLASS_REG_HALF:
       regnum = regmap_get_half_register (regmap, entry_idx, &high);
-      tmp = cuda_state::lane_get_register (c.dev (), c.sm (), c.wp (), c.ln (), regnum - tdep->first_regnum);
+      tmp = cuda_state::lane_get_register (c.dev (), c.sm (), c.wp (), c.ln (), regnum);
       *buf = high ? tmp >> 16 : tmp & 0xffff;
       break;
 
+    case REG_CLASS_REG_PRED:
+      regnum = regmap_get_predicate (regmap, entry_idx);
+      *buf = cuda_state::lane_get_predicate (c.dev (), c.sm (), c.wp (), c.ln (), regnum);
+      break;
+
     case REG_CLASS_UREG_FULL:
       regnum = regmap_get_uregister (regmap, entry_idx);
-      *buf = cuda_state::warp_get_uregister (c.dev (), c.sm (), c.wp (), regnum - tdep->first_uregnum);
+      *buf = cuda_state::warp_get_uregister (c.dev (), c.sm (), c.wp (), regnum);
       break;
 
     case REG_CLASS_UREG_HALF:
       regnum = regmap_get_half_uregister (regmap, entry_idx, &high);
-      tmp = cuda_state::warp_get_uregister (c.dev (), c.sm (), c.wp (), regnum - tdep->first_uregnum);
+      tmp = cuda_state::warp_get_uregister (c.dev (), c.sm (), c.wp (), regnum);
       *buf = high ? tmp >> 16 : tmp & 0xffff;
       break;
 
-    case REG_CLASS_REG_CC:
-    case REG_CLASS_REG_PRED:
-    case REG_CLASS_REG_ADDR:
     case REG_CLASS_UREG_PRED:
-      error (_("CUDA Register Class 0x%x not supported yet.\n"),
+      regnum = regmap_get_upredicate (regmap, entry_idx);
+      *buf = cuda_state::warp_get_upredicate (c.dev (), c.sm (), c.wp (), regnum);
+      break;
+
+    case REG_CLASS_REG_CC:
+    case REG_CLASS_REG_ADDR:
+      error (_("CUDA Register Class 0x%x not supported yet."),
              regmap_get_class (regmap, entry_idx));
       break;
 
@@ -215,15 +225,12 @@ cuda_special_register_write_entry (regmap_t regmap,
 
   switch (regmap_get_class (regmap, entry_idx))
   {
-    case REG_CLASS_REG_FULL:
-      regnum = regmap_get_register (regmap, entry_idx);
-      cuda_state::lane_set_register (c.dev (), c.sm (), c.wp (), c.ln (), regnum, *buf);
-      break;
-
     case REG_CLASS_MEM_LOCAL:
       gdb_assert (!cuda_current_active_elf_image_uses_abi ());
       offset = regmap_get_offset (regmap, entry_idx);
-      cuda_debugapi::write_local_memory (c.dev (), c.sm (), c.wp (), c.ln (), offset, ptr, sz);
+      if (!cuda_debugapi::write_local_memory (c.dev (), c.sm (), c.wp (), c.ln (), offset, ptr, sz))
+	error ("%s: Could not write local memory address 0x%x size %u",
+	       __FUNCTION__, offset, sz);
       break;
 
     case REG_CLASS_LMEM_REG_OFFSET:
@@ -231,7 +238,15 @@ cuda_special_register_write_entry (regmap_t regmap,
       sp_regnum = regmap_get_sp_register (regmap, entry_idx);
       offset = regmap_get_sp_offset (regmap, entry_idx);
       stack_addr = cuda_state::lane_get_register (c.dev (), c.sm (), c.wp (), c.ln (), sp_regnum);
-      cuda_debugapi::write_local_memory (c.dev (), c.sm (), c.wp (), c.ln (), stack_addr + offset, ptr, sz);
+      if (!cuda_debugapi::write_local_memory (c.dev (), c.sm (), c.wp (), c.ln (),
+					      stack_addr + offset, ptr, sz))
+	error ("%s: Could not write local memory address 0x%x size %u",
+	       __FUNCTION__, stack_addr + offset, sz);
+      break;
+
+    case REG_CLASS_REG_FULL:
+      regnum = regmap_get_register (regmap, entry_idx);
+      cuda_state::lane_set_register (c.dev (), c.sm (), c.wp (), c.ln (), regnum, *buf);
       break;
 
     case REG_CLASS_REG_HALF:
@@ -244,10 +259,34 @@ cuda_special_register_write_entry (regmap_t regmap,
       cuda_state::lane_set_register (c.dev (), c.sm (), c.wp (), c.ln (), regnum, new_val);
       break;
 
-    case REG_CLASS_REG_CC:
     case REG_CLASS_REG_PRED:
+      regnum = regmap_get_predicate (regmap, entry_idx);
+      cuda_state::lane_set_predicate (c.dev (), c.sm (), c.wp (), c.ln (), regnum, *buf);
+      break;
+
+    case REG_CLASS_UREG_FULL:
+      regnum = regmap_get_uregister (regmap, entry_idx);
+      cuda_state::warp_set_uregister (c.dev (), c.sm (), c.wp (), regnum, *buf);
+      break;
+
+    case REG_CLASS_UREG_HALF:
+      regnum = regmap_get_half_uregister (regmap, entry_idx, &high);
+      old_val = cuda_state::warp_get_uregister (c.dev (), c.sm (), c.wp (), regnum);
+      if (high)
+        new_val = (*buf << 16) | (old_val & 0x0000ffff);
+      else
+        new_val = (old_val & 0xffff0000) | (*buf);
+      cuda_state::warp_set_uregister (c.dev (), c.sm (), c.wp (), regnum, new_val);
+      break;
+
+    case REG_CLASS_UREG_PRED:
+      regnum = regmap_get_upredicate (regmap, entry_idx);
+      cuda_state::warp_set_upredicate (c.dev (), c.sm (), c.wp (), regnum, *buf);
+      break;
+
+    case REG_CLASS_REG_CC:
     case REG_CLASS_REG_ADDR:
-      error (_("CUDA Register Class 0x%x not supported yet.\n"),
+      error (_("CUDA Register Class 0x%x not supported yet."),
              regmap_get_class (regmap, entry_idx));
       break;
 
@@ -259,18 +298,16 @@ cuda_special_register_write_entry (regmap_t regmap,
 void
 cuda_special_register_read (regmap_t regmap, gdb_byte *buf)
 {
-  uint32_t idx, i, *ptr;
-
   gdb_assert (regmap);
   gdb_assert (buf);
 
   if (!regmap_is_readable (regmap))
     error (_("Read request impossible. Insufficient debug information."));
 
-  for (i = 0; i < regmap_get_num_entries (regmap); ++i)
+  for (auto i = 0; i < regmap_get_num_entries (regmap); ++i)
     {
-      idx = regmap_get_location_index (regmap, i);
-      ptr = &((uint32_t*)buf)[idx];
+      const auto idx = regmap_get_location_index (regmap, i);
+      auto ptr = &((uint32_t*)buf)[idx];
       cuda_special_register_read_entry (regmap, i, ptr);
     }
 }
@@ -278,18 +315,16 @@ cuda_special_register_read (regmap_t regmap, gdb_byte *buf)
 void
 cuda_special_register_write (regmap_t regmap, const gdb_byte *buf)
 {
-  uint32_t idx, i, *ptr;
-
   gdb_assert (regmap);
   gdb_assert (buf);
 
   if (!regmap_is_writable (regmap))
     error (_("Write request impossible. Insufficient debug information."));
 
-  for (i = 0; i < regmap_get_num_entries (regmap); ++i)
+  for (auto i = 0; i < regmap_get_num_entries (regmap); ++i)
     {
-      idx = regmap_get_location_index (regmap, i);
-      ptr = &((uint32_t*)buf)[idx];
+      const auto idx = regmap_get_location_index (regmap, i);
+      auto ptr = &((uint32_t*)buf)[idx];
       cuda_special_register_write_entry (regmap, i, ptr);
     }
 }
@@ -309,6 +344,7 @@ cuda_special_register_to_value (regmap_t regmap, frame_info_ptr frame,
   gdb_assert (regmap_is_readable (regmap));
 
   const auto& c = cuda_current_focus::get ().physical ();
+  const auto tdep = gdbarch_tdep<cuda_gdbarch_tdep> (cuda_get_gdbarch ());
 
   for (i = 0; i < regmap_get_num_entries (regmap); ++i)
     {
@@ -316,47 +352,61 @@ cuda_special_register_to_value (regmap_t regmap, frame_info_ptr frame,
 
       switch (regmap_get_class (regmap, i))
         {
-          case REG_CLASS_REG_FULL:
-            regnum = regmap_get_register (regmap, i);
-            get_frame_register (frame, regnum, (gdb_byte*)&p[idx]);
-            break;
-
           case REG_CLASS_MEM_LOCAL:
             offset = regmap_get_offset (regmap, i);
-            cuda_debugapi::read_local_memory (c.dev (), c.sm (), c.wp (), c.ln (), offset,
-                                        (void*)&p[idx], sizeof (p[idx]));
+            if (!cuda_debugapi::read_local_memory (c.dev (), c.sm (), c.wp (), c.ln (),
+						   offset,
+						   (void*)&p[idx], sizeof (p[idx])))
+	      error ("%s: Could not read local memory address 0x%x size %u",
+		     __FUNCTION__, offset, (uint32_t)sizeof (p[idx]));
             break;
 
           case REG_CLASS_LMEM_REG_OFFSET:
             sp_regnum = regmap_get_sp_register (regmap, i);
             offset = regmap_get_sp_offset (regmap, i);
-            get_frame_register (frame, sp_regnum, (gdb_byte*)&stack_addr);
-            cuda_debugapi::read_local_memory (c.dev (), c.sm (), c.wp (), c.ln (), stack_addr + offset,
-                                        (void*)&p[idx], sizeof (p[idx]));
+            get_frame_register (frame, sp_regnum  + tdep->first_regnum, (gdb_byte*)&stack_addr);
+            if (!cuda_debugapi::read_local_memory (c.dev (), c.sm (), c.wp (), c.ln (),
+						   stack_addr + offset,
+						   (void*)&p[idx], sizeof (p[idx])))
+	      error ("%s: Could not read local memory address 0x%x size %u",
+		     __FUNCTION__, stack_addr + offset, (uint32_t)sizeof (p[idx]));
+            break;
+
+          case REG_CLASS_REG_FULL:
+            regnum = regmap_get_register (regmap, i);
+            get_frame_register (frame, regnum + tdep->first_regnum, (gdb_byte*)&p[idx]);
             break;
 
           case REG_CLASS_REG_HALF:
             regnum = regmap_get_half_register (regmap, i, &high);
-            get_frame_register (frame, regnum, (gdb_byte*)&val32);
+            get_frame_register (frame, regnum + tdep->first_regnum, (gdb_byte*)&val32);
             p[idx] = high ? val32 >> 16 : val32 & 0xffff;
             break;
 
+          case REG_CLASS_REG_PRED:
+            regnum = regmap_get_predicate (regmap, i);
+            get_frame_register (frame, regnum + tdep->first_pred_regnum, (gdb_byte*)&p[idx]);
+            break;
+
           case REG_CLASS_UREG_FULL:
             regnum = regmap_get_uregister (regmap, i);
-            get_frame_register (frame, regnum, (gdb_byte*)&p[idx]);
+            get_frame_register (frame, regnum + tdep->first_uregnum, (gdb_byte*)&p[idx]);
             break;
 
           case REG_CLASS_UREG_HALF:
             regnum = regmap_get_half_uregister (regmap, i, &high);
-            get_frame_register (frame, regnum, (gdb_byte*)&val32);
+            get_frame_register (frame, regnum + tdep->first_uregnum, (gdb_byte*)&val32);
             p[idx] = high ? val32 >> 16 : val32 & 0xffff;
             break;
 
-          case REG_CLASS_REG_CC:
-          case REG_CLASS_REG_PRED:
-          case REG_CLASS_REG_ADDR:
           case REG_CLASS_UREG_PRED:
-            error (_("CUDA Register Class 0x%x not supported yet.\n"),
+            regnum = regmap_get_upredicate (regmap, i);
+            get_frame_register (frame, regnum + tdep->first_upred_regnum, (gdb_byte*)&p[idx]);
+            break;
+
+          case REG_CLASS_REG_CC:
+          case REG_CLASS_REG_ADDR:
+            error (_("CUDA Register Class 0x%x not supported yet."),
                    regmap_get_class (regmap, i));
             break;
 
@@ -383,6 +433,7 @@ cuda_value_to_special_register (regmap_t regmap, frame_info_ptr frame,
     error (_("Write request impossible. Insufficient debug information."));
 
   const auto& c = cuda_current_focus::get ().physical ();
+  const auto tdep = gdbarch_tdep<cuda_gdbarch_tdep> (cuda_get_gdbarch ());
 
   for (i = 0; i < regmap_get_num_entries (regmap); ++i)
     {
@@ -390,51 +441,65 @@ cuda_value_to_special_register (regmap_t regmap, frame_info_ptr frame,
 
       switch (regmap_get_class (regmap, i))
         {
-          case REG_CLASS_REG_FULL:
-            regnum = regmap_get_register (regmap, i);
-            put_frame_register (frame, regnum, (gdb_byte*)&p[idx]);
-            break;
-
           case REG_CLASS_MEM_LOCAL:
             offset = regmap_get_offset (regmap, i);
-            cuda_debugapi::write_local_memory (c.dev (), c.sm (), c.wp (), c.ln (), offset,
-                                        (void*)&p[idx], sizeof (p[idx]));
+            if (!cuda_debugapi::write_local_memory (c.dev (), c.sm (), c.wp (), c.ln (),
+						    offset,
+						    (void*)&p[idx], sizeof (p[idx])))
+	      error ("%s: Could not write local memory address 0x%x size %u",
+		     __FUNCTION__, offset, (uint32_t)sizeof (p[idx]));
             break;
 
           case REG_CLASS_LMEM_REG_OFFSET:
             sp_regnum = regmap_get_sp_register (regmap, i);
             offset = regmap_get_sp_offset (regmap, i);
-            get_frame_register (frame, sp_regnum, (gdb_byte*)&stack_addr);
-            cuda_debugapi::write_local_memory (c.dev (), c.sm (), c.wp (), c.ln (), stack_addr + offset,
-                                         (void*)&p[idx], sizeof (p[idx]));
+            get_frame_register (frame, sp_regnum + tdep->first_regnum, (gdb_byte*)&stack_addr);
+            if (!cuda_debugapi::write_local_memory (c.dev (), c.sm (), c.wp (), c.ln (),
+						    stack_addr + offset,
+						    (void*)&p[idx], sizeof (p[idx])))
+	      error ("%s: Could not write local memory address 0x%x size %u",
+		     __FUNCTION__, stack_addr + offset, (uint32_t)sizeof (p[idx]));
+            break;
+
+          case REG_CLASS_REG_FULL:
+            regnum = regmap_get_register (regmap, i);
+            put_frame_register (frame, regnum + tdep->first_regnum, (gdb_byte*)&p[idx]);
             break;
 
           case REG_CLASS_REG_HALF:
             regnum = regmap_get_half_register (regmap, i, &high);
-            get_frame_register (frame, regnum, (gdb_byte*)&val32);
+            get_frame_register (frame, regnum + tdep->first_regnum, (gdb_byte*)&val32);
             val32 = high ? (val32 & 0xffff)     | (p[idx] << 16)
                          : (val32 & 0xffff0000) | (p[idx] & 0xffff);
-            put_frame_register (frame, regnum, (gdb_byte*)&val32);
+            put_frame_register (frame, regnum + tdep->first_regnum, (gdb_byte*)&val32);
+            break;
+
+          case REG_CLASS_REG_PRED:
+            regnum = regmap_get_predicate (regmap, i);
+            put_frame_register (frame, regnum + tdep->first_pred_regnum, (gdb_byte*)&p[idx]);
             break;
 
           case REG_CLASS_UREG_FULL:
             regnum = regmap_get_uregister (regmap, i);
-            put_frame_register (frame, regnum, (gdb_byte*)&p[idx]);
+            put_frame_register (frame, regnum + tdep->first_uregnum, (gdb_byte*)&p[idx]);
             break;
 
           case REG_CLASS_UREG_HALF:
             regnum = regmap_get_half_uregister (regmap, i, &high);
-            get_frame_register (frame, regnum, (gdb_byte*)&val32);
+            get_frame_register (frame, regnum + tdep->first_uregnum, (gdb_byte*)&val32);
             val32 = high ? (val32 & 0xffff)     | (p[idx] << 16)
                          : (val32 & 0xffff0000) | (p[idx] & 0xffff);
-            put_frame_register (frame, regnum, (gdb_byte*)&val32);
+            put_frame_register (frame, regnum + tdep->first_uregnum, (gdb_byte*)&val32);
+            break;
+
+          case REG_CLASS_UREG_PRED:
+            regnum = regmap_get_upredicate (regmap, i);
+            put_frame_register (frame, regnum + tdep->first_upred_regnum, (gdb_byte*)&p[idx]);
             break;
 
           case REG_CLASS_REG_CC:
-          case REG_CLASS_REG_PRED:
           case REG_CLASS_REG_ADDR:
-	  case REG_CLASS_UREG_PRED:
-            error (_("CUDA Register Class 0x%x not supported yet.\n"),
+            error (_("CUDA Register Class 0x%x not supported yet."),
                    regmap_get_class (regmap, i));
             break;
 
@@ -448,60 +513,77 @@ void
 cuda_special_register_name (regmap_t regmap, char *buf, const int size)
 {
   uint32_t sp_regnum, offset, regnum;
-  int d, i;
   bool high;
 
   gdb_assert (regmap);
   gdb_assert (buf);
 
-  buf[0] = '\0';
+  // Collect the register indices for each location index so
+  // that we can print them in the correct order.
+  // location_index is limited to 0-3, so an array size of 4 is sufficient.
+  uint32_t regs[4] = { 0 };
+  for (auto i = 0; i < regmap_get_num_entries (regmap); ++i)
+    {
+      const auto idx = regmap_get_location_index (regmap, i);
+      gdb_assert (idx < 4);
+      regs[idx] = i;
+    }
 
-  d = 0;
-  for (i = 0; i < regmap_get_num_entries (regmap); ++i)
+  buf[0] = '\0';
+  int d = 0;
+  for (auto i = 0; i < regmap_get_num_entries (regmap); ++i)
     {
       if (i > 0)
         d += snprintf (buf + d, size - 1 - d, "/$");
 
-      switch (regmap_get_class (regmap, i))
+      switch (regmap_get_class (regmap, regs[i]))
         {
-          case REG_CLASS_REG_FULL:
-            regnum = regmap_get_register (regmap, i);
-            d += snprintf (buf + d, size - 1 - d, "R%d", regnum);
-            break;
-
           case REG_CLASS_MEM_LOCAL:
-            offset = regmap_get_offset (regmap, i);
+            offset = regmap_get_offset (regmap, regs[i]);
             d += snprintf (buf + d, size - 1 - d, "(spilled @ 0x%x)", offset);
             break;
 
           case REG_CLASS_LMEM_REG_OFFSET:
-            sp_regnum = regmap_get_sp_register (regmap, i);
-            offset = regmap_get_sp_offset (regmap, i);
+            sp_regnum = regmap_get_sp_register (regmap, regs[i]);
+            offset = regmap_get_sp_offset (regmap, regs[i]);
             d += snprintf (buf + d, size - 1 - d, "(spilled @ [R%d]+0x%x)",
-                           sp_regnum,  offset);
+                           sp_regnum, offset);
+            break;
+
+          case REG_CLASS_REG_FULL:
+            regnum = regmap_get_register (regmap, regs[i]);
+            d += snprintf (buf + d, size - 1 - d, "R%d", regnum);
             break;
 
           case REG_CLASS_REG_HALF:
-            regnum = regmap_get_half_register (regmap, i, &high);
+            regnum = regmap_get_half_register (regmap, regs[i], &high);
             d += snprintf (buf + d, size - 1 - d, "R%d.%s", regnum, high ? "hi" : "lo");
             break;
 
+          case REG_CLASS_REG_PRED:
+            regnum = regmap_get_predicate (regmap, regs[i]);
+            d += snprintf (buf + d, size - 1 - d, "P%d", regnum);
+            break;
+
           case REG_CLASS_UREG_FULL:
-            regnum = regmap_get_uregister (regmap, i);
+            regnum = regmap_get_uregister (regmap, regs[i]);
             d += snprintf (buf + d, size - 1 - d, "UR%d", regnum);
             break;
 
           case REG_CLASS_UREG_HALF:
-            regnum = regmap_get_half_uregister (regmap, i, &high);
+            regnum = regmap_get_half_uregister (regmap, regs[i], &high);
             d += snprintf (buf + d, size - 1 - d, "UR%d.%s", regnum, high ? "hi" : "lo");
             break;
 
+          case REG_CLASS_UREG_PRED:
+            regnum = regmap_get_upredicate (regmap, regs[i]);
+            d += snprintf (buf + d, size - 1 - d, "UP%d", regnum);
+            break;
+
           case REG_CLASS_REG_CC:
-          case REG_CLASS_REG_PRED:
           case REG_CLASS_REG_ADDR:
-	  case REG_CLASS_UREG_PRED:
-            error (_("CUDA Register Class 0x%x not supported yet.\n"),
-                   regmap_get_class (regmap, i));
+            error (_("CUDA Register Class 0x%x not supported yet."),
+                   regmap_get_class (regmap, regs[i]));
             break;
 
           default:
