@@ -1,6 +1,6 @@
 /*
  * NVIDIA CUDA Debugger CUDA-GDB
- * Copyright (C) 2007-2024 NVIDIA Corporation
+ * Copyright (C) 2007-2025 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -143,11 +143,29 @@ cuda_state::reset (void)
   unknown_lane_attribute_warning = 0;
 
   // Delete the contexts and modules through the use of unique pointers
-  m_instance.set_current_context (nullptr);
-  m_instance.m_context_map.clear ();
+
+  // Modules belong to contexts, so delete them first
+  CUDA_STATE_TRACE ("Unloading remaining objfiles");
+  for (auto &iter : m_instance.m_module_map)
+    {
+      auto module = iter.second.get ();
+      module->context ()->remove_module (module);
+      module->unload_objfile (true);
+    }
+
+  CUDA_STATE_TRACE ("Destroying module map");
   m_instance.m_module_map.clear ();
 
+  // Delete the kernel map (and the associated kernel objects)
+  CUDA_STATE_TRACE ("Destroying kernel map");
+  m_instance.m_kernel_map.clear ();
+  m_instance.m_next_kernel_id = 0;
+
+  CUDA_STATE_TRACE ("Destroying context map");
+  m_instance.m_context_map.clear ();
+
   // Automatically deletes all device/sm/warp/lane objects
+  CUDA_STATE_TRACE ("Destroying Device/SM/Warp/Lane objects");
   m_instance.m_devs.clear ();
 }
 
@@ -222,8 +240,6 @@ cuda_state::destroy_context (uint64_t context_id)
   auto context = find_context_by_id (context_id);
   cuda_trace ("Context destroy device %u context 0x%llx", context->dev_id (), context_id);
 
-  if (context == current_context ())
-    set_current_context (nullptr);
 
   m_instance.m_context_map.erase (context_id);
 }
@@ -246,6 +262,14 @@ cuda_state::create_module (uint64_t module_id,
   CUDA_STATE_TRACE ("Module create module_id 0x%llx context_id 0x%llx size %llu",
 		    module_id, context_id, elf_image_size);
 
+  /* Sanity - Classic stack at times will try to load the same module multiple times.
+     To deal with this, destroy the previous module first. */
+  if (find_module_by_id (module_id))
+    {
+      CUDA_STATE_TRACE ("Module 0x%llx already loaded, destroying", module_id);
+      destroy_module (module_id);
+    }
+
   auto context = find_context_by_id (context_id);
   gdb_assert (context);
   
@@ -272,36 +296,35 @@ cuda_state::destroy_module (uint64_t module_id)
   auto module = find_module_by_id (module_id);
   gdb_assert (module);
   
+  // Remove the module from the context map
   module->context ()->remove_module (module);
+
+  // Unload and request destruction of the objfile
+  module->unload_objfile (true);
+
+  // Remove the module from the module map, destroying it
   m_instance.m_module_map.erase (module->id ());
 }
 
 cuda_module*
 cuda_state::find_module_by_id (uint64_t module_id)
 {
-  CUDA_STATE_TRACE ("find_module_by_id 0x%llx", module_id);
-
   auto iter = m_instance.m_module_map.find (module_id);
   if (iter != m_instance.m_module_map.end ())
-    return iter->second.get ();
+    {
+      auto module = iter->second.get ();
+      CUDA_STATE_TRACE ("Module 0x%llx found %p", module_id, module);
+      return module;
+    }
 
-  gdb_assert (false);
+  CUDA_STATE_TRACE ("Module 0x%llx not found", module_id);
   return nullptr;
 }
 
 cuda_module*
 cuda_state::find_module_by_address (CORE_ADDR addr)
 {
-  for (auto& iter : m_instance.m_module_map)
-    if (iter.second.get ()->contains_address (addr))
-      {
-	CUDA_STATE_TRACE ("find_module_by_address 0x%llx module 0x%llx",
-			  addr, iter.second.get ()->id ());
-	return iter.second.get ();
-      }
-
-  CUDA_STATE_TRACE ("find_module_by_address 0x%llx - not found", addr);
-  return nullptr;
+  return cuda_module::find_cuda_module_by_address (addr);
 }
 
 bool
@@ -317,8 +340,9 @@ void
 cuda_state::device_invalidate_kernels (uint32_t dev_id)
 {
   CUDA_STATE_TRACE_DEV (device (dev_id), "");
-
-  kernels_invalidate (dev_id);
+  for (const auto& iter : kernels ())
+    if (iter.second.get ()->dev_id () == dev_id)
+      iter.second.get ()->invalidate ();
 }
 
 uint32_t
@@ -329,33 +353,13 @@ cuda_state::get_num_present_kernels (void)
   if (!cuda_initialized)
     return 0;
 
-  for (auto kernel = kernels_get_first_kernel (); kernel;
-       kernel = kernels_get_next_kernel (kernel))
-    if (kernel_is_present (kernel))
+  for (const auto& iter : kernels ())
+    if (iter.second.get ()->present ())
       ++num_present_kernel;
 
   return num_present_kernel;
 }
 
-void
-cuda_state::resolve_breakpoints (int bp_number_from)
-{
-  CUDA_STATE_TRACE ("");
-
-  for (auto& iter : m_instance.m_module_map)
-    cuda_resolve_breakpoints (bp_number_from, iter.second.get ());
-}
-
-void
-cuda_state::cleanup_breakpoints (void)
-{
-  CUDA_STATE_TRACE ("");
-
-  // Passing in nullptr unresolves all CUDA breakpoints
-  // across all elf_image objects in a single call
-  cuda_unresolve_breakpoints (nullptr);
-}
-
 bool
 cuda_state::is_context_active (cuda_context* context)
 {
@@ -405,6 +409,204 @@ cuda_state::broken (cuda_coords &coords)
   return true;
 }
 
+cuda_kernel*
+cuda_state::find_kernel_by_grid_id (uint32_t dev_id, uint64_t grid_id)
+{
+  for (const auto& iter : m_instance.m_kernel_map)
+    if ((iter.second->dev_id () == dev_id) && (iter.second->grid_id () == grid_id))
+      {
+	CUDA_STATE_TRACE_DEV (device (dev_id),
+			      "kernel %lu found for grid_id %ld",
+			      iter.second.get ()->id (), (int64_t)grid_id);
+	return iter.second.get ();
+      }
+
+  CUDA_STATE_TRACE_DEV (device (dev_id),
+			"kernel not found for grid_id %ld",
+			(int64_t)grid_id);
+  return nullptr;
+}
+
+cuda_kernel*
+cuda_state::find_kernel_by_kernel_id (uint64_t kernel_id)
+{
+  auto iter = m_instance.m_kernel_map.find (kernel_id);
+  if (iter != m_instance.m_kernel_map.end ())
+      {
+	CUDA_STATE_TRACE_DEV (device (iter->second.get ()->dev_id ()),
+			      "kernel %lu found by kernel id", kernel_id);
+	return iter->second.get ();
+      }
+
+  CUDA_STATE_TRACE ("kernel %lu not found", kernel_id);
+  return nullptr;
+}
+
+cuda_kernel*
+cuda_state::create_kernel (uint32_t dev_id, uint64_t grid_id)
+{
+  const auto grid_info = cuda_state::device_get_grid_info (dev_id, grid_id);
+  
+  auto context = cuda_state::find_context_by_id (grid_info.context);
+  if (!context)
+    error ("Could not find CUDA context for context_id 0x%lx", grid_info.context);
+
+  const auto module = cuda_state::find_module_by_id (grid_info.module);
+  if (!module)
+    error ("Could not find CUDA module for context_id 0x%lx module_id 0x%lx",
+	   grid_info.context, grid_info.module);
+
+  const auto module_id = grid_info.module;
+
+  const auto virt_code_base = grid_info.functionEntry;
+  const auto grid_dim = grid_info.gridDim;
+  const auto block_dim = grid_info.blockDim;
+  const auto cluster_dim = grid_info.clusterDim;
+
+  const auto type = grid_info.type;
+  const auto origin = grid_info.origin;
+  const auto parent_grid_id = grid_info.parentGridId;
+
+  return create_kernel (dev_id, grid_id, virt_code_base, module_id,
+			grid_dim, block_dim, cluster_dim,
+			type, origin, parent_grid_id);
+}
+
+cuda_kernel*
+cuda_state::create_kernel (uint32_t dev_id, uint64_t grid_id,
+			   uint64_t virt_code_base, uint64_t module_id,
+			   CuDim3 grid_dim, CuDim3 block_dim, CuDim3 cluster_dim,
+			   CUDBGKernelType type, CUDBGKernelOrigin origin, uint64_t parent_grid_id)
+{
+  // First see if there's a parent kernel for this grid.
+  // If not, create one before proceeding
+  if (parent_grid_id && !cuda_state::find_kernel_by_grid_id (dev_id, parent_grid_id))
+    cuda_state::add_parent_kernel (dev_id, grid_id);
+
+  auto module = find_module_by_id (module_id);
+  gdb_assert (module);
+
+  auto kernel_id = m_instance.m_next_kernel_id++;
+  auto kernel = std::make_unique<cuda_kernel> (kernel_id, dev_id, grid_id,
+					       virt_code_base, module,
+					       grid_dim, block_dim, cluster_dim,
+					       type, origin, parent_grid_id);
+
+  if (kernel->should_print_kernel_event ())
+    printf_unfiltered (
+        _ ("[Launch of CUDA Kernel %lu (%s%s) on Device %u, level %u]\n"),
+        kernel->id (),
+	kernel->name ().c_str (), kernel->dimensions ().c_str (),
+	kernel->dev_id (), kernel->depth ());
+
+  m_instance.m_kernel_map.emplace (kernel_id, std::move (kernel));
+
+  return m_instance.m_kernel_map[kernel_id].get ();
+}
+
+cuda_kernel*
+cuda_state::add_parent_kernel (uint32_t dev_id, uint64_t grid_id)
+{
+  if (!grid_id)
+    return nullptr;
+
+  CUDBGGridStatus grid_status;
+  cuda_debugapi::get_grid_status (dev_id, grid_id, &grid_status);
+  if (grid_status == CUDBG_GRID_STATUS_INVALID)
+    return nullptr;
+
+  CUDBGGridInfo grid_info;
+  cuda_debugapi::get_grid_info (dev_id, grid_id, &grid_info);
+
+  CUDBGGridStatus parent_grid_status;
+  cuda_debugapi::get_grid_status (dev_id, grid_info.parentGridId,
+                                  &parent_grid_status);
+  if (parent_grid_status == CUDBG_GRID_STATUS_INVALID)
+    return nullptr;
+
+  CUDBGGridInfo parent_grid_info;
+  cuda_debugapi::get_grid_info (dev_id, grid_info.parentGridId,
+                                &parent_grid_info);
+  return cuda_state::create_kernel (dev_id, parent_grid_info.gridId64);
+}
+
+void
+cuda_state::destroy_kernel (uint32_t dev_id, uint64_t grid_id)
+{
+  auto kernel = find_kernel_by_grid_id (dev_id, grid_id);
+  if (!kernel)
+    CUDA_STATE_TRACE_DEV (device (dev_id), "kernel not found for grid_id %ld", (int64_t)grid_id);
+  else
+    destroy_kernel (kernel);
+}
+
+void
+cuda_state::destroy_kernel (cuda_kernel* kernel)
+{
+  gdb_assert (kernel);
+
+  CUDA_STATE_TRACE_DEV (device (kernel->dev_id ()), "kernel %lu dev_id %u grid_id %ld",
+			kernel->id (), kernel->dev_id (), (int64_t)kernel->grid_id ());
+
+  if (kernel->should_print_kernel_event ())
+    printf_unfiltered (_ ("[Termination of CUDA Kernel %lu (%s%s) on Device %u, level %u]\n"),
+                       kernel->id (), kernel->name ().c_str (),
+                       kernel->dimensions ().c_str (), kernel->dev_id (), kernel->depth ());
+
+  m_instance.m_kernel_map.erase (kernel->id ());
+}
+
+void
+cuda_state::invalidate_kernels (uint32_t dev_id)
+{
+  for (const auto& iter : m_instance.m_kernel_map)
+    if (iter.second->dev_id () == dev_id)
+      iter.second->invalidate ();
+}
+
+void
+cuda_state::invalidate_kernels (cuda_module* module)
+{
+  for (const auto& iter : m_instance.m_kernel_map)
+    if (iter.second->module () == module)
+      iter.second->invalidate ();
+}
+
+void
+cuda_state::update_kernel_args (void)
+{
+  for (const auto& iter : m_instance.m_kernel_map)
+    if (iter.second->present ())
+      iter.second->populate_args ();
+}
+
+void
+cuda_state::update_kernels_terminated (void)
+{
+  /* Make sure we have up-to-date information about running kernels.
+   * TODO: This feels like we are relying on a side-effect by creating an
+   * iterator. */
+  cuda_coord_set<cuda_coord_set_type::kernels, select_valid> coord{
+    cuda_coords::wild ()
+  };
+
+  // rediscover the kernels currently running on the hardware
+  std::vector<cuda_kernel*> kernel_destroy_list;
+  for (const auto& iter : m_instance.m_kernel_map)
+    {
+      // terminate the kernels that we had seen running at some point
+      // but are not here on the hardware anymore. If there is any child kernel
+      // still present, keep the data available.
+      auto kernel = iter.second.get ();
+      if (kernel->present ())
+	kernel->launched (true);
+      else if (kernel->launched ())
+        kernel_destroy_list.push_back (kernel);
+    }
+  for (auto kernel : kernel_destroy_list)
+    destroy_kernel (kernel);
+}
+
 /******************************************************************************
  *
  *				    Device
@@ -517,7 +719,7 @@ cuda_device::invalidate (bool quietly, bool recurse)
   m_sm_exception_mask_p = false;
 
   // Invalidate any kernels on the device
-  kernels_invalidate (dev_idx ());
+  cuda_state::invalidate_kernels (dev_idx ());
 
   // Quietly invalidate the SMs/Warps/Lanes
   if (recurse)
@@ -671,9 +873,8 @@ cuda_device::get_num_kernels ()
   gdb_assert (valid ());
 
   uint32_t num_kernels = 0;
-  for (auto kernel = kernels_get_first_kernel (); kernel;
-       kernel = kernels_get_next_kernel (kernel))
-    if (kernel_get_dev_id (kernel) == dev_idx ())
+  for (auto& iter : cuda_state::kernels ())
+    if (iter.second->dev_id () == dev_idx ())
       ++num_kernels;
 
   return num_kernels;
@@ -716,18 +917,14 @@ cuda_device::get_active_sms_mask ()
   return m_sm_active_mask;
 }
 
-kernel_t
+cuda_kernel*
 cuda_device::get_kernel (uint64_t grid_id)
 {
   gdb_assert (grid_id != 0);
 
-  auto kernel = kernels_find_kernel_by_grid_id (dev_idx (), grid_id);
+  auto kernel = cuda_state::find_kernel_by_grid_id (dev_idx (), grid_id);
   if (!kernel)
-    {
-      create_kernel (grid_id);
-      kernel = kernels_find_kernel_by_grid_id (dev_idx (), grid_id);
-      gdb_assert (kernel);
-    }
+    kernel = cuda_state::create_kernel (dev_idx (), grid_id);
 
   return kernel;
 }
@@ -787,17 +984,6 @@ cuda_device::get_grid_info (uint64_t grid_id)
   return m_grid_info[grid_id];
 }
 
-void
-cuda_device::create_kernel (uint64_t grid_id)
-{
-  const auto &grid_info = get_grid_info (grid_id);
-
-  kernels_start_kernel (
-      dev_idx (), grid_id, grid_info.functionEntry, grid_info.context,
-      grid_info.module, grid_info.gridDim, grid_info.blockDim, grid_info.type,
-      grid_info.parentGridId, grid_info.origin, true, grid_info.clusterDim);
-}
-
 void
 cuda_device::suspend ()
 {
@@ -1116,12 +1302,15 @@ cuda_sm::single_step_warp (uint32_t wp_id, uint32_t lane_id_hint,
 			   wp_id);
   else
     {
+      const auto valid_lanes_mask = warp (wp_id)->get_valid_lanes_mask ();
       const auto active_lanes_mask = warp (wp_id)->get_active_lanes_mask ();
       CUDA_STATE_TRACE_WARP (
 	  warp (wp_id),
 	  "Stepped warp mask %" WARP_MASK_FORMAT
-	  " active lanes 0x%08x active pc 0x%llx",
-	  cuda_api_mask_string (single_stepped_warp_mask), active_lanes_mask,
+	  " lanes valid 0x%08x active 0x%08x active pc 0x%llx",
+	  cuda_api_mask_string (single_stepped_warp_mask),
+	  valid_lanes_mask,
+	  active_lanes_mask,
 	  active_lanes_mask ? warp (wp_id)->get_active_pc () : 0);
     }
 
@@ -1353,7 +1542,11 @@ cuda_warp::invalidate (bool quietly, bool recurse)
   m_registers_allocated = 0;
   // Amount of shared memory currently allocated
   m_shared_mem_size = 0;
-  
+
+  // Reset the warp cbu state
+  m_cbu_state_p = false;
+  m_cbu_state = { 0 };
+
   // Quietly invalidate the lanes
   if (recurse)
     for (auto &ln : m_lanes)
@@ -1460,7 +1653,7 @@ cuda_warp::get_cluster_dim ()
   return m_cluster_dim;
 }
 
-kernel_t
+cuda_kernel*
 cuda_warp::get_kernel ()
 {
   if (!m_kernel)
@@ -1538,12 +1731,11 @@ cuda_warp::has_cluster_exception_target_block_idx ()
   return m_cluster_exception_target_block_idx_available;
 }
 
-CuDim3
+const CuDim3&
 cuda_warp::get_cluster_exception_target_block_idx ()
 {
-  if (has_cluster_exception_target_block_idx ())
-    return m_cluster_exception_target_block_idx;
-  gdb_assert (0);
+  gdb_assert (has_cluster_exception_target_block_idx ());
+  return m_cluster_exception_target_block_idx;
 }
 
 void
@@ -1562,6 +1754,27 @@ cuda_warp::update_warp_resources ()
     }
 }
 
+void
+cuda_warp::update_cbu_state ()
+{
+  if (!m_cbu_state_p)
+    {
+      constexpr uint32_t num_warp_states = 1;
+      const cuda_api_warpmask warp_mask{ 0x1ull << warp_idx () };
+      cuda_debugapi::get_cbu_warp_state (dev_idx (), sm_idx (), warp_mask,
+					 &m_cbu_state, num_warp_states);
+
+      m_cbu_state_p = true;
+    }
+}
+
+const CUDBGCbuWarpState &
+cuda_warp::get_cbu_state ()
+{
+  update_cbu_state ();
+  return m_cbu_state;
+}
+
 uint32_t
 cuda_warp::registers_allocated ()
 {
@@ -1745,18 +1958,6 @@ cuda_lane::invalidate (bool quietly)
   set_timestamp (0);
 }
 
-bool
-cuda_lane::active ()
-{
-  return warp ()->lane_active (lane_idx ());
-}
-
-bool
-cuda_lane::divergent ()
-{
-  return warp ()->lane_divergent (lane_idx ());
-}
-
 uint64_t
 cuda_lane::get_pc ()
 {
@@ -1803,6 +2004,8 @@ cuda_lane::get_register (uint32_t regno)
 
   gdb_assert (m_registers_p[regno]);
 
+  CUDA_STATE_TRACE_LANE (this, "R%u = 0x%08x", regno, m_registers[regno]);
+
   return m_registers[regno];
 }
 
@@ -2113,6 +2316,15 @@ cuda_warp::decode (const CUDBGDeviceInfoSizes &info_sizes,
   m_cluster_dim = CuDim3{ 0, 0, 0 };
   m_cluster_dim_p = false;
 
+  // Clear CBU state
+  m_cbu_state_p = false;
+  m_cbu_state = { 0 };
+
+  // Clear warp resources
+  m_shared_mem_size = 0;
+  m_registers_allocated = 0;
+  m_warp_resources_p = false;
+
   // Default to no exception
   CUDBGException_t exception = CUDBG_EXCEPTION_NONE;
 
@@ -2274,7 +2486,7 @@ cuda_sm::decode (const CUDBGDeviceInfoSizes &info_sizes,
   // The ((1ULL << n) - 1) trick for generating the mask doesn't work on
   // devices with 64 warps, so special case it.
   const auto n_warps = device ()->get_num_warps ();
-  cuda_api_warpmask_t updated_warp_mask
+  cuda_api_warpmask updated_warp_mask
       = { (n_warps == 8 * sizeof (uint64_t)) ? ~0ULL
 					     : ((1ULL << n_warps) - 1) };
 
