@@ -1,6 +1,6 @@
 /*
  * NVIDIA CUDA Debugger CUDA-GDB
- * Copyright (C) 2007-2024 NVIDIA Corporation
+ * Copyright (C) 2007-2025 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -73,13 +73,14 @@ public:
 
   void invalidate (bool quietly);
 
+  bool valid ();
   bool active ();
   bool divergent ();
 
   cuda_clock_t timestamp () const
   { return m_timestamp; }
 
-  bool timestamp_valid ()
+  bool timestamp_valid () const
   { return m_timestamp != 0; }
 
   void set_timestamp (cuda_clock_t now)
@@ -210,7 +211,7 @@ public:
   uint64_t get_error_pc ();
 
   bool has_cluster_exception_target_block_idx ();
-  CuDim3 get_cluster_exception_target_block_idx ();
+  const CuDim3& get_cluster_exception_target_block_idx ();
 
   const CuDim3& get_block_idx ();
   void set_block_idx (const CuDim3& block_idx);
@@ -245,7 +246,7 @@ public:
   bool     get_upredicate (uint32_t predicate);
   void	   set_upredicate (uint32_t predicate, bool value);
 
-  kernel_t get_kernel ();
+  cuda_kernel* get_kernel ();
 
   uint32_t registers_allocated ();
   uint32_t shared_mem_size ();
@@ -258,8 +259,11 @@ public:
 	       const CUDBGDeviceInfo *dev_info,
 	       const CUDBGSMInfo *sm_info, uint8_t*& ptr);
 
+  const CUDBGCbuWarpState &get_cbu_state ();
+
 private:
   void update_warp_resources ();
+  void update_cbu_state ();
 
   /* parent and index */
   uint32_t	m_warp_idx = ~0;
@@ -268,7 +272,7 @@ private:
   cuda_clock_t	m_timestamp = 0;
 
   uint64_t	m_grid_id = 0;
-  kernel_t	m_kernel = nullptr;
+  cuda_kernel*	m_kernel = nullptr;
 
   CuDim3	m_block_idx = { 0, 0, 0 };
   bool	        m_block_idx_p = false;
@@ -312,6 +316,10 @@ private:
 
   /* Array of lanes belonging to this warp */
   std::array<cuda_lane, CUDBG_MAX_LANES> m_lanes;
+
+  /* CBU state */
+  bool m_cbu_state_p = false;
+  CUDBGCbuWarpState m_cbu_state = { 0 };
 };
 
 class cuda_sm final
@@ -433,8 +441,6 @@ public:
   void clear_sm_exception_mask_p ()
   { m_sm_exception_mask_p = false; }
 
-  void create_kernel (uint64_t grid_id);
-
   void invalidate (bool quietly, bool recurse);
 
   bool is_any_context_present () const;
@@ -475,7 +481,7 @@ public:
 
   const cuda_bitset& get_active_sms_mask ();
 
-  kernel_t get_kernel (uint64_t grid_id);
+  cuda_kernel* get_kernel (uint64_t grid_id);
   const CUDBGGridInfo& get_grid_info (uint64_t grid_id);
 
   void print ();
@@ -564,7 +570,6 @@ public:
   { return device (dev_id)->sm (sm_id)->warp (wp_id)->lane (ln_id); }
 
   static void reset ();
-  static void invalidate_kernels ();
   static void invalidate ();
 
   /* System State */
@@ -578,25 +583,16 @@ public:
   static uint32_t get_num_kernels ();
   static uint32_t get_num_present_kernels ();
 
-  static void resolve_breakpoints (int bp_number_from);
-  static void cleanup_breakpoints ();
-
   static cuda_context* create_context (uint32_t dev_id, uint64_t context_id, uint32_t thread_id);
   static void destroy_context (uint64_t context_id);
   static bool is_context_active (cuda_context* context);
   static bool is_any_context_present (uint32_t dev_id);
   static cuda_context* find_context_by_id (uint64_t context_id);
-  
-  static cuda_context* current_context ()
-  { return m_instance.m_current_context; }
 
-  static void set_current_context (cuda_context* context)
-  { m_instance.m_current_context = context; }
-
-  static std::unordered_map<uint64_t, std::unique_ptr<cuda_context>>& contexts ()
+  static const std::unordered_map<uint64_t, std::unique_ptr<cuda_context>>& contexts ()
   { return m_instance.m_context_map; }
 
-  static std::unordered_map<uint64_t, std::unique_ptr<cuda_module>>& modules ()
+  static const std::unordered_map<uint64_t, std::unique_ptr<cuda_module>>& modules ()
   { return m_instance.m_module_map; }
 
   static cuda_module* create_module (uint64_t module_id,
@@ -636,7 +632,71 @@ public:
 			       const char *dev_type, const char *sm_type)
   { device (dev_id)->set_device_spec (num_sms, num_warps, num_lanes, num_registers, dev_type, sm_type); }
 
-  // System helper functions
+  // Kernel helper functions
+  static std::unordered_map<uint64_t, std::unique_ptr<cuda_kernel>>& kernels ()
+  { return m_instance.m_kernel_map; }
+
+  static cuda_kernel* find_kernel_by_grid_id (uint32_t dev_id, uint64_t grid_id);
+  static cuda_kernel* find_kernel_by_kernel_id (uint64_t kernel_id);
+
+  static uint64_t last_launched_kernel_id ()
+  { return m_instance.m_next_kernel_id - 1; }
+
+  static cuda_kernel* create_kernel (uint32_t dev_id, uint64_t grid_id);
+  static cuda_kernel* create_kernel (uint32_t dev_id, uint64_t grid_id,
+				     uint64_t virt_code_base, uint64_t module_id,
+				     CuDim3 grid_dim, CuDim3 block_dim, CuDim3 cluster_dim,
+				     CUDBGKernelType type, CUDBGKernelOrigin origin, uint64_t parent_grid_id);
+  static void destroy_kernel (uint32_t dev_id, uint64_t grid_id);
+  static void destroy_kernel (cuda_kernel* kernel);
+  static void update_kernel_args ();
+  static void update_kernels_terminated ();
+  static void invalidate_kernels (uint32_t dev_id);
+  static void invalidate_kernels (cuda_module* module);
+
+  // Validity checkers
+  static bool device_valid (uint32_t dev_id)
+  {
+    if (dev_id >= m_instance.m_devs.size ())
+      return false;
+    return device (dev_id)->valid ();
+  }
+
+  static bool sm_valid (uint32_t dev_id, uint32_t sm_id)
+  {
+    if (dev_id >= m_instance.m_devs.size ())
+      return false;
+    const auto dev = device (dev_id);
+    if (sm_id >= dev->get_num_sms ())
+      return false;
+    return dev->sm (sm_id)->valid ();
+  }
+
+  static bool warp_valid (uint32_t dev_id, uint32_t sm_id, uint32_t wp_id)
+  {
+    if (dev_id >= m_instance.m_devs.size ())
+      return false;
+    const auto dev = device (dev_id);
+    if (sm_id >= dev->get_num_sms ())
+      return false;
+    if (wp_id >= dev->get_num_warps ())
+      return false;
+    return warp (dev_id, sm_id, wp_id)->valid ();
+  }
+
+  static bool lane_valid (uint32_t dev_id, uint32_t sm_id, uint32_t wp_id, uint32_t ln_id)
+  {
+    if (dev_id >= m_instance.m_devs.size ())
+      return false;
+    const auto dev = device (dev_id);
+    if (sm_id >= dev->get_num_sms ())
+      return false;
+    if (wp_id >= dev->get_num_warps ())
+      return false;
+    if (ln_id >= dev->get_num_lanes ())
+      return false;
+    return lane (dev_id, sm_id, wp_id, ln_id)->valid ();
+  }
 
   // Device helper functions
   static void device_invalidate_kernels (uint32_t dev_id);
@@ -692,9 +752,6 @@ public:
   static uint32_t device_get_pci_dev_id (uint32_t dev_id)
   { return device (dev_id)->get_pci_dev_id (); }
 
-  static bool device_valid (uint32_t dev_id)
-  { return device (dev_id)->valid (); }
-
   static bool device_has_exception (uint32_t dev_id)
   { return device (dev_id)->has_exception (); }
 
@@ -710,7 +767,10 @@ public:
   static void device_resume (uint32_t dev_id)
   { device (dev_id)->resume (); }
 
-  static kernel_t device_get_kernel (uint32_t dev_id, uint64_t grid_id)
+  static const CUDBGGridInfo& device_get_grid_info (uint32_t dev_id, uint64_t grid_id)
+  { return device (dev_id)->get_grid_info (grid_id); }
+
+  static cuda_kernel* device_get_kernel (uint32_t dev_id, uint64_t grid_id)
   { return device (dev_id)->get_kernel (grid_id); }
 
   static bool single_step_warp (uint32_t dev_id, uint32_t sm_id, uint32_t wp_id, uint32_t lane_id_hint, uint32_t nsteps, uint32_t flags, cuda_api_warpmask *single_stepped_warp_mask)
@@ -726,9 +786,6 @@ public:
   static void sm_update_state(uint32_t dev_id, uint32_t sm_id)
   { sm (dev_id, sm_id)->update_state (); }
 
-  static bool sm_valid (uint32_t dev_id, uint32_t sm_id)
-  { return sm (dev_id, sm_id)->valid (); }
-
   static bool sm_has_exception (uint32_t dev_id, uint32_t sm_id)
   { return sm (dev_id, sm_id)->has_exception (); }
 
@@ -747,15 +804,13 @@ public:
   static uint64_t sm_get_error_pc (uint32_t dev_id, uint32_t sm_id)
   { return sm (dev_id, sm_id)->get_error_pc (); }
 
+  // Warp helper functions
   static cuda_clock_t warp_timestamp (uint32_t dev_id, uint32_t sm_id, uint32_t wp_id)
   { return warp (dev_id, sm_id, wp_id)->timestamp (); }
 
   static bool warp_timestamp_valid (uint32_t dev_id, uint32_t sm_id, uint32_t wp_id)
   { return warp (dev_id, sm_id, wp_id)->timestamp_valid (); }
 
-  static bool warp_valid (uint32_t dev_id, uint32_t sm_id, uint32_t wp_id)
-  { return sm (dev_id, sm_id)->warp_valid (wp_id); }
-
   static bool warp_broken (uint32_t dev_id, uint32_t sm_id, uint32_t wp_id)
   { return sm (dev_id, sm_id)->warp_broken (wp_id); }
 
@@ -771,7 +826,7 @@ public:
 	->has_cluster_exception_target_block_idx ();
   }
 
-  static kernel_t warp_get_kernel (uint32_t dev_id, uint32_t sm_id, uint32_t wp_id)
+  static cuda_kernel* warp_get_kernel (uint32_t dev_id, uint32_t sm_id, uint32_t wp_id)
   { return device (dev_id)->sm (sm_id)->warp (wp_id)->get_kernel (); }
 
   static const CuDim3& warp_get_block_idx (uint32_t dev_id, uint32_t sm_id, uint32_t wp_id)
@@ -807,12 +862,13 @@ public:
   static uint64_t warp_get_error_pc (uint32_t dev_id, uint32_t sm_id, uint32_t wp_id)
   { return warp (dev_id, sm_id, wp_id)->get_error_pc (); }
 
-  static CuDim3
-  warp_get_cluster_exception_target_block_idx (uint32_t dev_id, uint32_t sm_id,
-					       uint32_t wp_id)
+  static const CuDim3& warp_get_cluster_exception_target_block_idx (uint32_t dev_id, uint32_t sm_id, uint32_t wp_id)
+  { return warp (dev_id, sm_id, wp_id)->get_cluster_exception_target_block_idx (); }
+
+  static const CUDBGCbuWarpState &
+  warp_get_cbu_state (uint32_t dev_id, uint32_t sm_id, uint32_t wp_id)
   {
-    return warp (dev_id, sm_id, wp_id)
-	->get_cluster_exception_target_block_idx ();
+    return warp (dev_id, sm_id, wp_id)->get_cbu_state ();
   }
 
   static void warp_set_grid_id (uint32_t dev_id, uint32_t sm_id, uint32_t wp_id, uint64_t grid_id)
@@ -849,9 +905,6 @@ public:
   static bool lane_timestamp_valid (uint32_t dev_id, uint32_t sm_id, uint32_t wp_id, uint32_t ln_id)
   { return lane (dev_id, sm_id, wp_id, ln_id)->timestamp_valid (); }
 
-  static bool lane_valid (uint32_t dev_id, uint32_t sm_id, uint32_t wp_id, uint32_t ln_id)
-  { return warp (dev_id, sm_id, wp_id)->lane_valid (ln_id); }
-
   static bool lane_active (uint32_t dev_id, uint32_t sm_id, uint32_t wp_id, uint32_t ln_id)
   { return warp (dev_id, sm_id, wp_id)->lane_active (ln_id); }
 
@@ -861,7 +914,7 @@ public:
   static uint64_t lane_get_pc (uint32_t dev_id, uint32_t sm_id, uint32_t wp_id, uint32_t ln_id)
   { return lane (dev_id, sm_id, wp_id, ln_id)->get_pc (); }
 
-  static CuDim3	lane_get_thread_idx (uint32_t dev_id, uint32_t sm_id, uint32_t wp_id, uint32_t ln_id)
+  static const CuDim3& lane_get_thread_idx (uint32_t dev_id, uint32_t sm_id, uint32_t wp_id, uint32_t ln_id)
   { return lane (dev_id, sm_id, wp_id, ln_id)->get_thread_idx (); }
 
   static CUDBGException_t lane_get_exception (uint32_t dev_id, uint32_t sm_id, uint32_t wp_id, uint32_t ln_id)
@@ -899,9 +952,12 @@ public:
   { return lane (dev_id, sm_id, wp_id, ln_id)->get_return_address (level); }
 
 private:
+
+  static cuda_kernel* add_parent_kernel (uint32_t dev_id, uint64_t grid_id);
+
   static cuda_state m_instance;
 
-  uint32_t    m_num_devices = 0;
+  uint32_t m_num_devices = 0;
 
   cuda_bitset m_suspended_devices_mask;
 
@@ -910,7 +966,27 @@ private:
   std::unordered_map<uint64_t, std::unique_ptr<cuda_context>> m_context_map;
   std::unordered_map<uint64_t, std::unique_ptr<cuda_module>> m_module_map;
 
-  cuda_context* m_current_context;
+  // Mapping of kernel_id to cuda_kernel*
+  uint64_t m_next_kernel_id;
+  std::unordered_map<uint64_t, std::unique_ptr<cuda_kernel>> m_kernel_map;
 };
 
+inline bool
+cuda_lane::valid ()
+{
+  return warp ()->lane_valid (lane_idx ());
+}
+
+inline bool
+cuda_lane::active ()
+{
+  return warp ()->lane_active (lane_idx ());
+}
+
+inline bool
+cuda_lane::divergent ()
+{
+  return warp ()->lane_divergent (lane_idx ());
+}
+
 #endif
