@@ -1,6 +1,6 @@
 /*
  * NVIDIA CUDA Debugger CUDA-GDB
- * Copyright (C) 2007-2024 NVIDIA Corporation
+ * Copyright (C) 2007-2025 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -342,8 +342,13 @@ cuda_gdb_record_remove_all (void *unused)
 static void
 cuda_gdb_lock_file_create (void)
 {
+  /* Default == false, can be overriden via a command-line option */
+  if (!cuda_use_lockfile)
+    return;
+
   struct stat st;
-  char buf[CUDA_GDB_TMP_BUF_SIZE];
+  /* This needs to fix cuda_gdb_get_tmp_basedir + cuda_gdb_lock_file */
+  char buf[2*CUDA_GDB_TMP_BUF_SIZE];
   char *visible_devices;
   uint32_t dev_id, num_devices = 0;
   int i;
@@ -352,10 +357,6 @@ cuda_gdb_lock_file_create (void)
   mode_t old_umask;
   int my_pid = (int)getpid ();
 
-  /* Default == false, can be overriden via a command-line option */
-  if (!cuda_use_lockfile)
-    return;
-
   snprintf (buf, sizeof (buf), "%s/%s", cuda_gdb_get_tmp_basedir (),
             cuda_gdb_lock_file);
 
@@ -438,7 +439,8 @@ cuda_gdb_lock_file_create (void)
 static void
 cuda_gdb_tmpdir_setup (void)
 {
-  char dirpath[CUDA_GDB_TMP_BUF_SIZE];
+  /* This needs to fit cuda_gdb_get_tmp_basedir + getpid output */
+  char dirpath[2*CUDA_GDB_TMP_BUF_SIZE];
   int ret;
   bool dir_exists = false;
   bool override_umask = false;
@@ -460,8 +462,9 @@ cuda_gdb_tmpdir_setup (void)
     cuda_gdb_dir_cleanup_files (dirpath);
 #endif /* __QNXHOST__ */
 
-  cuda_gdb_tmp_dir = (char *)xmalloc (strlen (dirpath) + 1);
-  strncpy (cuda_gdb_tmp_dir, dirpath, strlen (dirpath) + 1);
+  size_t buflen = strlen (dirpath) + 1;
+  cuda_gdb_tmp_dir = (char *)xmalloc (buflen);
+  strncpy (cuda_gdb_tmp_dir, dirpath, buflen);
 
   /* No final cleanup chain at server side,
    * cleanup function is called explicitly when server quits */
@@ -608,11 +611,11 @@ cuda_ptx_cache_store_register (frame_info_ptr frame, int dwarf_regnum,
   /* Store information about cached register in temporary cache element*/
   new_elem.frame_id = get_frame_id (frame);
   new_elem.dwarf_regnum = dwarf_regnum;
-  new_elem.len = value_type (value)->length ();
+  new_elem.len = value->type ()->length ();
   /* If element can not be cached - return */
   if (new_elem.len > sizeof (new_elem.data))
     return;
-  memcpy (new_elem.data, value_contents_raw (value).data (), new_elem.len);
+  memcpy (new_elem.data, value->contents_raw ().data (), new_elem.len);
 
   auto elem = cuda_ptx_cache_find_element (new_elem.frame_id, dwarf_regnum);
   if (elem != cuda_ptx_register_cache.end ())
@@ -638,20 +641,20 @@ cuda_ptx_cache_get_register (frame_info_ptr frame, int dwarf_regnum,
 {
   struct value *retval;
 
-  retval = allocate_value (type);
+  retval = value::allocate (type);
 
   auto elem = cuda_ptx_cache_find_element (get_frame_id (frame), dwarf_regnum);
   if ((elem == cuda_ptx_register_cache.end ()) || (elem->len != type->length ())
       || !cuda_options_variable_value_cache_enabled ())
     {
-      VALUE_LVAL (retval) = not_lval;
-      mark_value_bytes_optimized_out (retval, 0, type->length ());
+      retval->set_lval (not_lval);
+      retval->mark_bytes_optimized_out (0, type->length ());
       return retval;
     }
 
-  mark_value_bytes_optimized_out (retval, 0, 0);
-  set_value_cached (retval, 1);
-  memcpy (value_contents_raw (retval).data (), elem->data, elem->len);
+  retval->mark_bytes_optimized_out (0, 0);
+  retval->set_cached (true);
+  memcpy (retval->contents_raw ().data (), elem->data, elem->len);
   return retval;
 }
 
@@ -673,9 +676,7 @@ cuda_ptx_cache_update_local_vars (void)
   const struct block *block = NULL;
 
   if (!cuda_current_focus::isDevice ()
-      || find_thread_ptid (current_inferior ()->process_target (),
-                           inferior_ptid)
-             ->executing ())
+      || current_inferior ()->process_target ()->find_thread (inferior_ptid)->executing ())
     return;
   cb_data.m_frame = get_current_frame ();
   if (!cb_data.m_frame)
@@ -809,7 +810,7 @@ bool
 cuda_is_value_managed_pointer (struct value *value)
 {
   bool result = false;
-  struct type *type = value ? value_type (value) : NULL;
+  struct type *type = value ? value->type () : NULL;
 
   /* Sanity checks */
   if (type == NULL || type->code () != TYPE_CODE_PTR)
@@ -818,7 +819,7 @@ cuda_is_value_managed_pointer (struct value *value)
   try
     {
       result = cuda_managed_address_p (
-          unpack_pointer (type, value_contents_for_printing (value).data ()));
+          unpack_pointer (type, value->contents_for_printing ().data ()));
     }
   catch (const gdb_exception_error &e)
     {
