@@ -1,6 +1,6 @@
 /*
  * NVIDIA CUDA Debugger CUDA-GDB
- * Copyright (C) 2007-2024 NVIDIA Corporation
+ * Copyright (C) 2007-2025 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -36,6 +36,8 @@
 #include <signal.h>
 #include <stdio.h>
 
+#include "gdbsupport/gdb_assert.h"
+
 #include "cuda/libcudbg.h"
 #include "cuda/libcudbgipc.h"
 #if TRACE_LIBCUDBG
@@ -964,37 +966,15 @@ STUB_cudbgWriteGlobalMemory55 (uint32_t dev, uint32_t sm, uint32_t wp,
 }
 
 static CUDBGResult
-cudbgReadPinnedMemory (uint64_t addr, void *buf, uint32_t buf_size)
+STUB_cudbgReadPinnedMemory (uint64_t addr, void *buf, uint32_t buf_size)
 {
-  char *ipc_buf;
-  CUDBGResult result;
-
-  CUDBG_IPC_BEGIN (CUDBGAPIREQ_readPinnedMemory);
-  CUDBG_IPC_APPEND (&addr, sizeof (addr));
-  CUDBG_IPC_APPEND (&buf_size, sizeof (buf_size));
-
-  CUDBG_IPC_REQUEST ((void **)&ipc_buf);
-  CUDBG_IPC_RECEIVE (&result, &ipc_buf);
-  CUDBG_IPC_RECEIVE_ARRAY (buf, buf_size, &ipc_buf);
-
-  return result;
+  return CUDBG_ERROR_UNKNOWN;
 }
 
 static CUDBGResult
-cudbgWritePinnedMemory (uint64_t addr, const void *buf, uint32_t buf_size)
+STUB_cudbgWritePinnedMemory (uint64_t addr, const void *buf, uint32_t buf_size)
 {
-  char *ipc_buf;
-  CUDBGResult result;
-
-  CUDBG_IPC_BEGIN (CUDBGAPIREQ_writePinnedMemory);
-  CUDBG_IPC_APPEND (&addr, sizeof (addr));
-  CUDBG_IPC_APPEND (&buf_size, sizeof (buf_size));
-  CUDBG_IPC_APPEND ((char *)buf, buf_size);
-
-  CUDBG_IPC_REQUEST ((void **)&ipc_buf);
-  CUDBG_IPC_RECEIVE (&result, &ipc_buf);
-
-  return result;
+  return CUDBG_ERROR_UNKNOWN;
 }
 
 static CUDBGResult
@@ -1666,19 +1646,9 @@ cudbgGetManagedMemoryRegionInfo (uint64_t startAddress,
 }
 
 static CUDBGResult
-cudbgIsDeviceCodeAddress (uintptr_t addr, bool *isDeviceAddress)
+STUB_cudbgIsDeviceCodeAddress (uintptr_t addr, bool *isDeviceAddress)
 {
-  char *ipc_buf;
-  CUDBGResult result;
-
-  CUDBG_IPC_BEGIN (CUDBGAPIREQ_isDeviceCodeAddress);
-  CUDBG_IPC_APPEND (&addr, sizeof (addr));
-
-  CUDBG_IPC_REQUEST ((void **)&ipc_buf);
-  CUDBG_IPC_RECEIVE (&result, &ipc_buf);
-  CUDBG_IPC_RECEIVE (isDeviceAddress, &ipc_buf);
-
-  return result;
+  return CUDBG_ERROR_UNKNOWN;
 }
 
 static CUDBGResult
@@ -2301,13 +2271,13 @@ cudbgGetClusterDim (uint32_t dev, uint32_t sm, uint32_t wp, CuDim3 *clusterDim)
 }
 
 static CUDBGResult
-cudbgReadWarpState (uint32_t devId, uint32_t sm, uint32_t wp,
-		    CUDBGWarpState *state)
+cudbgReadWarpState127 (uint32_t devId, uint32_t sm, uint32_t wp,
+		    CUDBGWarpState127 *state)
 {
   char *ipc_buf;
   CUDBGResult result;
 
-  CUDBG_IPC_BEGIN (CUDBGAPIREQ_readWarpState);
+  CUDBG_IPC_BEGIN (CUDBGAPIREQ_readWarpState127);
   CUDBG_IPC_APPEND (&devId, sizeof (devId));
   CUDBG_IPC_APPEND (&sm, sizeof (sm));
   CUDBG_IPC_APPEND (&wp, sizeof (wp));
@@ -2358,7 +2328,100 @@ cudbgReadWarpResources (uint32_t devId, uint32_t sm, uint32_t wp,
   return result;
 }
 
-static const struct CUDBGAPI_st cudbgCurrentApi = {
+static CUDBGResult
+cudbgGetCbuWarpState (uint32_t devId, uint32_t sm, uint64_t wp_mask,
+		      CUDBGCbuWarpState *state, uint32_t num_states)
+{
+  char *ipc_buf;
+  CUDBGResult result;
+
+  CUDBG_IPC_BEGIN (CUDBGAPIREQ_getCbuWarpState);
+  CUDBG_IPC_APPEND (&devId, sizeof (devId));
+  CUDBG_IPC_APPEND (&sm, sizeof (sm));
+  CUDBG_IPC_APPEND (&wp_mask, sizeof (wp_mask));
+
+  CUDBG_IPC_REQUEST ((void **)&ipc_buf);
+  CUDBG_IPC_RECEIVE (&result, &ipc_buf);
+  if (result == CUDBG_SUCCESS)
+    {
+      uint32_t num_states_received;
+      CUDBG_IPC_RECEIVE (&num_states_received, &ipc_buf);
+      /* Sanity check as the backend must return CUDBG_ERROR_BUFFER_TOO_SMALL
+       * if num_states < popcount(wp_mask) */
+      gdb_assert (num_states_received <= num_states);
+      CUDBG_IPC_RECEIVE_ARRAY (state, num_states_received, &ipc_buf);
+    }
+
+  return result;
+}
+
+static CUDBGResult
+cudbgReadWarpState (uint32_t devId, uint32_t sm, uint32_t wp,
+		    CUDBGWarpState *state)
+{
+  char *ipc_buf;
+  CUDBGResult result;
+
+  CUDBG_IPC_BEGIN (CUDBGAPIREQ_readWarpState);
+  CUDBG_IPC_APPEND (&devId, sizeof (devId));
+  CUDBG_IPC_APPEND (&sm, sizeof (sm));
+  CUDBG_IPC_APPEND (&wp, sizeof (wp));
+
+  CUDBG_IPC_REQUEST ((void **)&ipc_buf);
+  CUDBG_IPC_RECEIVE (&result, &ipc_buf);
+  CUDBG_IPC_RECEIVE (state, &ipc_buf);
+
+  return result;
+}
+
+static CUDBGResult
+cudbgConsumeCudaLogs (CUDBGCudaLogMessage* logMessages, uint32_t numMessages,
+		      uint32_t* numConsumed)
+{
+  char *ipc_buf;
+  CUDBGResult result;
+
+  CUDBG_IPC_BEGIN (CUDBGAPIREQ_consumeCudaLogs);
+  CUDBG_IPC_APPEND (&numMessages, sizeof (numMessages));
+
+  CUDBG_IPC_REQUEST ((void **)&ipc_buf);
+  CUDBG_IPC_RECEIVE (&result, &ipc_buf);
+  CUDBG_IPC_RECEIVE (numConsumed, &ipc_buf);
+  gdb_assert (*numConsumed <= numMessages);
+  if (result == CUDBG_SUCCESS)
+    {
+      CUDBG_IPC_RECEIVE_ARRAY (logMessages, std::min (*numConsumed, numMessages), &ipc_buf);
+    }
+
+  return result;
+}
+
+static CUDBGResult
+cudbgReadCPUCallStack (uint32_t dev, uint64_t gridId64, uint64_t *addrs, uint32_t numAddrs, uint32_t* totalNumAddrs)
+{
+  char *ipc_buf;
+  CUDBGResult result;
+
+  CUDBG_IPC_BEGIN (CUDBGAPIREQ_readCPUCallStack);
+  CUDBG_IPC_APPEND (&dev, sizeof (dev));
+  CUDBG_IPC_APPEND (&gridId64, sizeof (gridId64));
+
+  CUDBG_IPC_REQUEST ((void **)&ipc_buf);
+  CUDBG_IPC_RECEIVE (&result, &ipc_buf);
+  CUDBG_IPC_RECEIVE (totalNumAddrs, &ipc_buf);
+  CUDBG_IPC_RECEIVE_ARRAY (addrs, std::min(numAddrs, *totalNumAddrs), &ipc_buf);
+
+  return result;
+}
+
+template<typename ...T>
+inline constexpr size_t numberOfArgs(T ... a) { return sizeof...(a); }
+
+#define DEFINE_CUDBGAPI(name, ...)											\
+  static_assert(numberOfArgs(__VA_ARGS__) == sizeof(struct CUDBGAPI_st) / sizeof(void *), "Not all fields initialized");\
+  static const struct CUDBGAPI_st name = { __VA_ARGS__ }
+
+DEFINE_CUDBGAPI(cudbgCurrentApi,
   /* Initialization */
   cudbgInitialize,
   cudbgFinalize,
@@ -2440,8 +2503,8 @@ static const struct CUDBGAPI_st cudbgCurrentApi = {
   /* 3.2 Extensions */
   STUB_cudbgReadGlobalMemory55,
   STUB_cudbgWriteGlobalMemory55,
-  cudbgReadPinnedMemory,
-  cudbgWritePinnedMemory,
+  STUB_cudbgReadPinnedMemory,
+  STUB_cudbgWritePinnedMemory,
   cudbgSetBreakpoint,
   cudbgUnsetBreakpoint,
   STUB_cudbgSetNotifyNewEventCallback40,
@@ -2498,7 +2561,7 @@ static const struct CUDBGAPI_st cudbgCurrentApi = {
   cudbgReadGlobalMemory,
   cudbgWriteGlobalMemory,
   cudbgGetManagedMemoryRegionInfo,
-  cudbgIsDeviceCodeAddress,
+  STUB_cudbgIsDeviceCodeAddress,
   cudbgRequestCleanupOnDetach,
 
   /* 6.5 Extensions */
@@ -2556,12 +2619,18 @@ static const struct CUDBGAPI_st cudbgCurrentApi = {
   /* 12.7 Extensions */
   cudbgGetGridInfo,
   cudbgGetClusterDim,
-  cudbgReadWarpState,
+  cudbgReadWarpState127,
   cudbgGetClusterExceptionTargetBlock,
 
   /* 12.8 Extensions */
   cudbgReadWarpResources,
-};
+
+  /* 12.9 Extensions */
+  cudbgGetCbuWarpState,
+  cudbgReadWarpState,
+  cudbgConsumeCudaLogs,
+  cudbgReadCPUCallStack
+);
 
 CUDBGResult
 cudbgGetAPI (uint32_t major, uint32_t minor, uint32_t rev, CUDBGAPI *api)
