@@ -1,6 +1,6 @@
 /*
  * NVIDIA CUDA Debugger CUDA-GDB
- * Copyright (C) 2007-2024 NVIDIA Corporation
+ * Copyright (C) 2007-2025 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -26,7 +26,7 @@
    extra space in-between each element, which will be the filter string for the
    info commands. */
 static char*
-concatenate_string (char **argv, int argc)
+concatenate_string (const char *const *argv, int argc)
 {
   int allocated = 0;
   int copied = 0;
@@ -59,7 +59,7 @@ concatenate_string (char **argv, int argc)
 }
 
 void
-mi_cmd_cuda_info_devices (const char *command, char **argv, int argc)
+mi_cmd_cuda_info_devices (const char *command, const char *const *argv, int argc)
 {
   char *filter = concatenate_string (argv, argc);
 
@@ -69,7 +69,7 @@ mi_cmd_cuda_info_devices (const char *command, char **argv, int argc)
 }
 
 void
-mi_cmd_cuda_info_sms (const char *command, char **argv, int argc)
+mi_cmd_cuda_info_sms (const char *command, const char *const *argv, int argc)
 {
   char *filter = concatenate_string (argv, argc);
 
@@ -79,7 +79,7 @@ mi_cmd_cuda_info_sms (const char *command, char **argv, int argc)
 }
 
 void
-mi_cmd_cuda_info_warps (const char *command, char **argv, int argc)
+mi_cmd_cuda_info_warps (const char *command, const char *const *argv, int argc)
 {
   char *filter = concatenate_string (argv, argc);
 
@@ -89,7 +89,7 @@ mi_cmd_cuda_info_warps (const char *command, char **argv, int argc)
 }
 
 void
-mi_cmd_cuda_info_lanes (const char *command, char **argv, int argc)
+mi_cmd_cuda_info_lanes (const char *command, const char *const *argv, int argc)
 {
   char *filter = concatenate_string (argv, argc);
 
@@ -99,7 +99,7 @@ mi_cmd_cuda_info_lanes (const char *command, char **argv, int argc)
 }
 
 void
-mi_cmd_cuda_info_kernels (const char *command, char **argv, int argc)
+mi_cmd_cuda_info_kernels (const char *command, const char *const *argv, int argc)
 {
   char *filter = concatenate_string (argv, argc);
 
@@ -109,7 +109,7 @@ mi_cmd_cuda_info_kernels (const char *command, char **argv, int argc)
 }
 
 void
-mi_cmd_cuda_info_contexts (const char *command, char **argv, int argc)
+mi_cmd_cuda_info_contexts (const char *command, const char *const *argv, int argc)
 {
   char *filter = concatenate_string (argv, argc);
 
@@ -119,7 +119,7 @@ mi_cmd_cuda_info_contexts (const char *command, char **argv, int argc)
 }
 
 void
-mi_cmd_cuda_info_blocks (const char *command, char **argv, int argc)
+mi_cmd_cuda_info_blocks (const char *command, const char *const *argv, int argc)
 {
   char *filter = concatenate_string (argv, argc);
 
@@ -129,7 +129,7 @@ mi_cmd_cuda_info_blocks (const char *command, char **argv, int argc)
 }
 
 void
-mi_cmd_cuda_info_threads (const char *command, char **argv, int argc)
+mi_cmd_cuda_info_threads (const char *command, const char *const *argv, int argc)
 {
   char *filter = concatenate_string (argv, argc);
 
@@ -139,7 +139,7 @@ mi_cmd_cuda_info_threads (const char *command, char **argv, int argc)
 }
 
 void
-mi_cmd_cuda_info_launch_trace (const char *command, char **argv, int argc)
+mi_cmd_cuda_info_launch_trace (const char *command, const char *const *argv, int argc)
 {
   char *filter = concatenate_string (argv, argc);
 
@@ -149,7 +149,7 @@ mi_cmd_cuda_info_launch_trace (const char *command, char **argv, int argc)
 }
 
 void
-mi_cmd_cuda_info_launch_children (const char *command, char **argv, int argc)
+mi_cmd_cuda_info_launch_children (const char *command, const char *const *argv, int argc)
 {
   char *filter = concatenate_string (argv, argc);
 
@@ -159,7 +159,7 @@ mi_cmd_cuda_info_launch_children (const char *command, char **argv, int argc)
 }
 
 void
-mi_cmd_cuda_info_managed (const char *command, char **argv, int argc)
+mi_cmd_cuda_info_managed (const char *command, const char *const *argv, int argc)
 {
   char *filter = concatenate_string (argv, argc);
 
@@ -169,7 +169,7 @@ mi_cmd_cuda_info_managed (const char *command, char **argv, int argc)
 }
 
 void
-mi_cmd_cuda_info_line (const char *command, char **argv, int argc)
+mi_cmd_cuda_info_line (const char *command, const char *const *argv, int argc)
 {
   char *filter = concatenate_string (argv, argc);
 
@@ -179,7 +179,7 @@ mi_cmd_cuda_info_line (const char *command, char **argv, int argc)
 }
 
 void
-mi_cmd_cuda_focus_query (const char *command, char **argv, int argc)
+mi_cmd_cuda_focus_query (const char *command, const char *const *argv, int argc)
 {
   char *query_string = concatenate_string (argv, argc);
 
@@ -189,7 +189,7 @@ mi_cmd_cuda_focus_query (const char *command, char **argv, int argc)
 }
 
 void
-mi_cmd_cuda_focus_switch (const char *command, char **argv, int argc)
+mi_cmd_cuda_focus_switch (const char *command, const char *const *argv, int argc)
 {
   char *switch_string = concatenate_string (argv, argc);
 
