@@ -1,6 +1,6 @@
 /*
  * NVIDIA CUDA Debugger CUDA-GDB
- * Copyright (C) 2007-2024 NVIDIA Corporation
+ * Copyright (C) 2007-2025 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -22,5 +22,8 @@
 #include "remote.h"
 
 bool cuda_remote_initialize_target ();
+#ifdef __QNXTARGET__
+void cuda_reset_qnx_symbols ();
+#endif
 
 #endif
