@@ -20,7 +20,7 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* NVIDIA CUDA Debugger CUDA-GDB
-   Copyright (C) 2007-2024 NVIDIA Corporation
+   Copyright (C) 2007-2025 NVIDIA Corporation
    Modified from the original GDB file referenced above by the CUDA-GDB
    team at NVIDIA <cudatools@nvidia.com>. */
 
@@ -145,11 +145,13 @@ nto_get_siginfo_type (struct gdbarch *gdbarch)
   if (nto_gdbarch_data->siginfo_type != NULL)
     return nto_gdbarch_data->siginfo_type;
 
-  int_type = arch_integer_type (gdbarch, gdbarch_int_bit (gdbarch),
+  type_allocator alloc (gdbarch);
+
+  int_type = init_integer_type (alloc, gdbarch_int_bit (gdbarch),
 				 0, "int");
-  uint_type = arch_integer_type (gdbarch, gdbarch_int_bit (gdbarch),
+  uint_type = init_integer_type (alloc, gdbarch_int_bit (gdbarch),
 				 1, "unsigned int");
-  long_type = arch_integer_type (gdbarch, gdbarch_long_bit (gdbarch),
+  long_type = init_integer_type (alloc, gdbarch_long_bit (gdbarch),
 				 0, "long");
   void_ptr_type = lookup_pointer_type (builtin_type (gdbarch)->builtin_void);
 
@@ -161,19 +163,19 @@ nto_get_siginfo_type (struct gdbarch *gdbarch)
 				       void_ptr_type, long_type->length ());
 
   /* pid_t */
-  pid_type = arch_type (gdbarch, TYPE_CODE_TYPEDEF,
+  pid_type = alloc.new_type (TYPE_CODE_TYPEDEF,
 			gdbarch_int_bit (gdbarch), "pid_t");
   pid_type->set_target_type (int_type);
   pid_type->set_is_stub (true);
 
   /* uid_t */
-  uid_type = arch_type (gdbarch, TYPE_CODE_TYPEDEF,
+  uid_type = alloc.new_type (TYPE_CODE_TYPEDEF,
 			gdbarch_int_bit (gdbarch), "uid_t");
   uid_type->set_target_type (int_type);
   uid_type->set_is_stub (true);
 
   /* clock_t */
-  clock_type = arch_type (gdbarch, TYPE_CODE_TYPEDEF,
+  clock_type = alloc.new_type (TYPE_CODE_TYPEDEF,
 			  gdbarch_int_bit (gdbarch), "clock_t");
   clock_type->set_target_type (uint_type);
   clock_type->set_is_stub (true);
@@ -1169,7 +1171,7 @@ nto_new_thread_listener( thread_info *ti ) {
     return;
   }
 
-  if( bfd_bread ((gdb_byte *)&ps, sectsize, core_bfd ) != sectsize ) {
+  if( bfd_read ((gdb_byte *)&ps, sectsize, core_bfd ) != sectsize ) {
     warning ( "could not read %d bytes from section %s!", sectsize, sectname);
     return;
   }
@@ -1257,7 +1259,7 @@ nto_inferior_appeared_listener( inferior *inf ) {
     return;
   }
 
-  len = bfd_bread ((gdb_byte *)&pinfo, sectsize, core_bfd);
+  len = bfd_read ((gdb_byte *)&pinfo, sectsize, core_bfd);
   if (len != sectsize) {
     warning ("Could not read procinfo from %s!", sectname );
     return;
