@@ -1,6 +1,6 @@
 /*
  * NVIDIA CUDA Debugger CUDA-GDB
- * Copyright (C) 2013-2024 NVIDIA Corporation
+ * Copyright (C) 2013-2025 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -556,20 +556,6 @@ cuda_process_query_device_spec_packet (char *buf)
   p = append_string (sm_type, p, false);
 }
 
-static void
-cuda_process_is_device_code_address_packet (char *buf)
-{
-  CUDBGResult res;
-  char *p;
-  uint64_t addr = 0;
-  bool is_device_address;
-
-  extract_bin (NULL, (unsigned char *) &addr, sizeof (addr));
-  res = cudbgAPI->isDeviceCodeAddress (addr, &is_device_address);
-  p = append_bin ((unsigned char *) &res, buf, sizeof (res), true);
-  p = append_bin ((unsigned char *) &is_device_address, p, sizeof (is_device_address), false);
-}
-
 static void
 cuda_process_get_grid_status_packet (char *buf)
 {
@@ -975,25 +961,6 @@ cuda_process_read_generic_memory_packet (char *buf)
   xfree (value);
 }
 
-static void
-cuda_process_read_pinned_memory_packet (char *buf)
-{
-  CUDBGResult res;
-  char *p;
-  uint64_t addr;
-  uint32_t sz;
-  void *value;
-
-  extract_bin (NULL, (unsigned char *) &addr, sizeof (addr));
-  extract_bin (NULL, (unsigned char *) &sz,   sizeof (sz));
-
-  value = xmalloc (sz);
-  res = cudbgAPI->readPinnedMemory (addr, value, sz);
-  p = append_bin ((unsigned char *) &res, buf, sizeof (res), true);
-  p = append_bin ((unsigned char *) value, p, sz, false);
-  xfree (value);
-}
-
 static void
 cuda_process_read_param_memory_packet (char *buf)
 {
@@ -1098,24 +1065,6 @@ cuda_process_write_generic_memory_packet (char *buf)
   xfree (value);
 }
 
-static void
-cuda_process_write_pinned_memory_packet (char *buf)
-{
-  CUDBGResult res;
-  uint64_t addr;
-  uint32_t sz;
-  void *value;
-
-  extract_bin (NULL, (unsigned char *) &addr, sizeof (addr));
-  extract_bin (NULL, (unsigned char *) &sz,   sizeof (sz));
-
-  value = xmalloc (sz);
-  extract_bin (NULL, (unsigned char *) value, sz);
-  res = cudbgAPI->writePinnedMemory (addr, value, sz);
-  append_bin ((unsigned char *) &res, buf, sizeof (res), false);
-  xfree (value);
-}
-
 static void
 cuda_process_write_param_memory_packet (char *buf)
 {
@@ -1475,7 +1424,8 @@ handle_cuda_packet (char *buf)
       cuda_process_read_generic_memory_packet (buf);
       break;
     case READ_PINNED_MEMORY:
-      cuda_process_read_pinned_memory_packet (buf);
+      /* Support dropped */
+      buf[0] = '\0';
       break;
     case READ_PARAM_MEMORY:
       cuda_process_read_param_memory_packet (buf);
@@ -1483,10 +1433,9 @@ handle_cuda_packet (char *buf)
     case READ_SHARED_MEMORY:
       cuda_process_read_shared_memory_packet (buf);
       break;
-    /* Texture support has been removed - let the other
-     * side know we no longer support it. */
     case READ_TEXTURE_MEMORY:
     case READ_TEXTURE_MEMORY_BINDLESS:
+      /* Support dropped */
       buf[0] = '\0';
       break;
     case READ_LOCAL_MEMORY:
@@ -1516,9 +1465,6 @@ handle_cuda_packet (char *buf)
     case WRITE_GENERIC_MEMORY:
       cuda_process_write_generic_memory_packet (buf);
       break;
-    case WRITE_PINNED_MEMORY:
-      cuda_process_write_pinned_memory_packet (buf);
-      break;
     case WRITE_PARAM_MEMORY:
       cuda_process_write_param_memory_packet (buf);
       break;
@@ -1532,7 +1478,7 @@ handle_cuda_packet (char *buf)
       cuda_process_write_register_packet (buf);
       break;
     case IS_DEVICE_CODE_ADDRESS:
-      cuda_process_is_device_code_address_packet (buf);
+      /* Support dropped */
       break;
     case DISASSEMBLE:
       cuda_process_disassemble_packet (buf);
