@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014-2024 NVIDIA CORPORATION. All rights reserved.
+ * Copyright (c) 2014-2025 NVIDIA CORPORATION. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -441,6 +441,98 @@ DEF_API_CALL(readConstMemory)(uint32_t dev, uint64_t addr, void *buf,
 	return API_CALL(readGenericMemory)(dev, 0, 0, 0, addr, buf, sz);
 }
 
+DEF_API_CALL(getCbuWarpState)(uint32_t devId, uint32_t sm, uint64_t wp_mask,
+			      CUDBGCbuWarpState *state, uint32_t num_states)
+{
+	Elf_Scn *scn;
+	Elf_Data data;
+	CudbgDeviceTableEntry *dte;
+	CudbgThreadTableEntry *tte;
+	CudbgWarpTableEntry *wte;
+	CUDBGResult rc;
+	uint32_t numConvBarriersPrWarp;
+	uint32_t wpStatesWritten = 0;
+	size_t dteSize;
+	size_t wteSize;
+	TRACE_FUNC("devId=%u sm=%u wp_mask=0x%llx state=%p num_states=%u",
+		   devId, sm, wp_mask, state, num_states);
+	
+	VERIFY(wp_mask != 0, CUDBG_ERROR_INVALID_ARGS, "wp_mask must be non-zero");
+	VERIFY_ARG(state);
+	VERIFY (num_states > 0 && num_states < CUDBG_MAX_WARPS,
+		CUDBG_ERROR_INVALID_ARGS,
+		"num_states must be > 0 and <= %u", CUDBG_MAX_WARPS);
+	VERIFY (devId < cuCoreGetNumDevices (curcc),
+		CUDBG_ERROR_INVALID_DEVICE,
+		"devId must be less than the number of CUDA devices");
+
+	GET_TABLE_ENTRY(dte, &dteSize, CUDBG_ERROR_INVALID_DEVICE, "dev%u", devId);
+	/* Get number of convergence barriers per warp */
+	if (offsetof (CudbgDeviceTableEntry, numConvergenceBarriersPrWarp)
+	    < dteSize)
+	  numConvBarriersPrWarp = dte->numConvergenceBarriersPrWarp;
+	else
+	  /* The old core doesn't have the CBU state */
+	  return CUDBG_ERROR_NOT_SUPPORTED;
+
+	for (uint32_t wp = 0;
+	     wp < CUDBG_MAX_WARPS && wpStatesWritten < num_states; wp++)
+	  {
+	    if (!getBit (wp_mask, wp))
+	      continue;
+
+	    /* Fill in the warp state */
+	    GET_TABLE_ENTRY (wte, &wteSize, CUDBG_ERROR_INVALID_WARP,
+			     "wp%u_sm%u_dev%u", wp, sm, devId);
+
+	    /* sanity check */
+	    if (offsetof (CudbgWarpTableEntry, cbuActiveLanesMask) >= wteSize)
+	      return CUDBG_ERROR_INTERNAL;
+
+	    /* Initialize the CBU warp state */
+	    memset (state, 0, sizeof (*state));
+
+	    /* Fill the CBU masks */
+	    state->activeMask = wte->cbuActiveLanesMask;
+	    state->exitedMask = wte->cbuExitedLanesMask;
+	    state->collectiveMask = wte->cbuCollectiveLanesMask;
+
+	    /* Fill the conv barrier masks */
+	    if (cuCoreReadSectionData (curcc->e, scn, &data) != 0)
+	      return CUDBG_ERROR_UNKNOWN;
+
+	    const size_t convBarriersByteSize
+		= numConvBarriersPrWarp * sizeof (state->barrierMasks[0]);
+
+	    if (convBarriersByteSize > data.d_size)
+	      return CUDBG_ERROR_INTERNAL;
+
+	    memcpy (&state->barrierMasks, (char *)data.d_buf,
+		    convBarriersByteSize);
+
+	    /* Fill the lane thread states */
+	    for (uint32_t ln = 0;
+		 ln < dte->numLanesPerWarp && ln < CUDBG_MAX_LANES; ++ln)
+	      {
+		if (!getBit (wte->validLanesMask, ln))
+		  continue;
+
+		/* For every valid lane there must be a corresponding
+		 * ThreadTableEntry */
+		GET_TABLE_ENTRY (tte, NULL, CUDBG_ERROR_INTERNAL,
+				 "ln%u_wp%u_sm%u_dev%u", ln, wp, sm, devId);
+
+		state->threadState[ln]
+		    = (CUDBGCbuThreadState)tte->cbuThreadState;
+	      }
+
+	    state++;
+	    wpStatesWritten++;
+	  }
+
+	return CUDBG_SUCCESS;
+}
+
 DEF_API_CALL(readWarpState)(uint32_t devId, uint32_t sm, uint32_t wp,
 			    CUDBGWarpState *state)
 {
@@ -452,6 +544,7 @@ DEF_API_CALL(readWarpState)(uint32_t devId, uint32_t sm, uint32_t wp,
 	CudbgThreadTableEntry *tte;
 	CudbgSmTableEntry *smte;
 	CUDBGResult rc;
+	size_t wteSize;
 	size_t ctateSize;
 	size_t smteSize;
 
@@ -459,7 +552,7 @@ DEF_API_CALL(readWarpState)(uint32_t devId, uint32_t sm, uint32_t wp,
 
 	VERIFY_ARG(state);
 
-	GET_TABLE_ENTRY(wte, NULL, CUDBG_ERROR_INVALID_WARP,
+	GET_TABLE_ENTRY(wte, &wteSize, CUDBG_ERROR_INVALID_WARP,
 			"wp%u_sm%u_dev%u", wp, sm, devId);
 
 	GET_TABLE_ENTRY(ctate, &ctateSize, CUDBG_ERROR_UNKNOWN,
@@ -507,6 +600,12 @@ DEF_API_CALL(readWarpState)(uint32_t devId, uint32_t sm, uint32_t wp,
 	state->activeLanes = wte->activeLanesMask;
 	state->errorPCValid = wte->errorPCValid;
 
+	if (offsetof (CudbgWarpTableEntry, inSyscallLanesMask)
+	    < wteSize)
+	  {
+	    state->inSyscallLanes = wte->inSyscallLanesMask;
+	  }
+
 	/* Get number of lanes */
 	rc = API_CALL(getNumLanes)(devId, &numLanes);
 	if (rc != CUDBG_SUCCESS)
@@ -1232,34 +1331,6 @@ DEF_API_CALL(readVirtualReturnAddress)(uint32_t dev, uint32_t sm, uint32_t wp,
 	return CUDBG_SUCCESS;
 }
 
-DEF_API_CALL(isDeviceCodeAddress)(uintptr_t addr, bool *isDeviceAddress)
-{
-	Elf64_Shdr *shdr;
-
-	cs_t callStack[] = {
-		PTR2CS(cuCoreIterateELFImages), true,
-		PTR2CS(cuCoreIterateELFSections),
-		PTR2CS(cuCoreIterateSymbolTable),
-		PTR2CS(cuCoreFilterSymbolByAddress), addr, 0, PTR2CS(isDeviceAddress),
-		PTR2CS(cuCoreReadSymbolSection), PTR2CS(&shdr),
-		PTR2CS(NULL),
-	};
-
-	TRACE_FUNC("addr=0x%llx isDeviceAddress=%p", (unsigned long long)addr, isDeviceAddress);
-
-	VERIFY_ARG(isDeviceAddress);
-
-	*isDeviceAddress = false;
-
-	cuCoreExecuteCallStack(curcc, callStack);
-
-	/* If found check if section is allocated */
-	if (*isDeviceAddress && !(shdr->sh_flags & SHF_ALLOC))
-		*isDeviceAddress = false;
-
-	return CUDBG_SUCCESS;
-}
-
 DEF_API_CALL(getAdjustedCodeAddress)(uint32_t devId, uint64_t address,
 				     uint64_t *adjustedAddress,
 				     CUDBGAdjAddrAction adjAction)
@@ -1840,14 +1911,14 @@ DEF_API_CALL(readWarpResources)(uint32_t dev, uint32_t sm, uint32_t wp,
   VERIFY_ARG(resources);
   memset (resources, 0, sizeof (*resources));
   
-  GET_TABLE_ENTRY(wte, NULL, CUDBG_ERROR_INVALID_WARP,
+  GET_TABLE_ENTRY(wte, &wteSize, CUDBG_ERROR_INVALID_WARP,
   		"wp%u_sm%u_dev%u", wp, sm, dev);
   
   if (offsetof (CudbgWarpTableEntry, numRegs) < wteSize)
     resources->numRegisters = wte->numRegs;
   if (offsetof (CudbgWarpTableEntry, sharedMemSize) < wteSize)
     resources->sharedMemSize = wte->sharedMemSize;
-  
+
   return CUDBG_SUCCESS;
 }
 
@@ -1991,7 +2062,7 @@ static const struct CUDBGAPI_st cudbgCoreApi = {
   API_CALL (readGlobalMemory),
   API_CALL (notSupported),
   API_CALL (getManagedMemoryRegionInfo),
-  API_CALL (isDeviceCodeAddress),
+  API_CALL (notSupported),
   API_CALL (notSupported),
 
   /* 6.5 Extensions */
@@ -2049,11 +2120,16 @@ static const struct CUDBGAPI_st cudbgCoreApi = {
   /* 12.7 Extensions */
   API_CALL (getGridInfo),
   API_CALL (getClusterDim),
-  API_CALL (readWarpState),
+  API_CALL (notSupported), /* readWarpState127 */
   API_CALL (getClusterExceptionTargetBlock),
 
   /* 12.8 Extensions */
   API_CALL (readWarpResources),
+
+  /* 12.9 Extensions */
+  API_CALL (getCbuWarpState),
+  API_CALL (readWarpState),
+  API_CALL  (notSupported) /* consumeCudaLogs */
 };
 
 CUDBGAPI cuCoreGetApi(CudaCore *cc)
