@@ -28,7 +28,8 @@
 
 #ifndef __CUDACOREDUMP_H__
 #define __CUDACOREDUMP_H__
-#include "cuda_stdint.h"
+
+#include <stdint.h>
 
 /*
  * cudacoredump.h - Public format description of the CUDA coredump
@@ -51,7 +52,9 @@
  * in the baseline version of the section. Section descriptions below make it
  * clear which fields might not be present.
  *
- * Coredump sections are named hierarchically, the hierarchy looks like this:
+ * Coredump sections are named hierarchically, the hierarchy looks like this
+ * (but not necessarily in this order):
+ *   - Metadata
  *   - Global memory
  *   - CUDA device information
  *     - CUDA context information
@@ -67,6 +70,7 @@
  *         - Warp information
  *           - Warp uniform registers
  *           - Warp uniform predicates
+ *           - Warp convergence barrier masks
  *           - Thread information
  *             - Thread local memory
  *             - Thread registers
@@ -95,6 +99,13 @@
  *   - Added cluster exception target block index
  * CUDA Driver r570:
  *   - Added per-warp shared memory usage
+ * CUDA Driver r575:
+ *   - Added per-warp in syscall lanes mask
+ *   - Added per-warp CBU active lanes mask
+ *   - Added per-warp CBU exited lanes mask
+ *   - Added per-warp CBU collective lanes mask
+ *   - Added per-warp convergence barrier masks
+ *   - Added per-lane thread state
  */
 
 /* ======================================= */
@@ -134,6 +145,8 @@ typedef enum {
     CUDBG_SHT_CB_TABLE    = SHT_LOUSER + 21,
     /* Since CUDA Driver r565 */
     CUDBG_SHT_META_DATA   = SHT_LOUSER + 22,
+    /* Since CUDA Driver r575 */
+    CUDBG_SHT_CBU_BAR     = SHT_LOUSER + 23,
 } CudbgSectionHeaderTypes;
 
 /* CUDA section name prefixes.
@@ -163,6 +176,8 @@ typedef enum {
 #define CUDBG_SHNAME_CBTABLE    ".cudbg.cbankstbl"
 /* Since CUDA Driver r565 */
 #define CUDBG_SHNAME_META_DATA  ".cudbg.meta"
+/* Since CUDA Driver r575 */
+#define CUDBG_SHNAME_CBU_BAR    ".cudbg.cbu_bar"
 
 /* ========================================= */
 /* ===== Detailed section descriptions ===== */
@@ -256,6 +271,15 @@ typedef struct {
     uint32_t numUniformRegsPrWarp;
     /* Number of uniform predicates per warp */
     uint32_t numUniformPredicatesPrWarp;
+
+    /* ================================== */
+    /* ===== Since CUDA Driver r575 ===== */
+    /* ================================== */
+
+    /* Maximum number of convergence barriers per warp */
+    uint32_t numConvergenceBarriersPrWarp;
+    /* Padding, ignore */
+    uint32_t padding0;
 } CudbgDeviceTableEntry;
 
 /*
@@ -624,7 +648,20 @@ typedef struct {
  * Section link: section header index of the corresponding warp table
  * Section info: warpIdx
  *
- * 3. Thread information, per warp - see below
+ * 3. Convergence barrier masks, per warp
+ *
+ * Since CUDA Driver r575.
+ *
+ * Contains warp convergence barrier masks stored as uint32_t values.
+ * Not present if the device doesn't have convergence barriers.
+ *
+ * Section name format: sprintf("%s.dev%d.sm%d.cta%d.wp%d", CUDBG_SHNAME_CBU_BAR, devIdx, smIdx, ctaIdx, warpIdx)
+ * Section header type: CUDBG_SHT_CBU_BAR
+ * Section element type: uint32_t data
+ * Section link: section header index of corresponding warp table
+ * Section info: warpIdx
+ *
+ * 4. Thread information, per warp - see below
  */
 typedef struct {
     /* PC which has triggered a warp error
@@ -665,6 +702,19 @@ typedef struct {
     uint32_t sharedMemSize;
     /* Padding, ignore */
     uint32_t padding2;
+
+    /* ================================== */
+    /* ===== Since CUDA Driver r575 ===== */
+    /* ================================== */
+
+    /* Mask of lanes stopped in a syscall */
+    uint32_t inSyscallLanesMask;
+    /* Mask of lanes marked as active in CBU */
+    uint32_t cbuActiveLanesMask;
+    /* Mask of lanes marked as exited in CBU */
+    uint32_t cbuExitedLanesMask;
+    /* Mask of lanes participating in a collective region */
+    uint32_t cbuCollectiveLanesMask;
 } CudbgWarpTableEntry;
 
 /*
@@ -745,6 +795,17 @@ typedef struct {
     uint32_t syscallCallDepth;
     /* Special CC register, only available on pre-Volta GPUs */
     uint32_t ccRegister;
+
+    /* ================================== */
+    /* ===== Since CUDA Driver r575 ===== */
+    /* ================================== */
+
+    /* CBU thread state
+     * This field is of type CUDBGCBUThreadState, see cudadebugger.h.
+     */
+    uint32_t cbuThreadState;
+    /* Padding, ignore */
+    uint32_t padding0;
 } CudbgThreadTableEntry;
 
 /*
