@@ -32,14 +32,8 @@
 #ifndef CUDADEBUGGER_H
 #define CUDADEBUGGER_H
 
-
-
+#include <stdint.h>
 #include <stdlib.h>
-#include "cuda_stdint.h"
-
-#if defined(__STDC__)
-#include <inttypes.h>
-#endif
 
 #if defined(_MSC_VER) && _MSC_VER < 1800
 // old MSVC does not support stdbool.h
@@ -67,15 +61,17 @@ extern "C" {
 /*--------------------------------- API Version ------------------------------*/
 
 #define CUDBG_API_VERSION_MAJOR      12 /* Major release version number */
-#define CUDBG_API_VERSION_MINOR       8 /* Minor release version number */
-#define CUDBG_API_VERSION_REVISION  152 /* Revision (build) number */
+#define CUDBG_API_VERSION_MINOR       9 /* Minor release version number */
+#define CUDBG_API_VERSION_REVISION  156 /* Revision (build) number */
 
 /*---------------------------------- Constants -------------------------------*/
 
-#define CUDBG_MAX_DEVICES 64  /* Maximum number of supported devices */
-#define CUDBG_MAX_SMS    256  /* Maximum number of SMs per device */
-#define CUDBG_MAX_WARPS   64  /* Maximum number of warps per SM */
-#define CUDBG_MAX_LANES   32  /* Maximum number of lanes per warp */
+#define CUDBG_MAX_DEVICES       64 /* Maximum number of supported devices */
+#define CUDBG_MAX_SMS          256 /* Maximum number of SMs per device */
+#define CUDBG_MAX_WARPS         64 /* Maximum number of warps per SM */
+#define CUDBG_MAX_LANES         32 /* Maximum number of lanes per warp */
+#define CUDBG_MAX_WARP_BARRIERS 16 /* Maximum number of convergence barriers per warp */
+#define CUDBG_MAX_LOG_LEN      256 /* Maximum length of a single CUDA log message */
 
 /*----------------------- Thread/Block Coordinates Types ---------------------*/
 
@@ -919,6 +915,21 @@ typedef struct {
     CuDim3 clusterDim;
     uint32_t clusterExceptionTargetBlockIdxValid;
     CuDim3 clusterExceptionTargetBlockIdx;
+} CUDBGWarpState127;
+
+typedef struct {
+    uint64_t gridId;
+    uint64_t errorPC;
+    CuDim3 blockIdx;
+    uint32_t validLanes;
+    uint32_t activeLanes;
+    uint32_t errorPCValid;
+    CUDBGLaneState lane[32];
+    CuDim3 clusterIdx;
+    CuDim3 clusterDim;
+    uint32_t clusterExceptionTargetBlockIdxValid;
+    CuDim3 clusterExceptionTargetBlockIdx;
+    uint32_t inSyscallLanes;
 } CUDBGWarpState;
 
 typedef struct {
@@ -963,7 +974,7 @@ typedef enum {
        Size: Number of SMs-sized bitmask, rounded up to be divisible by 8 */
     CUDBG_DEVICE_ATTRIBUTE_SM_EXCEPTION_MASK    = 2,
 
-    CUDBG_DEVICE_ATTRIBUTE_COUNT                = 3,
+    CUDBG_DEVICE_ATTRIBUTE_COUNT,
 } CUDBGDeviceInfoAttribute_t;
 
 /* uint32_t sized enum */
@@ -973,7 +984,7 @@ typedef enum {
        Size: uint64_t */
     CUDBG_SM_ATTRIBUTE_WARP_UPDATE_MASK  = 0,
 
-    CUDBG_SM_ATTRIBUTE_COUNT             = 1,
+    CUDBG_SM_ATTRIBUTE_COUNT,
 } CUDBGSMInfoAttribute_t;
 
 /* uint32_t sized enum */
@@ -1007,13 +1018,17 @@ typedef enum {
        Optional: Yes, assume no block index is available if absent
        Size: CuDim3 */
     CUDBG_WARP_ATTRIBUTE_CLUSTER_EXCEPTION_TARGET_BLOCK_IDX = 6,
+    /* Lane mask showing threads that are in a syscall
+       Optional: Yes, use readSyscallCallDepth() if this attribute is not present
+       Size: uint32_t */
+    CUDBG_WARP_ATTRIBUTE_IN_SYSCALL_LANES                   = 7,
 
-    CUDBG_WARP_ATTRIBUTE_COUNT                              = 7,
+    CUDBG_WARP_ATTRIBUTE_COUNT,
 } CUDBGWarpInfoAttribute_t;
 
 /* uint32_t sized enum */
 typedef enum {
-    CUDBG_LANE_ATTRIBUTE_COUNT           = 0,
+    CUDBG_LANE_ATTRIBUTE_COUNT,
 } CUDBGLaneInfoAttribute_t;
 
 /* Sizes of the various structs returned by the batched device update APIs
@@ -1120,6 +1135,65 @@ typedef enum {
                                      | CUDBG_COREDUMP_SKIP_CONSTBANK_MEMORY
 } CUDBGCoredumpGenerationFlags;
 
+/*-------------------------------- CBU state -------------------------------*/
+
+typedef enum {
+     /* Force sizeof(CUDBGCbuThreadState)==4 */
+    CUDBG_CBU_THREAD_STATE_INVALID = 0xFFFFFFFFU,
+    CUDBG_CBU_THREAD_STATE_EXITED  = 0,
+    CUDBG_CBU_THREAD_STATE_READY,
+    CUDBG_CBU_THREAD_STATE_YIELDED,
+    CUDBG_CBU_THREAD_STATE_SLEEP,
+    CUDBG_CBU_THREAD_STATE_SLEEPYIELD,
+    CUDBG_CBU_THREAD_STATE_READYATNEXT,
+    CUDBG_CBU_THREAD_STATE_BLOCKEDPLUS,
+    CUDBG_CBU_THREAD_STATE_BLOCKEDALL,
+    CUDBG_CBU_THREAD_STATE_BLOCKEDCOLLECTIVE,
+    CUDBG_CBU_THREAD_STATE_BLOCKEDB0,
+    CUDBG_CBU_THREAD_STATE_BLOCKEDB1,
+    CUDBG_CBU_THREAD_STATE_BLOCKEDB2,
+    CUDBG_CBU_THREAD_STATE_BLOCKEDB3,
+    CUDBG_CBU_THREAD_STATE_BLOCKEDB4,
+    CUDBG_CBU_THREAD_STATE_BLOCKEDB5,
+    CUDBG_CBU_THREAD_STATE_BLOCKEDB6,
+    CUDBG_CBU_THREAD_STATE_BLOCKEDB7,
+    CUDBG_CBU_THREAD_STATE_BLOCKEDB8,
+    CUDBG_CBU_THREAD_STATE_BLOCKEDB9,
+    CUDBG_CBU_THREAD_STATE_BLOCKEDB10,
+    CUDBG_CBU_THREAD_STATE_BLOCKEDB11,
+    CUDBG_CBU_THREAD_STATE_BLOCKEDB12,
+    CUDBG_CBU_THREAD_STATE_BLOCKEDB13,
+    CUDBG_CBU_THREAD_STATE_BLOCKEDB14,
+    CUDBG_CBU_THREAD_STATE_BLOCKEDB15,
+} CUDBGCbuThreadState;
+ 
+#pragma pack(push,1)
+typedef struct
+{
+    uint32_t activeMask;
+    uint32_t exitedMask;
+    uint32_t collectiveMask;
+    uint32_t barrierMasks[CUDBG_MAX_WARP_BARRIERS];
+    CUDBGCbuThreadState threadState[CUDBG_MAX_LANES];
+} CUDBGCbuWarpState;
+#pragma pack(pop)
+
+typedef enum {
+    CUDBG_CUDA_LOG_LEVEL_INVALID = 0xFFFFFFFFU,
+    CUDBG_CUDA_LOG_LEVEL_ERROR   = 0,
+    CUDBG_CUDA_LOG_LEVEL_WARNING = 1,
+} CUDBGCudaLogLevel;
+
+#pragma pack(push,1)
+typedef struct
+{
+    uint64_t unixTimestampNs;
+    uint32_t osThreadId;
+    CUDBGCudaLogLevel logLevel;
+    char message[CUDBG_MAX_LOG_LEN];
+} CUDBGCudaLogMessage;
+#pragma pack(pop)
+
 /*--------------------------------- Exports --------------------------------*/
 
 typedef const struct CUDBGAPI_st *CUDBGAPI;
@@ -1362,11 +1436,17 @@ struct CUDBGAPI_st {
     /* 12.7 Extensions */
     CUDBGResult (*getGridInfo)(uint32_t dev, uint64_t gridId64, CUDBGGridInfo *gridInfo);
     CUDBGResult (*getClusterDim)(uint32_t dev, uint32_t sm, uint32_t wp, CuDim3 *clusterDim);
-    CUDBGResult (*readWarpState)(uint32_t dev, uint32_t sm, uint32_t wp, CUDBGWarpState *state);
+    CUDBGResult (*readWarpState127)(uint32_t dev, uint32_t sm, uint32_t wp, CUDBGWarpState127 *state);
     CUDBGResult (*getClusterExceptionTargetBlock)(uint32_t dev, uint32_t sm, uint32_t wp, CuDim3 *blockIdx, bool *blockIdxValid);
 
     /* 12.8 Extensions */
     CUDBGResult (*readWarpResources)(uint32_t dev, uint32_t sm, uint32_t wp, CUDBGWarpResources *resources);
+
+    /* 12.9 Extensions */
+    CUDBGResult (*getCbuWarpState)(uint32_t dev, uint32_t sm, uint64_t warpMask, CUDBGCbuWarpState* warpStates, uint32_t numWarpStates);
+    CUDBGResult (*readWarpState)(uint32_t dev, uint32_t sm, uint32_t wp, CUDBGWarpState *state);
+    CUDBGResult (*consumeCudaLogs)(CUDBGCudaLogMessage* logMessages, uint32_t numMessages, uint32_t* numConsumed);
+    CUDBGResult (*readCPUCallStack)(uint32_t dev, uint64_t gridId64, uint64_t *addrs, uint32_t numAddrs, uint32_t* totalNumAddrs);
 };
 
 #ifdef __cplusplus
