@@ -176,27 +176,27 @@ typedef struct CUfileError {
 
 /* driver properties */
 typedef enum CUfileDriverStatusFlags {
-        CU_FILE_LUSTRE_SUPPORTED = 0, /*!< Support for DDN LUSTRE */
+    CU_FILE_LUSTRE_SUPPORTED = 0, /*!< Support for DDN LUSTRE */
 
-        CU_FILE_WEKAFS_SUPPORTED = 1, /*!< Support for WEKAFS */
+    CU_FILE_WEKAFS_SUPPORTED = 1, /*!< Support for WEKAFS */
 
-        CU_FILE_NFS_SUPPORTED = 2, /*!< Support for NFS */
+    CU_FILE_NFS_SUPPORTED = 2, /*!< Support for NFS */
 
-        CU_FILE_GPFS_SUPPORTED = 3, /*! < Support for GPFS */
+    CU_FILE_GPFS_SUPPORTED = 3, /*! < Support for GPFS */
 
-        CU_FILE_NVME_SUPPORTED = 4, /*!< Support for NVMe */
+    CU_FILE_NVME_SUPPORTED = 4, /*!< Support for NVMe */
 
-        CU_FILE_NVMEOF_SUPPORTED = 5, /*!< Support for NVMeOF */
+    CU_FILE_NVMEOF_SUPPORTED = 5, /*!< Support for NVMeOF */
 
-        CU_FILE_SCSI_SUPPORTED = 6, /*!< Support for SCSI */
+    CU_FILE_SCSI_SUPPORTED = 6, /*!< Support for SCSI */
 
-	CU_FILE_SCALEFLUX_CSD_SUPPORTED = 7, /*!< Support for Scaleflux CSD*/
+    CU_FILE_SCALEFLUX_CSD_SUPPORTED = 7, /*!< Support for Scaleflux CSD*/
 
-	CU_FILE_NVMESH_SUPPORTED = 8, /*!< Support for NVMesh Block Dev*/
-	CU_FILE_BEEGFS_SUPPORTED = 9, /*!< Support for BeeGFS */
+    CU_FILE_NVMESH_SUPPORTED = 8, /*!< Support for NVMesh Block Dev*/
+    CU_FILE_BEEGFS_SUPPORTED = 9, /*!< Support for BeeGFS */
     //10 is reserved for YRCloudFile
-        CU_FILE_NVME_P2P_SUPPORTED = 11,   /*!< Support for NVMe using PCI P2PDMA */
-
+    CU_FILE_NVME_P2P_SUPPORTED = 11,   /*!< Support for NVMe using PCI P2PDMA */
+    CU_FILE_SCATEFS_SUPPORTED = 12,      /*!< Support for ScateFS */
 }CUfileDriverStatusFlags_t;
 
 typedef enum CUfileDriverControlFlags {
@@ -730,6 +730,82 @@ CUfileError_t cuFileStreamDeregister(CUstream stream);
 
 CUfileError_t cuFileGetVersion(int *version);
 
+typedef enum CUFileSizeTConfigParameter_t {
+	CUFILE_PARAM_PROFILE_STATS,
+	CUFILE_PARAM_EXECUTION_MAX_IO_QUEUE_DEPTH,
+	CUFILE_PARAM_EXECUTION_MAX_IO_THREADS,
+	CUFILE_PARAM_EXECUTION_MIN_IO_THRESHOLD_SIZE_KB,
+	CUFILE_PARAM_EXECUTION_MAX_REQUEST_PARALLELISM,
+	CUFILE_PARAM_PROPERTIES_MAX_DIRECT_IO_SIZE_KB,
+	CUFILE_PARAM_PROPERTIES_MAX_DEVICE_CACHE_SIZE_KB,
+	CUFILE_PARAM_PROPERTIES_PER_BUFFER_CACHE_SIZE_KB,
+	CUFILE_PARAM_PROPERTIES_MAX_DEVICE_PINNED_MEM_SIZE_KB,
+	CUFILE_PARAM_PROPERTIES_IO_BATCHSIZE,
+	CUFILE_PARAM_POLLTHRESHOLD_SIZE_KB,
+	CUFILE_PARAM_PROPERTIES_BATCH_IO_TIMEOUT_MS,
+} CUFileSizeTConfigParameter_t;
+
+typedef enum CUFileBoolConfigParameter_t {
+	CUFILE_PARAM_PROPERTIES_USE_POLL_MODE,
+	CUFILE_PARAM_PROPERTIES_ALLOW_COMPAT_MODE,
+	CUFILE_PARAM_FORCE_COMPAT_MODE,
+	CUFILE_PARAM_FS_MISC_API_CHECK_AGGRESSIVE,
+	CUFILE_PARAM_EXECUTION_PARALLEL_IO,
+	CUFILE_PARAM_PROFILE_NVTX,
+	CUFILE_PARAM_PROPERTIES_ALLOW_SYSTEM_MEMORY,
+	CUFILE_PARAM_USE_PCIP2PDMA,
+	CUFILE_PARAM_PREFER_IO_URING,
+	CUFILE_PARAM_FORCE_ODIRECT_MODE,
+	CUFILE_PARAM_SKIP_TOPOLOGY_DETECTION,
+	CUFILE_PARAM_STREAM_MEMOPS_BYPASS,
+} CUFileBoolConfigParameter_t;
+
+typedef enum CUFileStringConfigParameter_t {
+	CUFILE_PARAM_LOGGING_LEVEL,
+	CUFILE_PARAM_ENV_LOGFILE_PATH,
+	CUFILE_PARAM_LOG_DIR,
+} CUFileStringConfigParameter_t;
+
+
+// GET Parameter API
+/**
+ *@brief
+
+ * @param param The parameter to get.
+ * @param value The location where the value will be stored.
+ *
+ * @return  CU_FILE_SUCCESS on success
+ * @return  CU_FILE_INVALID_VALUE if the input parameter is invalid
+ *
+ * @note If the driver is open, cuFileGetParameter*() will return the current runtime value for the given parameter. 
+ * @note If the driver is not opened yet, cuFileGetParameter*() will return the currently staged value for that parameter.
+ *       Staged parameter values are cleared when the driver is opened.
+ *
+ */
+CUfileError_t cuFileGetParameterSizeT(CUFileSizeTConfigParameter_t param, size_t *value);	
+CUfileError_t cuFileGetParameterBool(CUFileBoolConfigParameter_t param, bool *value);	
+CUfileError_t cuFileGetParameterString(CUFileStringConfigParameter_t param, char *desc_str, int len);	
+
+// SET Parameter APIs
+/**
+ *@brief
+
+ * @param param The parameter to set.
+ * @param value The source of the parameter value.
+ *
+ * @return  CU_FILE_SUCCESS on success
+ * @return  CU_FILE_INVALID_VALUE if the input parameter is inalid.
+ * @return  CU_FILE_DRIVER_ALREADY_OPEN if the driver is already open.
+ *
+ * @note  Setting values is only permitted when the driver is not open - set parameter values are applied at driver load time.
+ * @note  If the same parameter is set multiple times, only the last parameter is kept and used.
+ * @note  Parameter precedence (highest to losest) is: cuFileGetParameter*() (if set), then environment variable (if exists and set), then cufile.json,
+ *
+ */
+CUfileError_t cuFileSetParameterSizeT(CUFileSizeTConfigParameter_t param, size_t value);
+CUfileError_t cuFileSetParameterBool(CUFileBoolConfigParameter_t param, bool value);
+CUfileError_t cuFileSetParameterString(CUFileStringConfigParameter_t param, const char* desc_str);
+
 #pragma GCC visibility pop
 
 /// @cond DOXYGEN_SKIP_MACRO
